@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -30,22 +30,23 @@
 package org.jvnet.substance;
 
 import java.awt.*;
+import java.awt.image.BufferedImage;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 
 import javax.swing.*;
 import javax.swing.border.Border;
-import javax.swing.border.EmptyBorder;
-import javax.swing.plaf.BorderUIResource;
 import javax.swing.plaf.ComponentUI;
-import javax.swing.plaf.basic.BasicButtonListener;
-import javax.swing.plaf.basic.BasicButtonUI;
+import javax.swing.plaf.basic.*;
+import javax.swing.text.View;
 
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
 import org.jvnet.substance.button.SubstanceButtonShaper;
 import org.jvnet.substance.combo.SubstanceComboBoxButton;
-import org.jvnet.substance.scroll.SubstanceScrollBarButton;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
+import org.jvnet.substance.scroll.SubstanceScrollButton;
+import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.*;
 import org.jvnet.substance.utils.icon.GlowingIcon;
 
@@ -110,13 +111,6 @@ public class SubstanceButtonUI extends BasicButtonUI {
 	 */
 	protected GlowingIcon glowingIcon;
 
-	// private ChangeListener substanceIconGlowListener;
-
-	// /**
-	// * Model change listener for ghost image effects.
-	// */
-	// private GhostingListener substanceModelChangeListener;
-
 	/**
 	 * Property change listener. Listens on changes to the
 	 * {@link SubstanceLookAndFeel#BUTTON_SHAPER_PROPERTY} property and
@@ -129,11 +123,6 @@ public class SubstanceButtonUI extends BasicButtonUI {
 	 */
 	protected FadeStateListener substanceFadeStateListener;
 
-	//
-	// protected long focusLoopFadeInstanceId;
-	//
-	// protected FocusListener substanceFocusListener;
-
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -184,26 +173,9 @@ public class SubstanceButtonUI extends BasicButtonUI {
 			}
 		}
 		b.putClientProperty(SubstanceButtonUI.OPACITY_ORIGINAL, b.isOpaque());
-		if (b instanceof SubstanceComboBoxButton) {
-			b.setBorder(new BorderUIResource.CompoundBorderUIResource(
-					new EmptyBorder(1, 1, 1, 1), b.getBorder()));
-		}
 
-		// Color fg = b.getForeground();
-		// if ((fg == null) || (fg instanceof UIResource)) {
-		// b.setForeground(new ColorUIResource(SubstanceCoreUtilities
-		// .getDefaultScheme(b).getForegroundColor()));
-		// }
-		//
-		// Color bg = b.getBackground();
-		// if (bg instanceof UIResource) {
-		// b.setBackground(new ButtonColorDelegate(b, false));
-		// }
-
-		// Color fg = b.getForeground();
-		// if (fg instanceof UIResource) {
-		// b.setForeground(new ButtonColorDelegate(b));
-		// }
+		LookAndFeel.installProperty(b, "iconTextGap", SubstanceSizeUtils
+				.getTextIconGap(SubstanceSizeUtils.getComponentFontSize(b)));
 	}
 
 	/*
@@ -266,18 +238,14 @@ public class SubstanceButtonUI extends BasicButtonUI {
 						.getPropertyName())) {
 					if (substanceFadeStateListener != null)
 						substanceFadeStateListener.unregisterListeners();
+					boolean toRepaintParent = (b instanceof SubstanceScrollButton)
+							|| (b instanceof SubstanceSpinnerButton)
+							|| (b instanceof SubstanceComboBoxButton);
 					substanceFadeStateListener = new FadeStateListener(b, b
 							.getModel(), SubstanceCoreUtilities
-							.getFadeCallback(b));
+							.getFadeCallback(b, toRepaintParent));
 					substanceFadeStateListener
-							.registerListeners(b instanceof SubstanceScrollBarButton);
-
-					//
-					// if (substanceModelChangeListener != null)
-					// substanceModelChangeListener.unregisterListeners();
-					// substanceModelChangeListener = new GhostingListener(b, b
-					// .getModel());
-					// substanceModelChangeListener.registerListeners();
+							.registerListeners(toRepaintParent);
 				}
 				if (AbstractButton.ICON_CHANGED_PROPERTY.equals(evt
 						.getPropertyName())) {
@@ -287,14 +255,13 @@ public class SubstanceButtonUI extends BasicButtonUI {
 		};
 		b.addPropertyChangeListener(this.substancePropertyListener);
 
+		boolean toRepaintParent = (b instanceof SubstanceScrollButton)
+				|| (b instanceof SubstanceSpinnerButton)
+				|| (b instanceof SubstanceComboBoxButton);
 		this.substanceFadeStateListener = new FadeStateListener(b,
-				b.getModel(), SubstanceCoreUtilities.getFadeCallback(b));
-		this.substanceFadeStateListener
-				.registerListeners(b instanceof SubstanceScrollBarButton);
-		//
-		// this.substanceModelChangeListener = new GhostingListener(b, b
-		// .getModel());
-		// this.substanceModelChangeListener.registerListeners();
+				b.getModel(), SubstanceCoreUtilities.getFadeCallback(b,
+						toRepaintParent));
+		this.substanceFadeStateListener.registerListeners(toRepaintParent);
 	}
 
 	/*
@@ -316,9 +283,6 @@ public class SubstanceButtonUI extends BasicButtonUI {
 
 		this.substanceFadeStateListener.unregisterListeners();
 		this.substanceFadeStateListener = null;
-		//
-		// this.substanceModelChangeListener.unregisterListeners();
-		// this.substanceModelChangeListener = null;
 
 		super.uninstallListeners(b);
 	}
@@ -326,40 +290,29 @@ public class SubstanceButtonUI extends BasicButtonUI {
 	/*
 	 * (non-Javadoc)
 	 * 
-	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
+	 * @see javax.swing.plaf.basic.BasicButtonUI#paint(java.awt.Graphics,
+	 *      javax.swing.JComponent)
 	 */
 	@Override
-	public void update(Graphics g, JComponent c) {
-		AbstractButton button = (AbstractButton) c;
-		// trackGlowingIcon(button);
-		if (button instanceof JButton) {
-			JButton jb = (JButton) button;
+	public void paint(Graphics g, JComponent c) {
+		final AbstractButton b = (AbstractButton) c;
+
+		if (b instanceof JButton) {
+			JButton jb = (JButton) b;
 			if (PulseTracker.isPulsating(jb)) {
 				PulseTracker.update(jb);
 			} else {
-				// System.out.println(System.currentTimeMillis() + ":"
-				// + button.getText());
 			}
 		}
-		// if ("2".equals(button.getText()))
-		// System.out.println(((AlphaComposite) TransitionLayout
-		// .getAlphaComposite(button)).getAlpha()
-		// + ":" + button.getClientProperty(TransitionLayout.ALPHA));
-		this.delegate.updateBackground(g, button);
-		this.paint(g, c);
 
-		// Some ugly hack to allow fade-out of focus ring. The code
-		// in BasicButtonUI doesn't call paintFocus() at all
-		// when the component is not focus owner.
-		AbstractButton b = (AbstractButton) c;
-		FontMetrics fm = c.getFontMetrics(c.getFont());
+		FontMetrics fm = g.getFontMetrics();
 
 		Insets i = c.getInsets();
 
-		// Dimension size = new Dimension();
 		Rectangle viewRect = new Rectangle();
 		Rectangle iconRect = new Rectangle();
-		Rectangle textRect = new Rectangle();
+		final Rectangle textRect = new Rectangle();
+
 		viewRect.x = i.left;
 		viewRect.y = i.top;
 		viewRect.width = b.getWidth() - (i.right + viewRect.x);
@@ -369,22 +322,99 @@ public class SubstanceButtonUI extends BasicButtonUI {
 		iconRect.x = iconRect.y = iconRect.width = iconRect.height = 0;
 
 		Font f = c.getFont();
-		g.setFont(f);
 
 		// layout the text and icon
-		@SuppressWarnings("unused")
 		String text = SwingUtilities.layoutCompoundLabel(c, fm, b.getText(), b
 				.getIcon(), b.getVerticalAlignment(), b
 				.getHorizontalAlignment(), b.getVerticalTextPosition(), b
 				.getHorizontalTextPosition(), viewRect, iconRect, textRect, b
 				.getText() == null ? 0 : b.getIconTextGap());
 
-		// b.putClientProperty("icon.bounds", iconRect);
+		// Special case for scroll buttons - these are painted
+		// on top of the "real" controls (scroll bars). In order
+		// to properly handle the translucent states under the native
+		// text painter, we need to paint these buttons to offscreen
+		// image
+		Graphics2D g2d = null;
+		BufferedImage offscreen = null;
 
-		if (!(b.hasFocus() && b.isFocusPainted())) {
-			if (FadeTracker.getInstance().isTracked(c, FadeKind.FOCUS))
-				this.paintFocus(g, b, viewRect, textRect, iconRect);
+		boolean isPartOfCompositeControl = SubstanceCoreUtilities
+				.isScrollButton(b)
+				|| SubstanceCoreUtilities.isSpinnerButton(b);
+		if (isPartOfCompositeControl) {
+			offscreen = SubstanceCoreUtilities.getBlankImage(b.getWidth(), b
+					.getHeight());
+			g2d = offscreen.createGraphics();
+			SubstanceTheme buttonTheme = SubstanceThemeUtilities.getTheme(b);
+			float themeAlpha = buttonTheme.getThemeAlpha(b, ComponentState
+					.getState(b.getModel(), b));
+
+			g2d.setComposite(AlphaComposite.getInstance(
+					AlphaComposite.SRC_OVER, themeAlpha));
+		} else {
+			g2d = (Graphics2D) g.create();
 		}
+
+		boolean hasEmptyText = (text == null) || (text.length() == 0);
+
+		final View v = (View) c.getClientProperty(BasicHTML.propertyKey);
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		textPainter.init(b, null, v != null);
+		g2d.setFont(f);
+
+		if (!hasEmptyText && textPainter.needsBackgroundImage()) {
+			textPainter.setBackgroundFill(b, b.getParent().getBackground(),
+					true, 0, 0);
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							delegate.updateBackground(g, b);
+						};
+					});
+		} else {
+			this.delegate.updateBackground(g2d, b);
+		}
+		float textAlpha = 1.0f;
+		if (v != null) {
+			if (textPainter.needsBackgroundImage()) {
+				textPainter
+						.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+							public void paintBackground(Graphics g) {
+								v.paint(g, textRect);
+							}
+						});
+			} else {
+				v.paint(g2d, textRect);
+			}
+		} else {
+			textAlpha = this.paintButtonText(g, b, textRect, text);
+		}
+		g2d.setComposite(TransitionLayout.getAlphaComposite(b, textAlpha, g));
+		textPainter.renderSurface(g2d);
+		g2d.setComposite(TransitionLayout.getAlphaComposite(b, 1.0f, g));
+
+		// Paint the Icon
+		if (b.getIcon() != null) {
+			paintIcon(g2d, c, iconRect);
+		}
+
+		if (b.isFocusPainted()) {
+			if (b.hasFocus()
+					|| FadeTracker.getInstance().isTracked(c, FadeKind.FOCUS)) {
+				this.paintFocus(g2d, b, viewRect, textRect, iconRect);
+			}
+		}
+
+//		g2d.setColor(Color.red);
+//		g2d.draw(iconRect);
+//		g2d.draw(viewRect);
+//		g2d.draw(textRect);
+
+		if (isPartOfCompositeControl) {
+			g.drawImage(offscreen, 0, 0, null);
+		}
+
 	}
 
 	/*
@@ -420,39 +450,8 @@ public class SubstanceButtonUI extends BasicButtonUI {
 			return;
 
 		SubstanceCoreUtilities.paintFocus(g, b, b, null, textRect, 1.0f,
-				2 + SubstanceSizeUtils.getExtraPadding(b.getFont().getSize()));
-		//
-		// FadeTracker fadeTracker = FadeTracker.getInstance();
-		// FocusKind focusKind = SubstanceCoreUtilities.getFocusKind(b);
-		// if ((focusKind == FocusKind.NONE)
-		// && (!fadeTracker.isTracked(b, FadeKind.FOCUS)))
-		// return;
-		//
-		// Graphics2D graphics = (Graphics2D) g.create();
-		// graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
-		// RenderingHints.VALUE_ANTIALIAS_ON);
-		//
-		// float alpha = 1.0f;
-		// if (fadeTracker.isTracked(b, FadeKind.FOCUS)) {
-		// alpha = fadeTracker.getFade10(b, FadeKind.FOCUS) / 10.f;
-		// }
-		// graphics.setComposite(TransitionLayout.getAlphaComposite(b, alpha));
-		//
-		// Color color = SubstanceColorUtilities.getFocusColor(b);
-		// graphics.setColor(color);
-		// focusKind.paintFocus(b, graphics, textRect);
-		// graphics.dispose();
-	}
-
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see javax.swing.plaf.basic.BasicButtonUI#paintButtonPressed(java.awt.Graphics,
-	 *      javax.swing.AbstractButton)
-	 */
-	@Override
-	protected void paintButtonPressed(Graphics g, AbstractButton b) {
-		// overriden to remove default metal effect
+				SubstanceSizeUtils.getFocusRingPadding(SubstanceSizeUtils
+						.getComponentFontSize(b)));
 	}
 
 	/*
@@ -475,19 +474,22 @@ public class SubstanceButtonUI extends BasicButtonUI {
 	@Override
 	protected void paintIcon(Graphics g, JComponent c, Rectangle iconRect) {
 		Graphics2D graphics = (Graphics2D) g.create();
+
+		// graphics.setColor(Color.red);
+		// graphics.fill(iconRect);
+
 		FadeTracker fadeTracker = FadeTracker.getInstance();
 		AbstractButton b = (AbstractButton) c;
-		Icon icon = SubstanceCoreUtilities.getIcon(b, this.glowingIcon, false);
+		Icon icon = SubstanceCoreUtilities.getIcon(b, null, this.glowingIcon,
+				false);
 
 		graphics.setComposite(TransitionLayout.getAlphaComposite(b, g));
-		// super.paintIcon(graphics, c, iconRect);
 		if (fadeTracker.isTracked(b, FadeKind.ROLLOVER)) {
 			ComponentState state = ComponentState.getState(b.getModel(), b);
 			// System.out.println(state.name() + ":" + state.isRollover());
-			if (state.isKindActive(FadeKind.ROLLOVER)) {// ==
-				// ComponentState.ROLLOVER_UNSELECTED) {
+			if (state.isKindActive(FadeKind.ROLLOVER)) {
 				// Came from default state
-				SubstanceCoreUtilities.getIcon(b, this.glowingIcon, true)
+				SubstanceCoreUtilities.getIcon(b, null, this.glowingIcon, true)
 						.paintIcon(b, graphics, iconRect.x, iconRect.y);
 				graphics
 						.setComposite(TransitionLayout
@@ -510,72 +512,23 @@ public class SubstanceButtonUI extends BasicButtonUI {
 		graphics.dispose();
 	}
 
-	// @Override
-	// protected void paintText(Graphics g, AbstractButton b, Rectangle
-	// textRect,
-	// String text) {
-	// // blur the text shadow
-	// int width = b.getWidth();
-	// int height = b.getHeight();
-	// BufferedImage blurred = SubstanceCoreUtilities.getBlankImage(width,
-	// height);
-	// Graphics2D gBlurred = (Graphics2D) blurred.getGraphics();
-	// gBlurred.setFont(g.getFont());
-	// gBlurred.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
-	// RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
-	// Color negative = SubstanceColorUtilities.getNegativeColor(b
-	// .getForeground());
-	// int nr = negative.getRed();
-	// int ng = negative.getGreen();
-	// int nb = negative.getBlue();
-	// gBlurred.setColor(negative);
-	// float[] kernel = new float[36];
-	// Arrays.fill(kernel, 0.1f);
-	// ConvolveOp convolve = new ConvolveOp(new Kernel(6, 6, kernel),
-	// ConvolveOp.EDGE_NO_OP, null);
-	//
-	// BasicGraphicsUtils.drawStringUnderlineCharAt(gBlurred, text, b
-	// .getDisplayedMnemonicIndex(),
-	// textRect.x + getTextShiftOffset(), textRect.y
-	// + b.getFontMetrics(g.getFont()).getAscent()
-	// + getTextShiftOffset());
-	//
-	// // super.paintText(gBlurred, b, textRect, text);
-	// blurred = convolve.filter(blurred, null);
-	// int[] dstBuffer = ((DataBufferInt) blurred.getRaster().getDataBuffer())
-	// .getData();
-	// int count = dstBuffer.length;
-	// for (int i = 0; i < count; i++) {
-	// int val = dstBuffer[i];
-	// int alpha = (val >>> 24) & 0xFF;
-	// if (alpha > 0) {
-	// // alpha = 255 - (255-alpha)/2;
-	// dstBuffer[i] = (alpha << 24) | (nr << 16) | (ng << 8) | nb;
-	// }
-	// // else
-	// // dstBuffer[i] = (255<<24) | (255<<16);
-	// }
-	//
-	// Graphics2D graphics = (Graphics2D) g.create();
-	// // graphics.setComposite(AlphaComposite.getInstance(
-	// // AlphaComposite.SRC_OUT, 0.5f));
-	// graphics.drawImage(blurred, 0, 0, null);
-	// // graphics.fillRect(5, 5, 5, 5);
-	// graphics.dispose();
-	// super.paintText(g, b, textRect, text);
-	// }
-
-	/*
-	 * (non-Javadoc)
+	/**
+	 * Paints the text.
 	 * 
-	 * @see javax.swing.plaf.basic.BasicButtonUI#paintText(java.awt.Graphics,
-	 *      javax.swing.JComponent, java.awt.Rectangle, java.lang.String)
+	 * @param g
+	 *            Graphic context
+	 * @param button
+	 *            Button
+	 * @param textRect
+	 *            Text rectangle
+	 * @param text
+	 *            Text to paint
+	 * @return Text alpha channel.
 	 */
-	@Override
-	protected void paintText(Graphics g, JComponent c, Rectangle textRect,
-			String text) {
-		SubstanceCoreUtilities.paintText(g, (AbstractButton) c, textRect, text,
-				this.getTextShiftOffset(), this.getPropertyPrefix());
+	protected float paintButtonText(Graphics g, AbstractButton button,
+			Rectangle textRect, String text) {
+		return SubstanceCoreUtilities.paintText(button, textRect, text, button
+				.getDisplayedMnemonicIndex());
 	}
 
 	/**
@@ -592,4 +545,15 @@ public class SubstanceButtonUI extends BasicButtonUI {
 			return;
 		this.glowingIcon = new GlowingIcon(currIcon, b);
 	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.ComponentUI#update(java.awt.Graphics,
+	 *      javax.swing.JComponent)
+	 */
+	@Override
+	public void update(Graphics g, JComponent c) {
+		this.paint(g, c);
+	}
 }
