@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -43,9 +43,14 @@ import javax.swing.plaf.basic.ComboPopup;
 
 import org.jvnet.lafwidget.animation.FadeStateListener;
 import org.jvnet.lafwidget.layout.TransitionLayout;
+import org.jvnet.substance.button.BaseButtonShaper;
+import org.jvnet.substance.color.ColorScheme;
 import org.jvnet.substance.combo.*;
+import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.*;
+import org.jvnet.substance.utils.SubstanceConstants.Side;
 
 /**
  * UI for combo boxes in <b>Substance </b> look and feel.
@@ -62,11 +67,6 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 	 */
 	protected ComboBoxPropertyChangeHandler substanceChangeHandler;
 
-	/**
-	 * Painting delegate (for background).
-	 */
-	private static SubstanceGradientBackgroundDelegate backgroundDelegate = new SubstanceGradientBackgroundDelegate();
-
 	/**
 	 * Listener for fade animations.
 	 */
@@ -77,6 +77,8 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 	 */
 	protected FocusListener substanceFocusListener;
 
+	private static SubstanceFillBackgroundDelegate backgroundDelegate = new SubstanceFillBackgroundDelegate();
+
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -85,6 +87,7 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 	public static ComponentUI createUI(JComponent b) {
 		SubstanceComboBoxUI ui = new SubstanceComboBoxUI();
 		ui.comboBox = (JComboBox) b;
+		ui.comboBox.setOpaque(false);
 		return ui;
 	}
 
@@ -123,7 +126,6 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 	 */
 	@Override
 	protected ListCellRenderer createRenderer() {
-		// return new SubstanceDefaultListCellRenderer.SubstanceUIResource();
 		return new SubstanceDefaultComboBoxRenderer.SubstanceUIResource(
 				this.comboBox);
 	}
@@ -138,8 +140,6 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 		super.installListeners();
 		this.substanceChangeHandler = new ComboBoxPropertyChangeHandler();
 		this.comboBox.addPropertyChangeListener(this.substanceChangeHandler);
-		// this.substanceFocusListener = new FocusBorderListener(this.comboBox);
-		// this.comboBox.addFocusListener(this.substanceFocusListener);
 
 		this.substanceFadeStateListener = new FadeStateListener(this.comboBox,
 				null, null);
@@ -210,20 +210,20 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 			int height = cb.getHeight();
 
 			Insets insets = SubstanceComboBoxUI.this.getInsets();
-			int buttonWidth = SubstanceSizeUtils.getScrollBarWidth(comboBox
-					.getFont().getSize());
+			int buttonWidth = SubstanceSizeUtils
+					.getScrollBarWidth(SubstanceSizeUtils
+							.getComponentFontSize(comboBox));
 			buttonWidth = Math.max(buttonWidth,
 					arrowButton.getPreferredSize().width);
-			int buttonHeight = height - (insets.top + insets.bottom);
 
 			if (SubstanceComboBoxUI.this.arrowButton != null) {
 				if (cb.getComponentOrientation().isLeftToRight()) {
 					SubstanceComboBoxUI.this.arrowButton.setBounds(width
-							- (insets.right + buttonWidth), insets.top,
-							buttonWidth, buttonHeight);
+							- buttonWidth - insets.right, 0, buttonWidth
+							+ insets.right, height);
 				} else {
-					SubstanceComboBoxUI.this.arrowButton.setBounds(insets.left,
-							insets.top, buttonWidth, buttonHeight);
+					SubstanceComboBoxUI.this.arrowButton.setBounds(0, 0,
+							buttonWidth + insets.left, height);
 				}
 			}
 			if (SubstanceComboBoxUI.this.editor != null) {
@@ -259,27 +259,27 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 			return new Dimension(this.cachedMinimumSize);
 		}
 
-		Dimension size = null;
+		// Dimension size = null;
+		//
+		// if (!this.comboBox.isEditable() && this.arrowButton != null
+		// && this.arrowButton instanceof SubstanceComboBoxButton) {
+		//
+		SubstanceComboBoxButton button = (SubstanceComboBoxButton) this.arrowButton;
+		Insets buttonInsets = button.getInsets();
+		Insets insets = this.comboBox.getInsets();
 
-		if (!this.comboBox.isEditable() && this.arrowButton != null
-				&& this.arrowButton instanceof SubstanceComboBoxButton) {
+		Dimension size = this.getDisplaySize();
 
-			SubstanceComboBoxButton button = (SubstanceComboBoxButton) this.arrowButton;
-			Insets buttonInsets = button.getInsets();
-			Insets insets = this.comboBox.getInsets();
-
-			size = this.getDisplaySize();
-
-			size.width += insets.left + insets.right;
-			size.width += buttonInsets.left + buttonInsets.right;
-			size.width += button.getMinimumSize().getWidth();
-			size.height += insets.top + insets.bottom;
-		} else if (this.comboBox.isEditable() && this.arrowButton != null
-				&& this.editor != null) {
-			size = super.getMinimumSize(c);
-		} else {
-			size = super.getMinimumSize(c);
-		}
+		size.width += insets.left + insets.right;
+		size.width += buttonInsets.left + buttonInsets.right;
+		size.width += button.getMinimumSize().getWidth();
+		size.height += insets.top + insets.bottom;
+		// } else if (this.comboBox.isEditable() && this.arrowButton != null
+		// && this.editor != null) {
+		// size = super.getMinimumSize(c);
+		// } else {
+		// size = super.getMinimumSize(c);
+		// }
 
 		this.cachedMinimumSize.setSize(size.width, size.height);
 		this.isMinimumSizeDirty = false;
@@ -330,19 +330,23 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 						}
 						if (SubstanceComboBoxUI.this.comboBox != null)
 							SubstanceComboBoxUI.this.comboBox.repaint();
+
+						configureArrowButtonStraightSide();
 					}
 				});
 			}
 
 			if (SubstanceLookAndFeel.COMBO_BOX_POPUP_FLYOUT_ORIENTATION
 					.equals(propertyName)) {
-				SubstanceComboBoxUI.this.arrowButton
-						.setIcon(SubstanceCoreUtilities
-								.getArrowIcon(comboBox,
-										arrowButton,
-										// theme,
-										SubstanceCoreUtilities
-												.getPopupFlyoutOrientation(SubstanceComboBoxUI.this.comboBox)));
+				((SubstanceComboBoxButton) arrowButton)
+						.setIcon(
+								comboBox,
+								SubstanceCoreUtilities
+										.getArrowIcon(comboBox,
+												arrowButton,
+												// theme,
+												SubstanceCoreUtilities
+														.getPopupFlyoutOrientation(SubstanceComboBoxUI.this.comboBox)));
 
 			}
 
@@ -353,6 +357,14 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 					}
 				});
 			}
+
+			if ("background".equals(propertyName)) {
+				if (comboBox.isEditable()) {
+					comboBox.getEditor().getEditorComponent().setBackground(
+							comboBox.getBackground());
+					popup.getList().setBackground(comboBox.getBackground());
+				}
+			}
 			// Do not call super - fix for bug 63
 		}
 	}
@@ -417,35 +429,88 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 						.isControlAlwaysPaintedActive(this.comboBox);
 		if (isSelected && isEnabled) {
 			state = ComponentState.SELECTED;
-			// prevState = ComponentState.SELECTED;
 		}
 		if (!isEnabled) {
 			state = ComponentState.DISABLED_UNSELECTED;
-			// prevState = ComponentState.DISABLED_UNSELECTED;
 		}
 
-		// override to prevent the behaviour from the base class, which
-		// ignores the painting of the default cell renderer.
 		Graphics2D graphics = (Graphics2D) g.create();
-		// ComponentState state = this.comboBox.isEnabled() ?
-		// ComponentState.DEFAULT
-		// : ComponentState.DISABLED_UNSELECTED;
-		float alpha = SubstanceCoreUtilities.getTheme(this.comboBox, true)
-				.getThemeAlpha(this.comboBox, state);
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(this.comboBox,
+				state);
+		final ColorScheme colorScheme = theme.getColorScheme();
+		float alpha = theme.getThemeAlpha(this.comboBox, state);
 		if (!SubstanceCoreUtilities.isControlAlwaysPaintedActive(this.comboBox)) {
-			graphics.setComposite(TransitionLayout.getAlphaComposite(
-					this.comboBox, 0.8f * alpha, g));
-		} else {
-			graphics.setComposite(TransitionLayout.getAlphaComposite(
-					this.comboBox, alpha, g));
+			alpha = 0.8f * alpha;
 		}
-		SubstanceTheme theme = SubstanceCoreUtilities.getTheme(this.popup
-				.getList(), state, true, true);
 
-		Rectangle newBounds = new Rectangle(1, 1, this.comboBox.getWidth() - 2,
-				this.comboBox.getHeight() - 2);
-		SubstanceComboBoxUI.backgroundDelegate.update(graphics, this.comboBox,
-				newBounds, theme, 0.0f);
+		ListCellRenderer renderer = this.comboBox.getRenderer();
+		boolean toRegardAsFocused = hasFocus
+				|| SubstanceCoreUtilities
+						.isControlAlwaysPaintedActive(this.comboBox);
+		Component c = renderer.getListCellRendererComponent(this.listBox,
+				this.comboBox.getSelectedItem(), -1, toRegardAsFocused,
+				hasFocus);
+
+		Color background = c.getBackground();
+		if (!(background instanceof UIResource)) {
+			// support for enhancement 256 - colorizing components
+			double colorization = SubstanceCoreUtilities
+					.getColorizationFactor(this.comboBox);
+			if (!this.comboBox.isEnabled())
+				colorization /= 2.0;
+			if (colorization > 0.0) {
+				// the larger the colorization is, the lighter should the
+				// overlay be
+				alpha *= (1.0 - colorization);
+			}
+		}
+
+		final float finalAlpha = alpha;
+
+		// final SubstanceTheme theme =
+		// SubstanceCoreUtilities.getTheme(this.popup
+		// .getList(), state, true, true, true);
+
+		final Rectangle newBounds = new Rectangle(1, 1, this.comboBox
+				.getWidth() - 2, this.comboBox.getHeight() - 2);
+
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		textPainter.init(this.comboBox, null, true);
+		if (textPainter.needsBackgroundImage()) {
+			textPainter.setBackgroundFill(this.comboBox, this.comboBox
+					.getBackground(), false, 0, 0);
+			// final ComponentState finalState = state;
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							Graphics2D g2d = (Graphics2D) g.create();
+							g2d
+									.setComposite(TransitionLayout
+											.getAlphaComposite(comboBox,
+													finalAlpha, g));
+							SubstanceHighlightUtils.paintHighlight(g2d,
+									comboBox, new Rectangle(0, 0, comboBox
+											.getWidth(), comboBox.getHeight()),
+									0.0f, null, colorScheme, colorScheme, 0.0f);
+							g2d.dispose();
+						}
+					});
+			// The text GC will be clipped to the renderer bound. To
+			// have the background painted across the entire area, enforce
+			// this by painting space string now that the clip is not set
+			textPainter.attachText(this.comboBox, newBounds, " ", -1, graphics
+					.getFont(), graphics.getColor(), graphics.getClipBounds());
+			textPainter.renderSurface(graphics);
+		} else {
+			backgroundDelegate.fillAndWatermark(graphics, this.comboBox,
+					background, newBounds);
+			graphics.setComposite(TransitionLayout.getAlphaComposite(
+					this.comboBox, finalAlpha, g));
+
+			SubstanceHighlightUtils.paintHighlight(graphics, this.comboBox,
+					newBounds, 0.0f, null, colorScheme, colorScheme, 0.0f);
+		}
 		graphics.dispose();
 	}
 
@@ -460,41 +525,59 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 		Graphics2D graphics = (Graphics2D) g.create();
 		ComponentState state = this.comboBox.isEnabled() ? ComponentState.DEFAULT
 				: ComponentState.DISABLED_UNSELECTED;
-		float alpha = SubstanceCoreUtilities.getTheme(this.comboBox, true)
+		float alpha = SubstanceThemeUtilities.getTheme(this.comboBox)
 				.getThemeAlpha(this.comboBox, state);
 		graphics.setComposite(TransitionLayout.getAlphaComposite(this.comboBox,
 				alpha, g));
 
 		ListCellRenderer renderer = this.comboBox.getRenderer();
-		if (renderer instanceof SubstanceDefaultListCellRenderer) {
-			Component c;
-			if (hasFocus
-					|| SubstanceCoreUtilities
-							.isControlAlwaysPaintedActive(this.comboBox)) {
-				c = renderer.getListCellRendererComponent(this.listBox,
-						this.comboBox.getSelectedItem(), -1, true, hasFocus);
-			} else {
-				c = renderer.getListCellRendererComponent(this.listBox,
-						this.comboBox.getSelectedItem(), -1, false, hasFocus);
-			}
-			c.setFont(this.comboBox.getFont());
-
-			// Fix for 4238829: should lay out the JPanel.
-			boolean shouldValidate = false;
-			if (c instanceof JPanel) {
-				shouldValidate = true;
-			}
-
-			SubstanceCoreUtilities.workaroundBug6576507(graphics);
-
-			this.currentValuePane.paintComponent(graphics, c, this.comboBox,
-					bounds.x, bounds.y, bounds.width, bounds.height,
-					shouldValidate);
+		// if (renderer instanceof SubstanceDefaultListCellRenderer) {
+		Component c;
+		if (hasFocus
+				|| SubstanceCoreUtilities
+						.isControlAlwaysPaintedActive(this.comboBox)) {
+			c = renderer.getListCellRendererComponent(this.listBox,
+					this.comboBox.getSelectedItem(), -1, true, hasFocus);
 		} else {
-			super.paintCurrentValue(g, bounds, hasFocus);
+			c = renderer.getListCellRendererComponent(this.listBox,
+					this.comboBox.getSelectedItem(), -1, false, hasFocus);
+		}
+		c.setFont(this.comboBox.getFont());
+
+		// Fix for 4238829: should lay out the JPanel.
+		boolean shouldValidate = false;
+		if (c instanceof JPanel) {
+			shouldValidate = true;
 		}
 
-		// super.paintCurrentValue(graphics, bounds, hasFocus);
+		// SubstanceCoreUtilities.workaroundBug6576507(graphics);
+
+		if (c instanceof JComponent) {
+			((JComponent) comboBox)
+					.putClientProperty(
+							SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND,
+							Boolean.TRUE);
+		}
+		// Insets ins = this.comboBox.getInsets();
+
+		if (this.comboBox.getComponentOrientation().isLeftToRight()) {
+			this.currentValuePane.paintComponent(graphics, c, this.comboBox, 1,
+					1, this.arrowButton.getX() - 1,
+					this.comboBox.getHeight() - 2, shouldValidate);
+		} else {
+			int startX = this.arrowButton.getX() + this.arrowButton.getWidth();
+			this.currentValuePane.paintComponent(graphics, c, this.comboBox,
+					startX, 1, this.comboBox.getWidth() - startX - 1,
+					this.comboBox.getHeight() - 2, shouldValidate);
+		}
+		if (c instanceof JComponent) {
+			((JComponent) comboBox).putClientProperty(
+					SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, null);
+		}
+		// } else {
+		// super.paintCurrentValue(g, bounds, hasFocus);
+		// }
+
 		graphics.dispose();
 	}
 
@@ -512,6 +595,7 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 			Rectangle r = this.rectangleForCurrentValue();
 			r.y += 1;
 			r.height -= 2;
+			r.width -= (this.arrowButton.getWidth() + 10);
 			this.paintFocus(g, r);
 		}
 	}
@@ -525,9 +609,14 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 	 *            Bounds for text.
 	 */
 	protected void paintFocus(Graphics g, Rectangle bounds) {
+		int fontSize = SubstanceSizeUtils.getComponentFontSize(this.comboBox);
+		int focusRingPadding = SubstanceSizeUtils.getFocusRingPadding(fontSize);
 		SubstanceCoreUtilities.paintFocus(g, this.comboBox, this.comboBox,
-				null, bounds, 0.4f, 2 + SubstanceSizeUtils
-						.getExtraPadding(this.comboBox.getFont().getSize()));
+				BaseButtonShaper.getBaseOutline(this.arrowButton.getX() - 1,
+						this.comboBox.getHeight() - 3 * focusRingPadding / 2,
+						SubstanceSizeUtils
+								.getClassicButtonCornerRadius(fontSize), null,
+						0), bounds, 0.4f, 3 * focusRingPadding / 2);
 	}
 
 	/**
@@ -547,15 +636,14 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 	 */
 	@Override
 	public void update(Graphics g, JComponent c) {
-		if (SubstanceCoreUtilities.hasSubstanceBorder(c)) {
-			if (c.isOpaque()) {
-				g.setColor(c.getBackground());
-				g.fillRect(1, 1, c.getWidth() - 2, c.getHeight() - 2);
-			}
-			this.paint(g, c);
+		if (!this.comboBox.isEditable()) {
+			SubstanceCoreUtilities.paintTextCompBackground(g, c);
 		} else {
-			super.update(g, c);
+			// SubstanceCoreUtilities.paintTextCompBackground(g, c, Color.red);
+			SubstanceCoreUtilities.paintTextCompBackground(g, c, this.editor
+					.getBackground(), false);
 		}
+		this.paint(g, c);
 	}
 
 	/*
@@ -581,6 +669,18 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 		};
 		this.arrowButton.setSelected(this.comboBox.hasFocus());
 		this.comboBox.addFocusListener(this.substanceFocusListener);
+
+		this.configureArrowButtonStraightSide();
+	}
+
+	/**
+	 * Configures the straight side of the arrow button.
+	 */
+	protected void configureArrowButtonStraightSide() {
+		this.arrowButton.putClientProperty(
+				SubstanceLookAndFeel.BUTTON_SIDE_PROPERTY, this.comboBox
+						.getComponentOrientation().isLeftToRight() ? Side.LEFT
+						: Side.RIGHT);
 	}
 
 	/*
@@ -607,7 +707,8 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 		// the text field in the combo editor.
 		if (this.editor instanceof JComponent) {
 			Insets ins = SubstanceSizeUtils
-					.getComboTextBorderInsets(this.editor.getFont().getSize());
+					.getComboTextBorderInsets(SubstanceSizeUtils
+							.getComponentFontSize(this.editor));
 			((JComponent) this.editor).setBorder(new EmptyBorder(ins.top,
 					ins.left, ins.bottom, ins.right));
 			this.editor.setBackground(this.comboBox.getBackground());
