@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -44,6 +44,8 @@ import javax.swing.plaf.basic.BasicListUI;
 import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
+import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.*;
 
@@ -80,9 +82,9 @@ public class SubstanceListUI extends BasicListUI {
 	protected boolean isLeftToRight;
 
 	/**
-	 * Delegate for painting the background.
+	 * Delegate for painting the background of list rows.
 	 */
-	private static SubstanceGradientBackgroundDelegate backgroundDelegate = new SubstanceGradientBackgroundDelegate();
+	private static SubstanceFillBackgroundDelegate backgroundDelegate = new SubstanceFillBackgroundDelegate();
 
 	/**
 	 * Listener for fade animations on list selections.
@@ -170,6 +172,13 @@ public class SubstanceListUI extends BasicListUI {
 			}
 
 			int roIndex = list.locationToIndex(e.getPoint());
+			if ((roIndex >= 0) && (roIndex < list.getModel().getSize())) {
+				// test actual hit
+				if (!list.getCellBounds(roIndex, roIndex)
+						.contains(e.getPoint())) {
+					roIndex = -1;
+				}
+			}
 			if ((roIndex < 0) || (roIndex >= list.getModel().getSize())) {
 				fadeOut();
 				// System.out.println("Nulling RO index");
@@ -515,10 +524,10 @@ public class SubstanceListUI extends BasicListUI {
 		boolean cellHasFocus = list.hasFocus() && (row == leadIndex);
 		boolean isSelected = selModel.isSelectedIndex(row);
 
-		boolean isWatermarkBleed = SubstanceCoreUtilities
+		final boolean isWatermarkBleed = SubstanceCoreUtilities
 				.toBleedWatermark(list);
 
-		Component rendererComponent = cellRenderer
+		final Component rendererComponent = cellRenderer
 				.getListCellRendererComponent(list, value, row, isSelected,
 						cellHasFocus);
 
@@ -555,30 +564,58 @@ public class SubstanceListUI extends BasicListUI {
 
 		Graphics2D g2d = (Graphics2D) g.create();
 		g2d.setComposite(TransitionLayout.getAlphaComposite(list, g));
-		if (!isWatermarkBleed) {
-			// fill with the renderer background color
-			g2d.setColor(rendererComponent.getBackground());
-			g2d.fillRect(cx, cy, cw, ch);
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		if (textPainter.needsBackgroundImage()) {
+			final Rectangle offsetCellRect = new Rectangle(0, 0, cw, ch);
+			textPainter.init(this.list, offsetCellRect, true);
+			// textPainter.setBackgroundImage(this.list, null, false, 0, 0);
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							Graphics2D g2d = (Graphics2D) g.create();
+							if (!isWatermarkBleed) {
+								// fill with the renderer background color
+								g2d.setColor(rendererComponent.getBackground());
+								g2d.fillRect(0, 0, offsetCellRect.width,
+										offsetCellRect.height);
+							} else {
+								backgroundDelegate.fillAndWatermark(g2d, list,
+										rendererComponent.getBackground(),
+										offsetCellRect);
+							}
+							g2d.dispose();
+						}
+					});
+		} else {
+			if (!isWatermarkBleed) {
+				// fill with the renderer background color
+				g2d.setColor(rendererComponent.getBackground());
+				g2d.fillRect(cx, cy, cw, ch);
+			} else {
+				backgroundDelegate.fillAndWatermark(g2d, this.list,
+						rendererComponent.getBackground(), new Rectangle(cx,
+								cy, cw, ch));
+			}
 		}
 
 		ComponentState prevState = getPrevCellState(row);
 		ComponentState currState = getCellState(row);
-		float alphaForPrevBackground = 0.0f;
 
 		// Compute the alpha values for the animation.
-		float startAlpha = SubstanceCoreUtilities.getHighlightAlpha(list,
-				prevState, true);
-		float endAlpha = SubstanceCoreUtilities.getHighlightAlpha(list,
-				currState, true);
-		float alphaForCurrBackground = endAlpha;
+		float startAlpha = SubstanceThemeUtilities.getHighlightAlpha(list,
+				prevState);
+		float endAlpha = SubstanceThemeUtilities.getHighlightAlpha(list,
+				currState);
 
 		FadeState state = SubstanceFadeUtilities.getFadeState(list, row,
 				FadeKind.SELECTION, FadeKind.ROLLOVER);
+		float totalAlpha = endAlpha;
+		float fadeCoef = 0.0f;
 		if (state != null) {
-			float fadeCoef = state.getFadePosition();
+			fadeCoef = state.getFadePosition();
 
 			// compute the total alpha of the overlays.
-			float totalAlpha = 0.0f;
 			if (state.isFadingIn()) {
 				totalAlpha = startAlpha + (endAlpha - startAlpha) * fadeCoef
 						/ 10.0f;
@@ -589,43 +626,68 @@ public class SubstanceListUI extends BasicListUI {
 
 			if (state.isFadingIn())
 				fadeCoef = 10.0f - fadeCoef;
-
-			// compute the alpha for each one of the animation overlays
-			alphaForPrevBackground = totalAlpha * fadeCoef / 10.0f;
-			alphaForCurrBackground = totalAlpha * (10.0f - fadeCoef) / 10.0f;
 		}
 
-		SubstanceTheme prevTheme = SubstanceCoreUtilities.getHighlightTheme(
-				list, prevState, true, false);
-		SubstanceTheme currTheme = SubstanceCoreUtilities.getHighlightTheme(
-				list, currState, true, false);
+		final SubstanceTheme prevTheme = SubstanceThemeUtilities
+				.getHighlightTheme(list, prevState);
+		final SubstanceTheme currTheme = SubstanceThemeUtilities
+				.getHighlightTheme(list, currState);
 
 		// System.out.println(row + ":" + prevTheme.getDisplayName() + "["
 		// + alphaForPrevBackground + "]:" + currTheme.getDisplayName()
 		// + "[" + alphaForCurrBackground + "]");
 
-		if (alphaForPrevBackground > 0.0f) {
-			g2d.setComposite(TransitionLayout.getAlphaComposite(list,
-					alphaForPrevBackground, g));
-			SubstanceListUI.backgroundDelegate.update(g2d, rendererComponent,
-					new Rectangle(cx, cy, cw, ch), prevTheme, 0.8f);
-			g2d.setComposite(TransitionLayout.getAlphaComposite(list, g));
-		}
-		if (alphaForCurrBackground > 0.0f) {
-			g2d.setComposite(TransitionLayout.getAlphaComposite(list,
-					alphaForCurrBackground, g));
-			SubstanceListUI.backgroundDelegate.update(g2d, rendererComponent,
-					new Rectangle(cx, cy, cw, ch), currTheme, 0.8f);
-			g2d.setComposite(TransitionLayout.getAlphaComposite(list, g));
+		if (textPainter.needsBackgroundImage()) {
+			final float finalTotalAlpha = totalAlpha;
+			final float finalFadeCoef = fadeCoef;
+
+			// The text GC will be clipped to the renderer bound. To have the
+			// background painted in the correct location, enforce this by
+			// painting the background in 0, 0 position.
+			final Rectangle cellRect = new Rectangle(0, 0, cw, ch);
+			// textPainter.setBackgroundImage(this.list, rendererComponent
+			// .getBackground(), true, cx, cy);
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							Graphics2D g2d = (Graphics2D) g.create();
+
+							if (finalTotalAlpha > 0.0f) {
+								g2d.setComposite(TransitionLayout
+										.getAlphaComposite(list,
+												finalTotalAlpha, g));
+								SubstanceHighlightUtils.paintHighlight(g2d,
+										rendererComponent, cellRect, 0.8f,
+										null, currTheme.getColorScheme(),
+										prevTheme.getColorScheme(),
+										finalFadeCoef);
+								g2d.setComposite(TransitionLayout
+										.getAlphaComposite(list, g));
+							}
+							g2d.dispose();
+						}
+					});
+		} else {
+			Rectangle cellRect = new Rectangle(cx, cy, cw, ch);
+			if (totalAlpha > 0.0f) {
+				g2d.setComposite(TransitionLayout.getAlphaComposite(list,
+						totalAlpha, g));
+				SubstanceHighlightUtils.paintHighlight(g2d, rendererComponent,
+						cellRect, 0.8f, null, currTheme.getColorScheme(),
+						prevTheme.getColorScheme(), fadeCoef);
+				g2d.setComposite(TransitionLayout.getAlphaComposite(list, g));
+			}
 		}
 
+		this.list.putClientProperty(
+				SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, Boolean.TRUE);
 		if (rendererComponent instanceof JComponent) {
 			// Play with opacity to make our own gradient background
 			// on selected elements to show.
 			JComponent jRenderer = (JComponent) rendererComponent;
 			synchronized (jRenderer) {
 				boolean newOpaque = !(isSelected || isRollover || (state != null));
-				if (SubstanceCoreUtilities.toBleedWatermark(list))
+				if (isWatermarkBleed)
 					newOpaque = false;
 
 				Map<Component, Boolean> opacity = new HashMap<Component, Boolean>();
@@ -656,24 +718,11 @@ public class SubstanceListUI extends BasicListUI {
 			rendererPane.paintComponent(g2d, rendererComponent, list, cx, cy,
 					cw, ch, true);
 		}
+		this.list.putClientProperty(
+				SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, null);
 		g2d.dispose();
 	}
 
-	//
-	// /*
-	// * (non-Javadoc)
-	// *
-	// * @see javax.swing.plaf.ComponentUI#update(java.awt.Graphics,
-	// * javax.swing.JComponent)
-	// */
-	// @Override
-	// public void update(Graphics g, JComponent c) {
-	// if (!SubstanceCoreUtilities.toBleedWatermark(c))
-	// super.update(g, c);
-	// else
-	// super.paint(g, c);
-	// }
-
 	/**
 	 * Returns the previous state for the specified cell.
 	 * 
@@ -711,4 +760,29 @@ public class SubstanceListUI extends BasicListUI {
 	public void resetRolloverIndex() {
 		rolledOverIndex = -1;
 	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.ComponentUI#update(java.awt.Graphics,
+	 *      javax.swing.JComponent)
+	 */
+	@Override
+	public void update(Graphics g, JComponent c) {
+		SubstanceFillBackgroundDelegate.GLOBAL_INSTANCE.updateIfOpaque(g, c);
+
+		Graphics2D g2d = (Graphics2D) g.create();
+		// install state-aware alpha channel (support for skins that use
+		// translucency on disabled states).
+		// SubstanceTheme theme = SubstanceCoreUtilities.getTheme(this.list,
+		// true,
+		// true);
+		// float themeAlpha = theme.getThemeAlpha(this.list,
+		// this.list.isEnabled() ? ComponentState.DEFAULT
+		// : ComponentState.DISABLED_UNSELECTED);
+		// g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
+		// themeAlpha));
+		this.paint(g2d, c);
+		g2d.dispose();
+	}
 }
