@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -42,14 +42,15 @@ import javax.swing.plaf.basic.BasicProgressBarUI;
 
 import org.jvnet.lafwidget.animation.FadeKind;
 import org.jvnet.lafwidget.animation.FadeTracker;
+import org.jvnet.lafwidget.layout.TransitionLayout;
 import org.jvnet.substance.border.InnerDelegateBorderPainter;
 import org.jvnet.substance.border.SubstanceBorderPainter;
 import org.jvnet.substance.button.BaseButtonShaper;
 import org.jvnet.substance.color.ColorScheme;
 import org.jvnet.substance.painter.SubstanceGradientPainter;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.theme.SubstanceTheme;
-import org.jvnet.substance.utils.SubstanceCoreUtilities;
-import org.jvnet.substance.utils.SubstanceSizeUtils;
+import org.jvnet.substance.utils.*;
 
 /**
  * UI for progress bars in <b>Substance</b> look and feel.
@@ -228,104 +229,142 @@ public class SubstanceProgressBarUI extends BasicProgressBarUI {
 			return;
 		}
 
-		Insets insets = progressBar.getInsets();
+		final Insets insets = progressBar.getInsets();
 		insets.top /= 2;
 		insets.left /= 2;
 		insets.bottom /= 2;
 		insets.right /= 2;
-		int barRectWidth = progressBar.getWidth()
+		final int barRectWidth = progressBar.getWidth()
 				- (insets.right + insets.left);
-		int barRectHeight = progressBar.getHeight()
+		final int barRectHeight = progressBar.getHeight()
 				- (insets.top + insets.bottom);
 
 		// amount of progress to draw
-		int amountFull = getAmountFull(insets, barRectWidth, barRectHeight);
+		final int amountFull = getAmountFull(insets, barRectWidth,
+				barRectHeight);
 
-		Graphics2D graphics = (Graphics2D) g;
+		SubstanceTextPainter.BackgroundPaintingCallback callback = new SubstanceTextPainter.BackgroundPaintingCallback() {
+			public void paintBackground(Graphics g) {
+				Graphics2D graphics = (Graphics2D) g.create();
 
-		SubstanceTheme bgTheme = SubstanceCoreUtilities.getTheme(progressBar,
-				true);
-		if (progressBar.isEnabled())
-			bgTheme = bgTheme.getDefaultTheme();
-		else
-			bgTheme = bgTheme.getDisabledTheme();
-		SubstanceGradientPainter gp = SubstanceCoreUtilities
-				.getGradientPainter(this.progressBar);
-		// background
-		if (progressBar.getOrientation() == SwingConstants.HORIZONTAL) {
-			Shape contour = BaseButtonShaper.getBaseOutline(barRectWidth + 1,
-					barRectHeight, 0, null);
-			BufferedImage bg = gp.getContourBackground(barRectWidth + 1,
-					barRectHeight, contour, false, bgTheme.getColorScheme(),
-					bgTheme.getColorScheme(), 0, true, false);
-			graphics.drawImage(bg, insets.left, insets.top, null);
-			// SubstanceImageCreator.paintRectangularStripedBackground(graphics,
-			// b.left, b.top, barRectWidth, barRectHeight + 1, bgTheme
-			// .getColorScheme(), null, 0, false);
-		} else { // VERTICAL
-			Shape contour = BaseButtonShaper.getBaseOutline(barRectHeight + 1,
-					barRectWidth, 0, null);
-			BufferedImage bg = gp.getContourBackground(barRectHeight + 1,
-					barRectWidth, contour, false, bgTheme.getColorScheme(),
-					bgTheme.getColorScheme(), 0, true, false);
-			graphics.drawImage(SubstanceImageCreator.getRotated(bg, 3),
-					insets.left, insets.top, null);
-			//
-			// SubstanceImageCreator.paintRectangularStripedBackground(graphics,
-			// b.left, b.top, barRectWidth + 1, barRectHeight, bgTheme
-			// .getColorScheme(), null, 0, true);
-		}
+				SubstanceTheme bgTheme = SubstanceThemeUtilities
+						.getTheme(progressBar);
+				if (progressBar.isEnabled())
+					bgTheme = bgTheme.getDefaultTheme();
+				else
+					bgTheme = bgTheme.getDisabledTheme();
 
-		if (amountFull > 0) {
-			float borderStrokeWidth = SubstanceSizeUtils
-					.getBorderStrokeWidth(SubstanceSizeUtils
-							.getComponentFontSize(this.progressBar));
-			SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-					.getBorderPainter(null, null);
-			int borderDelta = (borderPainter instanceof InnerDelegateBorderPainter) ? (int) (Math
-					.ceil(2.0 * borderStrokeWidth)) / 2
-					: (int) (borderStrokeWidth / 2.0);
+				SubstanceGradientPainter gp = SubstanceCoreUtilities
+						.getGradientPainter(progressBar);
+				// background
+				if (progressBar.getOrientation() == SwingConstants.HORIZONTAL) {
+					Shape contour = BaseButtonShaper.getBaseOutline(
+							barRectWidth + 1, barRectHeight + 1, 0, null);
+					BufferedImage bg = gp.getContourBackground(
+							barRectWidth + 1, barRectHeight + 1, contour,
+							false, bgTheme.getColorScheme(), bgTheme
+									.getColorScheme(), 0, true, false);
+					graphics.drawImage(bg, insets.left, insets.top, null);
+				} else { // VERTICAL
+					Shape contour = BaseButtonShaper.getBaseOutline(
+							barRectHeight + 1, barRectWidth + 1, 0, null);
+					BufferedImage bg = gp.getContourBackground(
+							barRectHeight + 1, barRectWidth + 1, contour,
+							false, bgTheme.getColorScheme(), bgTheme
+									.getColorScheme(), 0, true, false);
+					graphics.drawImage(SubstanceImageCreator.getRotated(bg, 3),
+							insets.left, insets.top, null);
+				}
 
-			ColorScheme fillColorScheme = progressBar.isEnabled() ? SubstanceCoreUtilities
-					.getActiveScheme(progressBar)
-					: SubstanceCoreUtilities.getDisabledScheme(progressBar);
-			if (progressBar.getOrientation() == SwingConstants.HORIZONTAL) {
-				int barWidth = amountFull - 2 * borderDelta;
-				int barHeight = barRectHeight - 2 * borderDelta;
-				if ((barWidth > 0) && (barHeight > 0)) {
-					if (c.getComponentOrientation().isLeftToRight()) {
-						SubstanceImageCreator.paintRectangularBackground(g,
-								insets.left + borderDelta, insets.top
-										+ borderDelta, barWidth, barHeight,
-								fillColorScheme, 0.6f, false);
-					} else {
-						// fix for RTL determinate horizontal progress bar in
-						// 2.3
-						SubstanceImageCreator.paintRectangularBackground(g,
-								insets.left + barRectWidth - amountFull - 2
-										* borderDelta,
-								insets.top + borderDelta, barWidth, barHeight,
-								fillColorScheme, 0.6f, false);
+				if (amountFull > 0) {
+					float borderStrokeWidth = SubstanceSizeUtils
+							.getBorderStrokeWidth(SubstanceSizeUtils
+									.getComponentFontSize(progressBar));
+					SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
+							.getBorderPainter(null);
+					int borderDelta = (borderPainter instanceof InnerDelegateBorderPainter) ? (int) (Math
+							.ceil(2.0 * borderStrokeWidth)) / 2
+							: (int) (borderStrokeWidth / 2.0);
+
+					ColorScheme fillColorScheme = SubstanceThemeUtilities
+							.getTheme(
+									progressBar,
+									progressBar.isEnabled() ? ComponentState.SELECTED
+											: ComponentState.DISABLED_UNSELECTED)
+							.getColorScheme();
+					if (progressBar.getOrientation() == SwingConstants.HORIZONTAL) {
+						int barWidth = amountFull - 2 * borderDelta;
+						int barHeight = barRectHeight - 2 * borderDelta;
+						if ((barWidth > 0) && (barHeight > 0)) {
+							if (progressBar.getComponentOrientation()
+									.isLeftToRight()) {
+								SubstanceImageCreator
+										.paintRectangularBackground(g,
+												insets.left + borderDelta,
+												insets.top + borderDelta,
+												barWidth, barHeight,
+												fillColorScheme, 0.6f, false);
+							} else {
+								// fix for RTL determinate horizontal progress
+								// bar in 2.3
+								SubstanceImageCreator
+										.paintRectangularBackground(g,
+												insets.left + barRectWidth
+														- amountFull - 2
+														* borderDelta,
+												insets.top + borderDelta,
+												barWidth, barHeight,
+												fillColorScheme, 0.6f, false);
+							}
+						}
+					} else { // VERTICAL
+						int barWidth = progressBar.getHeight() - insets.bottom
+								- amountFull - 2 * borderDelta;
+						int barHeight = barRectWidth - 2 * borderDelta;
+						if ((barWidth > 0) && (barHeight > 0)) {
+							// fix for issue 95. Vertical bar is growing from
+							// the bottom
+							SubstanceImageCreator.paintRectangularBackground(g,
+									insets.left + borderDelta, barWidth
+											+ borderDelta, barHeight,
+									amountFull, fillColorScheme, 0.6f, true);
+						}
 					}
 				}
-			} else { // VERTICAL
-				int barWidth = progressBar.getHeight() - insets.bottom
-						- amountFull - 2 * borderDelta;
-				int barHeight = barRectWidth - 2 * borderDelta;
-				if ((barWidth > 0) && (barHeight > 0)) {
-					// fix for issue 95. Vertical bar is growing from the bottom
-					SubstanceImageCreator.paintRectangularBackground(g,
-							insets.left + borderDelta, barWidth + borderDelta,
-							barHeight, amountFull, fillColorScheme, 0.6f, true);
-				}
+				graphics.dispose();
 			}
+		};
+
+		Graphics2D g2d = (Graphics2D) g.create();
+		// install state-aware alpha channel (support for skins
+		// that use translucency on disabled states).
+		float themeAlpha = SubstanceThemeUtilities.getTheme(progressBar)
+				.getThemeAlpha(
+						progressBar,
+						progressBar.isEnabled() ? ComponentState.DEFAULT
+								: ComponentState.DISABLED_UNSELECTED);
+		g2d.setComposite(TransitionLayout.getAlphaComposite(progressBar,
+				themeAlpha, g));
+
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		textPainter.init(this.progressBar, null, true);
+		textPainter.setBackgroundFill(progressBar, progressBar.getParent()
+				.getBackground(), true, 0, 0);
+
+		if (textPainter.needsBackgroundImage()) {
+			textPainter.attachCallback(callback);
+		} else {
+			callback.paintBackground(g2d);
 		}
 
 		// Deal with possible text painting
 		if (progressBar.isStringPainted()) {
-			this.paintString(g, insets.left, insets.top, barRectWidth,
+			this.paintString(g2d, insets.left, insets.top, barRectWidth,
 					barRectHeight, amountFull, insets);
 		}
+		textPainter.renderSurface(g2d);
+		g2d.dispose();
 	}
 
 	/*
@@ -335,13 +374,18 @@ public class SubstanceProgressBarUI extends BasicProgressBarUI {
 	 */
 	@Override
 	protected Color getSelectionBackground() {
-		SubstanceTheme bgTheme = SubstanceCoreUtilities.getTheme(progressBar,
-				true);
-		if (progressBar.isEnabled())
-			bgTheme = bgTheme.getDefaultTheme();
-		else
-			bgTheme = bgTheme.getDisabledTheme();
-		return bgTheme.getForegroundColor();
+		ComponentState state = this.progressBar.isEnabled() ? ComponentState.DEFAULT
+				: ComponentState.DISABLED_UNSELECTED;
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(
+				this.progressBar, state);
+		return theme.getForegroundColor();
+		// SubstanceTheme bgTheme = SubstanceCoreUtilities.getTheme(progressBar,
+		// true, true);
+		// if (progressBar.isEnabled())
+		// bgTheme = bgTheme.getDefaultTheme();
+		// else
+		// bgTheme = bgTheme.getDisabledTheme();
+		// return bgTheme.getForegroundColor();
 	}
 
 	/*
@@ -351,10 +395,16 @@ public class SubstanceProgressBarUI extends BasicProgressBarUI {
 	 */
 	@Override
 	protected Color getSelectionForeground() {
-		ColorScheme fillColorScheme = progressBar.isEnabled() ? SubstanceCoreUtilities
-				.getActiveScheme(progressBar)
-				: SubstanceCoreUtilities.getDefaultScheme(progressBar);
-		return fillColorScheme.getForegroundColor();
+		ComponentState state = this.progressBar.isEnabled() ? ComponentState.ACTIVE
+				: ComponentState.DISABLED_UNSELECTED;
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(
+				this.progressBar, state);
+		return theme.getForegroundColor();
+		// ColorScheme fillColorScheme = progressBar.isEnabled() ?
+		// SubstanceCoreUtilities
+		// .getActiveScheme(progressBar)
+		// : SubstanceCoreUtilities.getDefaultScheme(progressBar);
+		// return fillColorScheme.getForegroundColor();
 	}
 
 	/*
@@ -369,51 +419,71 @@ public class SubstanceProgressBarUI extends BasicProgressBarUI {
 			return;
 		}
 
-		Insets b = progressBar.getInsets(); // area for border
-		int barRectWidth = progressBar.getWidth() - (b.right + b.left);
-		int barRectHeight = progressBar.getHeight() - (b.top + b.bottom);
+		final Insets b = progressBar.getInsets(); // area for border
+		final int barRectWidth = progressBar.getWidth() - (b.right + b.left);
+		final int barRectHeight = progressBar.getHeight() - (b.top + b.bottom);
 
-		int valComplete = (int) animationIndex;// int)
-		// (progressBar.getPercentComplete()
-		// * barRectWidth);
-		Graphics2D graphics = (Graphics2D) g;
+		final int valComplete = (int) animationIndex;
 
-		// background
-		if (progressBar.getOrientation() == SwingConstants.HORIZONTAL) {
-			// SubstanceImageCreator.paintRectangularStripedBackground(graphics,
-			// b.left, b.top, barRectWidth, barRectHeight + 1,
-			// SubstanceCoreUtilities.getDisabledScheme(progressBar),
-			// null, 0, false);
-		} else { // VERTICAL
-		// SubstanceImageCreator.paintRectangularStripedBackground(graphics,
-		// b.left, b.top, barRectWidth + 1, barRectHeight,
-		// SubstanceCoreUtilities.getDisabledScheme(progressBar),
-		// null, 0, 0.6f, true);
-		}
+		SubstanceTextPainter.BackgroundPaintingCallback callback = new SubstanceTextPainter.BackgroundPaintingCallback() {
+			public void paintBackground(Graphics g) {
+				Graphics2D graphics = (Graphics2D) g.create();
 
-		ColorScheme fillColorScheme = progressBar.isEnabled() ? SubstanceCoreUtilities
-				.getActiveScheme(progressBar)
-				: SubstanceCoreUtilities.getDefaultScheme(progressBar);
-		if (progressBar.getOrientation() == SwingConstants.HORIZONTAL) {
-			SubstanceImageCreator.paintRectangularStripedBackground(graphics,
-					b.left, b.top, barRectWidth, barRectHeight,
-					fillColorScheme, SubstanceProgressBarUI.getStripe(
-							barRectHeight, false, fillColorScheme),
-					valComplete, 0.6f, false);
-		} else { // VERTICAL
-			// fix for issue 95. Vertical progress bar rises from the bottom.
-			SubstanceImageCreator.paintRectangularStripedBackground(graphics,
-					b.left, b.top, barRectWidth, barRectHeight,
-					fillColorScheme, SubstanceProgressBarUI.getStripe(
-							barRectWidth, true, fillColorScheme), 2
-							* barRectWidth - valComplete, 0.6f, true);
+				ColorScheme fillColorScheme = SubstanceThemeUtilities.getTheme(
+						progressBar,
+						progressBar.isEnabled() ? ComponentState.SELECTED
+								: ComponentState.DISABLED_UNSELECTED)
+						.getColorScheme();
+				if (progressBar.getOrientation() == SwingConstants.HORIZONTAL) {
+					SubstanceImageCreator.paintRectangularStripedBackground(
+							graphics, b.left, b.top, barRectWidth,
+							barRectHeight, fillColorScheme,
+							SubstanceProgressBarUI.getStripe(barRectHeight,
+									false, fillColorScheme), valComplete, 0.6f,
+							false);
+				} else {
+					// fix for issue 95. Vertical progress bar grows from the
+					// bottom.
+					SubstanceImageCreator.paintRectangularStripedBackground(
+							graphics, b.left, b.top, barRectWidth,
+							barRectHeight, fillColorScheme,
+							SubstanceProgressBarUI.getStripe(barRectWidth,
+									true, fillColorScheme), 2 * barRectWidth
+									- valComplete, 0.6f, true);
+				}
+				graphics.dispose();
+			}
+		};
+
+		Graphics2D g2d = (Graphics2D) g.create();
+		// install state-aware alpha channel (support for skins
+		// that use translucency on disabled states).
+		float themeAlpha = SubstanceThemeUtilities.getTheme(progressBar)
+				.getThemeAlpha(
+						progressBar,
+						progressBar.isEnabled() ? ComponentState.DEFAULT
+								: ComponentState.DISABLED_UNSELECTED);
+		g2d.setComposite(TransitionLayout.getAlphaComposite(progressBar,
+				themeAlpha, g));
+
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		textPainter.init(this.progressBar, null, true);
+		textPainter.setBackgroundFill(progressBar, progressBar.getParent()
+				.getBackground(), true, 0, 0);
+		if (textPainter.needsBackgroundImage()) {
+			textPainter.attachCallback(callback);
+		} else {
+			callback.paintBackground(g2d);
 		}
 
 		// Deal with possible text painting
 		if (progressBar.isStringPainted()) {
-			this.paintString(g, b.left, b.top, barRectWidth, barRectHeight,
+			this.paintString(g2d, b.left, b.top, barRectWidth, barRectHeight,
 					barRectWidth, b);
 		}
+		textPainter.renderSurface(g2d);
+		g2d.dispose();
 	}
 
 	/*
@@ -477,7 +547,7 @@ public class SubstanceProgressBarUI extends BasicProgressBarUI {
 			int from = (Integer) progressBar
 					.getClientProperty(PROGRESS_BAR_FROM);
 			int to = (Integer) progressBar.getClientProperty(PROGRESS_BAR_TO);
-			currentValue = from + (int) (fade10 * (to - from) / 10.0);
+			currentValue = from + (fade10 * (to - from) / 10.0);
 		}
 
 		double percentComplete = (currentValue - model.getMinimum()) / span;
@@ -517,4 +587,122 @@ public class SubstanceProgressBarUI extends BasicProgressBarUI {
 		return new Dimension(size, 146 + SubstanceSizeUtils.getAdjustedSize(
 				size, 0, 1, 10, false));
 	}
+
+	@Override
+	protected void paintString(Graphics g, int x, int y, int width, int height,
+			int amountFull, Insets b) {
+		if (progressBar.getOrientation() == JProgressBar.HORIZONTAL) {
+			if (progressBar.getComponentOrientation().isLeftToRight()) {
+				if (progressBar.isIndeterminate()) {
+					boxRect = getBox(boxRect);
+					paintString(x, y, width, height, boxRect.x, boxRect.width,
+							b);
+				} else {
+					paintString(x, y, width, height, x, amountFull, b);
+				}
+			} else {
+				paintString(x, y, width, height, x + width - amountFull,
+						amountFull, b);
+			}
+		} else {
+			if (progressBar.isIndeterminate()) {
+				boxRect = getBox(boxRect);
+				paintString(x, y, width, height, boxRect.y, boxRect.height, b);
+			} else {
+				paintString(x, y, width, height, y + height - amountFull,
+						amountFull, b);
+			}
+		}
+	}
+
+	/**
+	 * Paints the progress string.
+	 * 
+	 * @param g
+	 *            Graphics used for drawing.
+	 * @param x
+	 *            x location of bounding box
+	 * @param y
+	 *            y location of bounding box
+	 * @param width
+	 *            width of bounding box
+	 * @param height
+	 *            height of bounding box
+	 * @param fillStart
+	 *            start location, in x or y depending on orientation, of the
+	 *            filled portion of the progress bar.
+	 * @param amountFull
+	 *            size of the fill region, either width or height depending upon
+	 *            orientation.
+	 * @param b
+	 *            Insets of the progress bar.
+	 */
+	private void paintString(int x, int y, int width, int height,
+			int fillStart, int amountFull, Insets b) {
+		String progressString = progressBar.getString();
+		Rectangle renderRectangle = getStringRectangle(progressString, x, y,
+				width, height);
+
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		if (progressBar.getOrientation() == JProgressBar.HORIZONTAL) {
+			textPainter.attachText(this.progressBar, renderRectangle,
+					progressString, -1, progressBar.getFont(),
+					getSelectionBackground(), new Rectangle(amountFull, y,
+							progressBar.getWidth() - amountFull, height));
+
+			textPainter.attachText(this.progressBar, renderRectangle,
+					progressString, -1, progressBar.getFont(),
+					getSelectionForeground(), new Rectangle(fillStart, y,
+							amountFull, height));
+		} else { // VERTICAL
+			textPainter.attachVerticalText(this.progressBar, renderRectangle,
+					progressString, -1, progressBar.getFont(),
+					getSelectionBackground(), new Rectangle(x, y, width,
+							progressBar.getHeight() - amountFull), progressBar
+							.getComponentOrientation().isLeftToRight());
+
+			textPainter.attachVerticalText(this.progressBar, renderRectangle,
+					progressString, -1, progressBar.getFont(),
+					getSelectionForeground(), new Rectangle(x, fillStart,
+							width, amountFull), progressBar
+							.getComponentOrientation().isLeftToRight());
+		}
+	}
+
+	/**
+	 * Returns the rectangle for the progress bar string.
+	 * 
+	 * @param progressString
+	 *            Progress bar string.
+	 * @param x
+	 *            x location of bounding box
+	 * @param y
+	 *            y location of bounding box
+	 * @param width
+	 *            width of bounding box
+	 * @param height
+	 *            height of bounding box
+	 * @return The rectangle for the progress bar string.
+	 */
+	protected Rectangle getStringRectangle(String progressString, int x, int y,
+			int width, int height) {
+		FontMetrics fontSizer = progressBar.getFontMetrics(progressBar
+				.getFont());
+
+		int stringWidth = fontSizer.stringWidth(progressString);
+
+		if (progressBar.getOrientation() == JProgressBar.HORIZONTAL) {
+			return new Rectangle(x + Math.round(width / 2 - stringWidth / 2), y
+					+ (height - fontSizer.getHeight()) / 2, stringWidth,
+					fontSizer.getHeight());
+		} else {
+			return new Rectangle(x + (width - fontSizer.getHeight()) / 2, y
+					+ Math.round(height / 2 - stringWidth / 2), fontSizer
+					.getHeight(), stringWidth);
+			// return new Rectangle(x - height / 2 - fontSizer.getHeight(), y,
+			// fontSizer.getHeight(), stringWidth);
+		}
+	}
+
 }
