@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -162,6 +162,9 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 		return new SubstanceRootPaneUI();
 	}
 
+	public SubstanceRootPaneUI() {
+	}
+
 	/**
 	 * Invokes supers implementation of <code>installUI</code> to install the
 	 * necessary state onto the passed in <code>JRootPane</code> to render the
@@ -488,6 +491,7 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 		root.addHierarchyListener(this.substanceHierarchyListener);
 		// System.out.println(root.hashCode() + ":"
 		// + root.getHierarchyListeners().length);
+
 	}
 
 	/*
@@ -508,6 +512,7 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 		}
 		root.removeHierarchyListener(this.substanceHierarchyListener);
 		this.substanceHierarchyListener = null;
+
 		super.uninstallListeners(root);
 	}
 
@@ -721,12 +726,17 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 			if (style != JRootPane.NONE) {
 				this.installClientDecorations(root);
 			}
-		} else if (propertyName.equals("ancestor")) {
+		}
+		if (propertyName.equals("ancestor")) {
 			this.uninstallWindowListeners(this.root);
 			if (((JRootPane) e.getSource()).getWindowDecorationStyle() != JRootPane.NONE) {
 				this.installWindowListeners(this.root, this.root.getParent());
 			}
 		}
+		if (propertyName.equals("background")) {
+			SubstanceLookAndFeel.getTitlePaneComponent(window).setBackground(
+					(Color) e.getNewValue());
+		}
 		return;
 	}
 
