@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -43,6 +43,7 @@ import javax.swing.event.ChangeListener;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.UIResource;
 import javax.swing.plaf.basic.BasicScrollPaneUI;
+import javax.swing.table.JTableHeader;
 
 import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.animation.*;
@@ -100,14 +101,20 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 	 * @see javax.swing.plaf.basic.BasicScrollPaneUI#installDefaults(javax.swing.JScrollPane)
 	 */
 	@Override
-	protected void installDefaults(JScrollPane scrollpane) {
+	protected void installDefaults(final JScrollPane scrollpane) {
 		super.installDefaults(scrollpane);
-		if (SubstanceCoreUtilities.toBleedWatermark(this.scrollpane)) {
-			this.scrollpane.setOpaque(false);
-			this.scrollpane.getViewport().setOpaque(false);
+		if (SubstanceCoreUtilities.toBleedWatermark(scrollpane)) {
+			scrollpane.setOpaque(false);
+			scrollpane.getViewport().setOpaque(false);
 		}
 		scrollpane.setLayout(new AdjustedLayout((ScrollPaneLayout) scrollpane
 				.getLayout()));
+
+		SwingUtilities.invokeLater(new Runnable() {
+			public void run() {
+				installTableHeaderCornerFiller(scrollpane);
+			}
+		});
 	}
 
 	/*
@@ -117,6 +124,15 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 	 */
 	@Override
 	protected void uninstallDefaults(JScrollPane c) {
+		Component upperRight = c.getCorner(JScrollPane.UPPER_RIGHT_CORNER);
+		if (upperRight instanceof UIResource) {
+			c.setCorner(JScrollPane.UPPER_RIGHT_CORNER, null);
+		}
+		Component upperLeft = c.getCorner(JScrollPane.UPPER_LEFT_CORNER);
+		if (upperLeft instanceof UIResource) {
+			c.setCorner(JScrollPane.UPPER_LEFT_CORNER, null);
+		}
+
 		LayoutManager lm = scrollpane.getLayout();
 		if (lm instanceof AdjustedLayout) {
 			c.setLayout(((AdjustedLayout) lm).delegate);
@@ -182,6 +198,48 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 					// }
 					// }
 				}
+				if ("background".equals(evt.getPropertyName())) {
+					// propagate application-specific background color to the
+					// scroll bars.
+					Color newBackgr = (Color) evt.getNewValue();
+					if (!(newBackgr instanceof UIResource)) {
+						JScrollBar vertical = scrollpane.getVerticalScrollBar();
+						if (vertical != null) {
+							if (vertical.getBackground() instanceof UIResource) {
+								vertical.setBackground(newBackgr);
+							}
+						}
+						JScrollBar horizontal = scrollpane
+								.getHorizontalScrollBar();
+						if (horizontal != null) {
+							if (horizontal.getBackground() instanceof UIResource) {
+								horizontal.setBackground(newBackgr);
+							}
+						}
+					}
+				}
+				if ("columnHeader".equals(evt.getPropertyName())) {
+					SwingUtilities.invokeLater(new Runnable() {
+						public void run() {
+							// need to switch the corner filler based on the
+							// header
+							if (scrollpane != null) {
+								installTableHeaderCornerFiller(scrollpane);
+							}
+						}
+					});
+				}
+				if ("componentOrientation".equals(evt.getPropertyName())) {
+					SwingUtilities.invokeLater(new Runnable() {
+						public void run() {
+							// need to switch the corner filler based on the
+							// component orientation
+							if (scrollpane != null) {
+								installTableHeaderCornerFiller(scrollpane);
+							}
+						}
+					});
+				}
 			}
 		};
 		c.addPropertyChangeListener(this.substancePropertyChangeListener);
@@ -510,20 +568,21 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 								.getComponentFontSize(scrollPane));
 				int dx = 0, dy = 0, dw = 0, dh = 0;
 				if (scrollPane.getComponentOrientation().isLeftToRight()) {
-					if (vertical.isVisible()) {
+					if ((vertical != null) && vertical.isVisible()) {
 						Rectangle vBounds = vertical.getBounds();
 						dw += (1 + borderDelta);
 						vertical.setBounds(vBounds.x + 1 + borderDelta,
 								vBounds.y + 1 - 2 * borderWidth, vBounds.width,
 								vBounds.height + 2 * borderWidth);
 					}
-					if (horizontal.isVisible()) {
+					if ((horizontal != null) && horizontal.isVisible()) {
 						dh += (1 + borderDelta);
 						Rectangle hBounds = horizontal.getBounds();
 						horizontal.setBounds(hBounds.x + 1 - 2 * borderWidth,
 								hBounds.y + 1, hBounds.width + 2 * borderWidth,
 								hBounds.height);
 					}
+
 					if (delegate.getCorner(ScrollPaneLayout.LOWER_RIGHT_CORNER) != null) {
 						Rectangle lrBounds = delegate.getCorner(
 								ScrollPaneLayout.LOWER_RIGHT_CORNER)
@@ -533,8 +592,17 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 										lrBounds.y + 1 + borderDelta,
 										lrBounds.width, lrBounds.height);
 					}
+					if (delegate.getCorner(ScrollPaneLayout.UPPER_RIGHT_CORNER) != null) {
+						Rectangle urBounds = delegate.getCorner(
+								ScrollPaneLayout.UPPER_RIGHT_CORNER)
+								.getBounds();
+						delegate.getCorner(ScrollPaneLayout.UPPER_RIGHT_CORNER)
+								.setBounds(urBounds.x + 1 + borderDelta,
+										urBounds.y + borderDelta,
+										urBounds.width - 1, urBounds.height);
+					}
 				} else {
-					if (vertical.isVisible()) {
+					if ((vertical != null) && vertical.isVisible()) {
 						dx -= (1 + borderDelta);
 						dw += (1 + borderDelta);
 						Rectangle vBounds = vertical.getBounds();
@@ -542,7 +610,7 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 								vBounds.y - 1 - borderDelta, vBounds.width,
 								vBounds.height + 2 * borderWidth);
 					}
-					if (horizontal.isVisible()) {
+					if ((horizontal != null) && horizontal.isVisible()) {
 						dh += (1 + borderDelta);
 						Rectangle hBounds = horizontal.getBounds();
 						horizontal.setBounds(hBounds.x - 1 - borderDelta,
@@ -557,6 +625,14 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 										llBounds.y - 1 - borderDelta,
 										llBounds.width, llBounds.height);
 					}
+					if (delegate.getCorner(ScrollPaneLayout.UPPER_LEFT_CORNER) != null) {
+						Rectangle ulBounds = delegate.getCorner(
+								ScrollPaneLayout.UPPER_LEFT_CORNER).getBounds();
+						delegate.getCorner(ScrollPaneLayout.UPPER_LEFT_CORNER)
+								.setBounds(ulBounds.x - borderDelta,
+										ulBounds.y - borderDelta,
+										ulBounds.width - 1, ulBounds.height);
+					}
 				}
 
 				if (delegate.getViewport() != null) {
@@ -565,6 +641,15 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 							new Rectangle(vpBounds.x + dx, vpBounds.y + dy,
 									vpBounds.width + dw, vpBounds.height + dh));
 				}
+				if (delegate.getColumnHeader() != null) {
+					Rectangle columnHeaderBounds = delegate.getColumnHeader()
+							.getBounds();
+					delegate.getColumnHeader().setBounds(
+							new Rectangle(columnHeaderBounds.x + dx,
+									columnHeaderBounds.y + dy,
+									columnHeaderBounds.width + dw,
+									columnHeaderBounds.height));
+				}
 			}
 		}
 	}
@@ -649,4 +734,41 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 
 		super.paint(g, c);
 	}
+
+	protected static void installTableHeaderCornerFiller(JScrollPane scrollpane) {
+		// install custom scroll pane corner filler
+		// for continuous painting of table headers
+		JViewport columnHeader = scrollpane.getColumnHeader();
+		if (columnHeader == null)
+			return;
+		Component columnHeaderComp = columnHeader.getView();
+		if (!(columnHeaderComp instanceof JTableHeader))
+			return;
+		JTableHeader tableHeader = (JTableHeader) columnHeaderComp;
+		SubstanceTableHeaderUI ui = (SubstanceTableHeaderUI) tableHeader
+				.getUI();
+		JComponent scrollPaneCornerFiller = ui.getScrollPaneCornerFiller();
+		// System.err.println("Installed " + scrollPaneCornerFiller.hashCode()
+		// + " on " + scrollpane.hashCode());
+		String cornerKey = scrollpane.getComponentOrientation().isLeftToRight() ? JScrollPane.UPPER_RIGHT_CORNER
+				: JScrollPane.UPPER_LEFT_CORNER;
+		Component cornerComp = scrollpane.getCorner(cornerKey);
+		// Corner component can be replaced when the current one is null or
+		// UIResource
+		boolean canReplace = (cornerComp == null)
+				|| (cornerComp instanceof UIResource);
+		// Fix for issue 343 - don't set the corner when the
+		// component is JXTable
+		Class tableClass = tableHeader.getTable().getClass();
+		while (tableClass != null) {
+			if ("org.jdesktop.swingx.JXTable".equals(tableClass.getName())) {
+				canReplace = false;
+				break;
+			}
+			tableClass = tableClass.getSuperclass();
+		}
+		if (canReplace) {
+			scrollpane.setCorner(cornerKey, scrollPaneCornerFiller);
+		}
+	}
 }
