@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -35,10 +35,14 @@ import java.awt.geom.GeneralPath;
 import java.awt.image.BufferedImage;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
+import java.util.Dictionary;
+import java.util.Enumeration;
 
 import javax.swing.*;
 import javax.swing.plaf.*;
+import javax.swing.plaf.basic.BasicHTML;
 import javax.swing.plaf.basic.BasicSliderUI;
+import javax.swing.text.View;
 
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
@@ -47,6 +51,7 @@ import org.jvnet.substance.button.BaseButtonShaper;
 import org.jvnet.substance.color.ColorScheme;
 import org.jvnet.substance.painter.ClassicGradientPainter;
 import org.jvnet.substance.painter.SubstanceGradientPainter;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.*;
 import org.jvnet.substance.utils.icon.SubstanceIconFactory;
@@ -82,8 +87,6 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	 */
 	protected FadeStateListener substanceFadeStateListener;
 
-	// private MouseMotionListener substanceMouseMotionListener;
-
 	/**
 	 * Icon for horizontal sliders.
 	 */
@@ -117,19 +120,11 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	public SubstanceSliderUI(JSlider slider) {
 		super(null);
 		this.thumbModel = new DefaultButtonModel();
-		// {
-		// @Override
-		// public void setRollover(boolean b) {
-		// super.setRollover(b);
-		// System.out.println("Rollover -> " + b);
-		// }
-		// };
 		this.thumbModel.setArmed(false);
 		this.thumbModel.setSelected(false);
 		this.thumbModel.setPressed(false);
 		this.thumbModel.setRollover(false);
 		this.thumbModel.setEnabled(slider.isEnabled());
-
 	}
 
 	/*
@@ -203,37 +198,27 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 		ComponentState prevState = SubstanceCoreUtilities
 				.getPrevComponentState(this.slider);
 
-		SubstanceTheme trackTheme = SubstanceCoreUtilities.getTheme(
-				this.slider, true);
+		SubstanceTheme trackTheme = SubstanceThemeUtilities
+				.getTheme(this.slider);
 		if (this.slider.isEnabled())
 			trackTheme = trackTheme.getDefaultTheme();
 		else
 			trackTheme = trackTheme.getDisabledTheme();
 
-		// trackTheme = trackTheme.getDefaultTheme();
-		// if (!this.slider.isEnabled()) {
-		// graphics.setComposite(TransitionLayout.getAlphaComposite(
-		// this.slider, 0.7f, g));
-		// }
-		// System.out.println("Painting track " + width + ":" + height);
 		this.paintSliderTrack(graphics, drawInverted, trackTheme, width,
 				height, (this.slider.getOrientation() == JSlider.VERTICAL));
-		// graphics.setComposite(TransitionLayout.getAlphaComposite(
-		// this.slider, g));
 
-		SubstanceTheme theme = SubstanceCoreUtilities.getTheme(this.slider,
-				currState, true, true);
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(this.slider,
+				currState);
 		FadeState fadeState = SubstanceFadeUtilities.getFadeState(this.slider,
 				FadeKind.ROLLOVER, FadeKind.SELECTION, FadeKind.PRESS);
 		if (fadeState != null) {
-			SubstanceTheme prevTheme = SubstanceCoreUtilities.getTheme(
-					this.slider, prevState, true, true);
+			SubstanceTheme prevTheme = SubstanceThemeUtilities.getTheme(
+					this.slider, prevState);
 			float cyclePos = fadeState.getFadePosition();
 			if (!fadeState.isFadingIn())
 				cyclePos = 10.0f - cyclePos;
 
-			// System.out.println(prevTheme.getDisplayName() + " -> "
-			// + theme.getDisplayName() + " at " + cyclePos);
 			if (prevState != ComponentState.DEFAULT) {
 				graphics.setComposite(TransitionLayout.getAlphaComposite(
 						this.slider, 1.0f - cyclePos / 10.0f, g));
@@ -257,11 +242,6 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 			}
 		}
 
-		// graphics.setColor(Color.blue);
-		// graphics.translate(-paintRect.x, -paintRect.y);
-		// graphics.draw(this.getPaintTrackRect());
-		// graphics.setColor(Color.green);
-		// graphics.draw(this.trackRect);
 		graphics.dispose();
 	}
 
@@ -290,7 +270,7 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 		SubstanceGradientPainter gradientPainter = new ClassicGradientPainter();
 		ColorScheme fillColorScheme = theme.getColorScheme();
 		SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-				.getBorderPainter(this.slider, gradientPainter);
+				.getBorderPainter(this.slider);
 
 		// some strange bug in clipping a rotated Graphics removes the
 		// last pixel row during the border painting. So, for vertical sliders
@@ -356,19 +336,10 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 		insets.bottom /= 2;
 		insets.right /= 2;
 
-		// boolean isDark = SubstanceCoreUtilities.isThemeDark(theme);
-		// ColorScheme colorScheme = theme.getColorScheme();
-		//
-		// int radius = 2;
-		// Color fillColor1 = isDark ? colorScheme.getUltraLightColor()
-		// : colorScheme.getUltraLightColor();
-		// Color fillColor2 = isDark ? colorScheme.getLightColor() : colorScheme
-		// .getLightColor();
-
 		SubstanceGradientPainter gp = SubstanceCoreUtilities
 				.getGradientPainter(this.slider);
 		SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-				.getBorderPainter(this.slider, gp);
+				.getBorderPainter(this.slider);
 		float radius = SubstanceSizeUtils
 				.getClassicButtonCornerRadius(SubstanceSizeUtils
 						.getComponentFontSize(slider)) / 2.0f;
@@ -442,16 +413,14 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 		}
 	}
 
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.basic.BasicSliderUI#getThumbSize()
+	 */
 	@Override
 	protected Dimension getThumbSize() {
 		Icon thumbIcon = this.getIcon();
-		// if (slider.getOrientation() == JSlider.HORIZONTAL) {
-		// thumbIcon = UIManager.getIcon("Slider.horizontalThumbIcon");
-		// } else {
-		// thumbIcon = UIManager.getIcon("Slider.verticalThumbIcon");
-		// }
-		// System.out.println(thumbIcon.getIconWidth() + ":" +
-		// thumbIcon.getIconHeight());
 		return new Dimension(thumbIcon.getIconWidth(), thumbIcon
 				.getIconHeight());
 	}
@@ -491,31 +460,15 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 
 		Icon icon = this.getIcon();
 		if (this.slider.getOrientation() == JSlider.HORIZONTAL) {
-			int yOffset = 0;
-			// if (this.slider.getPaintTicks()) {
-			// yOffset = 1;
-			// } else {
-			// yOffset = this.slider.getPaintLabels() ? 1 : -1;
-			// }
 			if (icon != null)
-				icon.paintIcon(this.slider, graphics, -1, yOffset);
+				icon.paintIcon(this.slider, graphics, -1, 0);
 		} else {
 			if (this.slider.getComponentOrientation().isLeftToRight()) {
-				int xOffset = 0;
-				// if (this.slider.getPaintTicks())
-				// xOffset = 4;
-				// else
-				// xOffset = this.slider.getPaintLabels() ? 2 : 0;
 				if (icon != null)
-					icon.paintIcon(this.slider, graphics, xOffset, -1);
+					icon.paintIcon(this.slider, graphics, 0, -1);
 			} else {
-				int xOffset = 0;
-				// if (this.slider.getPaintTicks())
-				// xOffset = -1;
-				// else
-				// xOffset = this.slider.getPaintLabels() ? 2 : 2;
 				if (icon != null)
-					icon.paintIcon(this.slider, graphics, xOffset, 1);
+					icon.paintIcon(this.slider, graphics, 0, 1);
 			}
 		}
 
@@ -530,28 +483,75 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	 *      javax.swing.JComponent)
 	 */
 	@Override
-	public synchronized void paint(Graphics g, JComponent c) {
+	public synchronized void paint(Graphics g, final JComponent c) {
 		Graphics2D graphics = (Graphics2D) g.create();
+
 		ComponentState currState = ComponentState.getState(this.thumbModel,
 				this.slider);
-		float alpha = SubstanceCoreUtilities.getTheme(this.slider, true)
+		float alpha = SubstanceThemeUtilities.getTheme(this.slider)
 				.getThemeAlpha(this.slider, currState);
+
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		textPainter.init(c, null, true);
+		if (textPainter.needsBackgroundImage()) {
+			textPainter.setBackgroundFill(c, c.getBackground(), true, 0, 0);
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							// need for the non-showing labels
+							bgDelegate.updateIfOpaque(g, c);
+						}
+					});
+		} else {
+			bgDelegate.updateIfOpaque(graphics, c);
+			// graphics.setColor(c.getBackground());
+			// graphics.fillRect(0, 0, c.getWidth(), c.getHeight());
+			// System.out.println(c.isEnabled() + ":" + c.getBackground());
+		}
+		// // remove opacity
+		// boolean isOpaque = c.isOpaque();
+		// c.setOpaque(false);
+		// // super.paint(graphics, c);
+
+		recalculateIfInsetsChanged();
+		recalculateIfOrientationChanged();
+		final Rectangle clip = graphics.getClipBounds();
+
+		if (!clip.intersects(trackRect) && slider.getPaintTrack())
+			calculateGeometry();
+
 		graphics.setComposite(TransitionLayout.getAlphaComposite(this.slider,
 				alpha, g));
-
-		SubstanceCoreUtilities.workaroundBug6576507(graphics);
-
-		// important to synchronize on the slider as we are
-		// about to fiddle with its opaqueness
-		synchronized (c) {
-			SubstanceSliderUI.bgDelegate.updateIfOpaque(graphics, c);
-			// remove opaqueness
-			boolean isOpaque = c.isOpaque();
-			c.setOpaque(false);
-			super.paint(graphics, c);
-			// restore opaqueness
-			c.setOpaque(isOpaque);
+		SubstanceTextPainter.BackgroundPaintingCallback callback = new SubstanceTextPainter.BackgroundPaintingCallback() {
+			public void paintBackground(Graphics g) {
+				if (slider.getPaintTrack() && clip.intersects(trackRect)) {
+					paintTrack(g);
+				}
+				if (slider.getPaintTicks() && clip.intersects(tickRect)) {
+					paintTicks(g);
+				}
+				if (slider.hasFocus() && clip.intersects(focusRect)) {
+					paintFocus(g);
+				}
+				if (clip.intersects(thumbRect)) {
+					paintThumb(g);
+				}
+			}
+		};
+		if (textPainter.needsBackgroundImage()) {
+			textPainter.attachCallback(callback);
+		} else {
+			callback.paintBackground(graphics);
 		}
+		if (slider.getPaintLabels() && clip.intersects(labelRect)) {
+			paintLabels(graphics);
+		}
+
+		textPainter.renderSurface(graphics);
+
+		// // restore opacity
+		// c.setOpaque(isOpaque);
 
 		if (!this.slider.hasFocus()) {
 			if (FadeTracker.getInstance().isTracked(c, FadeKind.FOCUS))
@@ -559,16 +559,6 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 		}
 
 		graphics.dispose();
-		// g.setColor(Color.green);
-		// ((Graphics2D) g).drawRect(0, 0, c.getWidth() - 1, c.getHeight() - 1);
-		// g.setColor(Color.blue);
-		// ((Graphics2D) g).draw(this.thumbRect);
-		// g.setColor(Color.red);
-		// ((Graphics2D) g).draw(this.trackRect);
-		// g.setColor(Color.green);
-		// ((Graphics2D) g).draw(this.tickRect);
-		// g.setColor(Color.magenta);
-		// ((Graphics2D) g).draw(this.labelRect);
 	}
 
 	/**
@@ -655,19 +645,10 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 				.addPropertyChangeListener(this.substancePropertyChangeListener);
 
 		this.substanceFadeStateListener = new FadeStateListener(this.slider,
-				this.thumbModel, SubstanceCoreUtilities.getFadeCallback(
-						this.slider, this.thumbModel, false));
+				this.thumbModel, SubstanceCoreUtilities
+						.getFadeCallback(this.slider, this.thumbModel, false,
+								false, this.slider));
 		this.substanceFadeStateListener.registerListeners(false);
-
-		//
-		// this.substanceMouseMotionListener = new MouseMotionAdapter() {
-		// @Override
-		// public void mouseMoved(MouseEvent e) {
-		// calculateThumbSize();
-		// //System.out.println("-->" + thumbRect);
-		// }
-		// };
-		// this.slider.addMouseMotionListener(this.substanceMouseMotionListener);
 	}
 
 	/*
@@ -690,9 +671,6 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 
 		this.substanceFadeStateListener.unregisterListeners();
 		this.substanceFadeStateListener = null;
-		//
-		// slider.removeMouseMotionListener(this.substanceMouseMotionListener);
-		// this.substanceMouseMotionListener = null;
 	}
 
 	/*
@@ -835,28 +813,17 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 			Rectangle tickBounds, int x) {
 		boolean isDark = SubstanceCoreUtilities
 				.isThemeDark(SubstanceLookAndFeel.getTheme());
-		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
-				.getDefaultScheme(this.slider)
-				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
-		// Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
-		// .getDarkColor();
-		// Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
-		// .getUltraLightColor();
+
+		ColorScheme colorScheme = SubstanceThemeUtilities.getTheme(
+				this.slider,
+				this.slider.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED).getColorScheme();
 
 		Graphics2D graphics = (Graphics2D) g.create();
 		graphics.translate(x - 1, 0);
 		SubstanceCoreUtilities.paintSeparator(this.slider, graphics,
 				colorScheme, isDark, 0, tickBounds.height, JSeparator.VERTICAL,
 				true, 0, 4);
-		//		
-		// graphics.setPaint(new GradientPaint(x - 1, 0, color, x - 1,
-		// tickBounds.height, SubstanceColorUtilities.getAlphaColor(color,
-		// 96)));
-		// graphics.drawLine(x - 1, 0, x - 1, tickBounds.height - 1);
-		// graphics.setPaint(new GradientPaint(x, 0, icolor, x,
-		// tickBounds.height,
-		// SubstanceColorUtilities.getAlphaColor(icolor, 96)));
-		// graphics.drawLine(x, 0, x, tickBounds.height - 1);
 		graphics.dispose();
 	}
 
@@ -871,13 +838,10 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 			Rectangle tickBounds, int y) {
 		boolean isDark = SubstanceCoreUtilities
 				.isThemeDark(SubstanceLookAndFeel.getTheme());
-		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
-				.getDefaultScheme(this.slider)
-				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
-		// Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
-		// .getDarkColor();
-		// Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
-		// .getUltraLightColor();
+		ColorScheme colorScheme = SubstanceThemeUtilities.getTheme(
+				this.slider,
+				this.slider.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED).getColorScheme();
 
 		Graphics2D graphics = (Graphics2D) g.create();
 		graphics.translate(0, y);
@@ -886,25 +850,6 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 				JSeparator.HORIZONTAL, true, this.slider
 						.getComponentOrientation().isLeftToRight() ? 0 : 4,
 				this.slider.getComponentOrientation().isLeftToRight() ? 4 : 0);
-		// if (this.slider.getComponentOrientation().isLeftToRight()) {
-		// graphics.setPaint(new GradientPaint(0, y, color,
-		// tickBounds.width - 1, y, SubstanceColorUtilities
-		// .getAlphaColor(color, 96)));
-		// } else {
-		// graphics.setPaint(new GradientPaint(0, y, SubstanceColorUtilities
-		// .getAlphaColor(color, 96), tickBounds.width - 1, y, color));
-		// }
-		// graphics.drawLine(0, y, tickBounds.width - 1, y);
-		// if (this.slider.getComponentOrientation().isLeftToRight()) {
-		// graphics.setPaint(new GradientPaint(0, y + 1, icolor,
-		// tickBounds.width - 1, y + 1, SubstanceColorUtilities
-		// .getAlphaColor(icolor, 96)));
-		// } else {
-		// graphics.setPaint(new GradientPaint(0, y + 1,
-		// SubstanceColorUtilities.getAlphaColor(icolor, 96),
-		// tickBounds.width - 1, y + 1, icolor));
-		// }
-		// graphics.drawLine(0, y + 1, tickBounds.width - 1, y + 1);
 		graphics.dispose();
 	}
 
@@ -919,27 +864,16 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 			Rectangle tickBounds, int x) {
 		boolean isDark = SubstanceCoreUtilities
 				.isThemeDark(SubstanceLookAndFeel.getTheme());
-		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
-				.getDefaultScheme(this.slider)
-				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
-		// Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
-		// .getDarkColor();
-		// Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
-		// .getUltraLightColor();
+		ColorScheme colorScheme = SubstanceThemeUtilities.getTheme(
+				this.slider,
+				this.slider.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED).getColorScheme();
 
 		Graphics2D graphics = (Graphics2D) g.create();
 		graphics.translate(x - 1, 0);
 		SubstanceCoreUtilities.paintSeparator(this.slider, graphics,
 				colorScheme, isDark, 0, tickBounds.height / 2,
 				JSeparator.VERTICAL, true, 0, 4);
-		// graphics.setPaint(new GradientPaint(x - 1, 0, color, x - 1,
-		// tickBounds.height / 2, SubstanceColorUtilities.getAlphaColor(
-		// color, 96)));
-		// graphics.drawLine(x - 1, 0, x - 1, tickBounds.height / 2);
-		// graphics.setPaint(new GradientPaint(x, 0, icolor, x,
-		// tickBounds.height / 2, SubstanceColorUtilities.getAlphaColor(
-		// icolor, 96)));
-		// graphics.drawLine(x, 0, x, tickBounds.height / 2);
 		graphics.dispose();
 	}
 
@@ -954,13 +888,10 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 			Rectangle tickBounds, int y) {
 		boolean isDark = SubstanceCoreUtilities
 				.isThemeDark(SubstanceLookAndFeel.getTheme());
-		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
-				.getDefaultScheme(this.slider)
-				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
-		// Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
-		// .getDarkColor();
-		// Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
-		// .getUltraLightColor();
+		ColorScheme colorScheme = SubstanceThemeUtilities.getTheme(
+				this.slider,
+				this.slider.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED).getColorScheme();
 
 		Graphics2D graphics = (Graphics2D) g.create();
 		graphics.translate(0, y);
@@ -969,28 +900,6 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 				JSeparator.HORIZONTAL, true, this.slider
 						.getComponentOrientation().isLeftToRight() ? 0 : 4,
 				this.slider.getComponentOrientation().isLeftToRight() ? 4 : 0);
-		// if (this.slider.getComponentOrientation().isLeftToRight()) {
-		// graphics.setPaint(new GradientPaint(0, y, color,
-		// tickBounds.width / 2, y, SubstanceColorUtilities
-		// .getAlphaColor(color, 96)));
-		// graphics.drawLine(0, y, tickBounds.width / 2, y);
-		// } else {
-		// graphics.setPaint(new GradientPaint(1, y, SubstanceColorUtilities
-		// .getAlphaColor(color, 96), tickBounds.width / 2 + 1, y,
-		// color));
-		// graphics.drawLine(1, y, tickBounds.width / 2 + 1, y);
-		// }
-		// if (this.slider.getComponentOrientation().isLeftToRight()) {
-		// graphics.setPaint(new GradientPaint(0, y + 1, icolor,
-		// tickBounds.width / 2, y + 1, SubstanceColorUtilities
-		// .getAlphaColor(icolor, 96)));
-		// graphics.drawLine(0, y + 1, tickBounds.width / 2, y + 1);
-		// } else {
-		// graphics.setPaint(new GradientPaint(1, y + 1,
-		// SubstanceColorUtilities.getAlphaColor(icolor, 96),
-		// tickBounds.width / 2 + 1, y + 1, icolor));
-		// graphics.drawLine(1, y + 1, tickBounds.width / 2 + 1, y + 1);
-		// }
 		graphics.dispose();
 	}
 
@@ -1040,22 +949,21 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	@Override
 	protected void calculateLabelRect() {
 		super.calculateLabelRect();
-		if (this.slider.getOrientation() == JSlider.HORIZONTAL) {
-			if (!this.slider.getPaintTicks()) {
-				// labelRect.y += 1;
-			}
-		} else {
-			if (!this.slider.getPaintTicks()) {
-				if (this.slider.getComponentOrientation().isLeftToRight()) {
-					this.labelRect.x += 3;
-				} else {
-					// labelRect.x -= 2;
-				}
-
-			}
+		if ((this.slider.getOrientation() == JSlider.VERTICAL)
+				&& !this.slider.getPaintTicks()
+				&& this.slider.getComponentOrientation().isLeftToRight()) {
+			this.labelRect.x += 3;
+		}
+		if (this.slider.getOrientation() == JSlider.VERTICAL) {
+			this.labelRect.width = getHeightOfTallestLabel();
 		}
 	}
 
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.basic.BasicSliderUI#calculateThumbLocation()
+	 */
 	@Override
 	protected void calculateThumbLocation() {
 		super.calculateThumbLocation();
@@ -1077,30 +985,6 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 		}
 	}
 
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see javax.swing.plaf.basic.BasicSliderUI#paintHorizontalLabel(java.awt.Graphics,
-	 *      int, java.awt.Component)
-	 */
-	@Override
-	protected void paintHorizontalLabel(Graphics g, int value, Component label) {
-		label.setEnabled(this.slider.isEnabled());
-		super.paintHorizontalLabel(g, value, label);
-	}
-
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see javax.swing.plaf.basic.BasicSliderUI#paintVerticalLabel(java.awt.Graphics,
-	 *      int, java.awt.Component)
-	 */
-	@Override
-	protected void paintVerticalLabel(Graphics g, int value, Component label) {
-		label.setEnabled(this.slider.isEnabled());
-		super.paintVerticalLabel(g, value, label);
-	}
-
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -1114,22 +998,17 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 			d = new Dimension(this.getPreferredVerticalSize());
 			d.width = this.insetCache.left + this.insetCache.right;
 			d.width += this.focusInsets.left + this.focusInsets.right;
-			d.width += this.trackRect.width; /* + tickRect.width */
-			// + this.labelRect.width;
-			// d.width -= 6;
+			d.width += this.trackRect.width;
 			if (this.slider.getPaintTicks())
 				d.width += getTickLength();
 			if (this.slider.getPaintLabels())
 				d.width += getWidthOfWidestLabel();
 			d.width += 3;
-			// d.width += 6;
 		} else {
 			d = new Dimension(this.getPreferredHorizontalSize());
 			d.height = this.insetCache.top + this.insetCache.bottom;
 			d.height += this.focusInsets.top + this.focusInsets.bottom;
-			d.height += this.trackRect.height;/* + tickRect.height */
-			// + this.labelRect.height;
-			// d.height -= 6;
+			d.height += this.trackRect.height;
 			if (this.slider.getPaintTicks())
 				d.height += getTickLength();
 			if (this.slider.getPaintLabels())
@@ -1174,4 +1053,95 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 				SubstanceSizeUtils.getComponentFontSize(this.slider), 200, 1,
 				20, false));
 	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.basic.BasicSliderUI#paintLabels(java.awt.Graphics)
+	 */
+	@Override
+	public void paintLabels(Graphics g) {
+		final Rectangle labelBounds = labelRect;
+
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		Dictionary<?, ?> dictionary = slider.getLabelTable();
+		if (dictionary != null) {
+			Enumeration<?> keys = dictionary.keys();
+			int minValue = slider.getMinimum();
+			int maxValue = slider.getMaximum();
+			while (keys.hasMoreElements()) {
+				Integer key = (Integer) keys.nextElement();
+				int value = key.intValue();
+				if (value >= minValue && value <= maxValue) {
+					Component label = (Component) dictionary.get(key);
+					Dimension prefSize = label.getPreferredSize();
+					JLabel jlabel = (JLabel) label;
+					final View v = (View) jlabel
+							.getClientProperty(BasicHTML.propertyKey);
+					String text = jlabel.getText();
+
+					// fix for issue 316 - respecting custom foreground
+					// colors on slider labels.
+					ComponentState state = slider.isEnabled() ? ComponentState.DEFAULT
+							: ComponentState.DISABLED_UNSELECTED;
+					Color labelColor = SubstanceCoreUtilities
+							.getForegroundColor(jlabel, state, state);
+
+					if (slider.getOrientation() == JSlider.HORIZONTAL) {
+						final Rectangle textRect = new Rectangle(
+								xPositionForValue(value) - prefSize.width / 2,
+								labelBounds.y, prefSize.width, prefSize.height);
+						if (v != null) {
+							// fix for issue 327 - support for custom HTML
+							// labels
+							if (textPainter.needsBackgroundImage()) {
+								textPainter
+										.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+											public void paintBackground(
+													Graphics g) {
+												v.paint(g, textRect);
+											}
+										});
+							} else {
+								v.paint(g, textRect);
+							}
+						} else {
+							textPainter.attachText(this.slider, textRect, text,
+									-1, g.getFont(), labelColor, g
+											.getClipBounds());
+						}
+					} else {
+						int offset = 0;
+						if (slider.getComponentOrientation().isLeftToRight()) {
+							offset = labelBounds.width
+									- label.getPreferredSize().height;
+						}
+						final Rectangle labelRect = new Rectangle(labelBounds.x
+								+ offset, yPositionForValue(value)
+								- prefSize.width / 2, prefSize.height,
+								prefSize.width);
+						// System.out.println(labelRect + ":" + text);
+						// textPainter
+						// .attachCallback(new
+						// SubstanceTextPainter.BackgroundPaintingCallback() {
+						// public void paintBackground(Graphics g) {
+						// g.setColor(Color.blue);
+						// g.drawRect(labelRect.x, labelRect.y,
+						// labelRect.width - 1,
+						// labelRect.height - 1);
+						// }
+						// });
+						textPainter.attachVerticalText(this.slider, labelRect,
+								text, -1, g.getFont(), labelColor, null, slider
+										.getComponentOrientation()
+										.isLeftToRight());
+					}
+				}
+			}
+		}
+		// textPainter.setBackgroundFill(slider, slider.getBackground(), true,
+		// 0,
+		// 0);
+	}
 }
