@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -30,6 +30,8 @@
 package org.jvnet.substance;
 
 import java.awt.*;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.util.*;
 
 import javax.swing.*;
@@ -39,9 +41,10 @@ import javax.swing.plaf.*;
 import javax.swing.plaf.basic.BasicTableHeaderUI;
 import javax.swing.table.*;
 
-import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
+import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.*;
 
@@ -51,11 +54,6 @@ import org.jvnet.substance.utils.*;
  * @author Kirill Grouchnikov
  */
 public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
-	/**
-	 * Delegate for painting the background.
-	 */
-	private static SubstanceGradientBackgroundDelegate backgroundDelegate = new SubstanceGradientBackgroundDelegate();
-
 	/**
 	 * Repaints the header on column selection.
 	 */
@@ -86,6 +84,8 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 	 */
 	private Map<Integer, ComponentState> nextStateMap;
 
+	protected PropertyChangeListener substancePropertyChangeListener;
+
 	/**
 	 * Listener for table header.
 	 * 
@@ -158,7 +158,7 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 		}
 
 		// Add listener for the selection animation
-		substanceFadeSelectionListener = new ListSelectionListener() {
+		this.substanceFadeSelectionListener = new ListSelectionListener() {
 			protected void cancelFades(Set<Long> initiatedFadeSequences) {
 				FadeTracker fadeTrackerInstance = FadeTracker.getInstance();
 				for (long fadeId : initiatedFadeSequences) {
@@ -172,8 +172,13 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 					return;
 
 				// optimization on large tables and large selections
-				if (LafWidgetUtilities.hasNoFades(header.getTable(),
-						FadeKind.SELECTION))
+				// and syncing the fade presence with the table
+				// (issue 309)
+				SubstanceTableUI tableUI = (SubstanceTableUI) header.getTable()
+						.getUI();
+				// System.out.println("Sel anim: "
+				// + tableUI.hasSelectionAnimations());
+				if (!tableUI.hasSelectionAnimations())
 					return;
 
 				Set<Long> initiatedFadeSequences = new HashSet<Long>();
@@ -249,6 +254,24 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 				lsm.addListSelectionListener(substanceFadeSelectionListener);
 			}
 		}
+
+		this.substancePropertyChangeListener = new PropertyChangeListener() {
+			public void propertyChange(PropertyChangeEvent evt) {
+				if ("table".equals(evt.getPropertyName())) {
+					// track changes to the table and re-register the
+					// column model listener to the new table.
+					TableColumnModel oldModel = (evt.getOldValue() instanceof JTable) ? ((JTable) evt
+							.getOldValue()).getColumnModel()
+							: null;
+					TableColumnModel newModel = (evt.getNewValue() instanceof JTable) ? ((JTable) evt
+							.getNewValue()).getColumnModel()
+							: null;
+					processColumnModelChangeEvent(oldModel, newModel);
+				}
+			}
+		};
+		this.header
+				.addPropertyChangeListener(this.substancePropertyChangeListener);
 	}
 
 	/*
@@ -291,6 +314,10 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 			}
 		}
 
+		this.header
+				.removePropertyChangeListener(this.substancePropertyChangeListener);
+		this.substancePropertyChangeListener = null;
+
 		super.uninstallListeners();
 	}
 
@@ -377,6 +404,8 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 			}
 		}
 
+		this.paintGrid(g, c);
+
 		// Paint the dragged column if we are dragging.
 		if (draggedColumn != null) {
 			int draggedColumnIndex = viewIndexForColumn(draggedColumn);
@@ -420,6 +449,123 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 				aColumn.getHeaderValue(), false, false, -1, columnIndex);
 	}
 
+	public void paintGrid(Graphics g, JComponent c) {
+		boolean ltr = header.getComponentOrientation().isLeftToRight();
+
+		Graphics2D g2d = (Graphics2D) g.create();
+
+		Rectangle clip = g.getClipBounds();
+		Point left = clip.getLocation();
+		Point right = new Point(clip.x + clip.width - 1, clip.y);
+
+		TableColumnModel cm = header.getColumnModel();
+
+		int cMin = header.columnAtPoint(ltr ? left : right);
+		int cMax = header.columnAtPoint(ltr ? right : left);
+		// This should never happen.
+		if (cMin == -1) {
+			cMin = 0;
+		}
+
+		Rectangle cellRect0 = header.getHeaderRect(cMin);
+		int top = cellRect0.y;
+		int bottom = cellRect0.y + cellRect0.height;
+
+		Color gridColor = getGridColor(this.header);
+
+		float strokeWidth = SubstanceSizeUtils
+				.getBorderStrokeWidth(SubstanceSizeUtils
+						.getComponentFontSize(this.header));
+		g2d.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_ROUND,
+				BasicStroke.JOIN_BEVEL));
+		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
+				RenderingHints.VALUE_ANTIALIAS_ON);
+		g2d.setColor(gridColor);
+		g2d.setComposite(TransitionLayout.getAlphaComposite(this.header, 0.7f,
+				g));
+
+		g2d.drawLine((int) left.getX(), (int) (bottom - strokeWidth / 2),
+				(int) right.getX(), (int) (bottom - strokeWidth / 2));
+		// g2d.drawLine((int) left.getX(), top + (int) (strokeWidth / 2),
+		// (int) right.getX(), top + (int) (strokeWidth / 2));
+
+		// If the table does not have enough columns to fill the view we'll
+		// get
+		// -1.
+		// Replace this with the index of the last column.
+		if (cMax == -1) {
+			cMax = cm.getColumnCount() - 1;
+		}
+
+		TableColumn draggedColumn = header.getDraggedColumn();
+		int columnWidth;
+		Rectangle cellRect = header.getHeaderRect(ltr ? cMin : cMax);
+		TableColumn aColumn;
+		if (ltr) {
+			for (int column = cMin; column <= cMax; column++) {
+				aColumn = cm.getColumn(column);
+				columnWidth = aColumn.getWidth();
+				cellRect.width = columnWidth;
+				if (aColumn != draggedColumn) {
+					g2d.drawLine(cellRect.x - 1, cellRect.y, cellRect.x - 1,
+							bottom);
+					// if (column == cMax) {
+					// g2d.drawLine(cellRect.x + cellRect.width - 1,
+					// cellRect.y, cellRect.x + cellRect.width - 1,
+					// bottom);
+					// }
+				}
+				cellRect.x += columnWidth;
+			}
+		} else {
+			for (int column = cMax; column >= cMin; column--) {
+				aColumn = cm.getColumn(column);
+				columnWidth = aColumn.getWidth();
+				cellRect.width = columnWidth;
+				if (aColumn != draggedColumn) {
+					g2d.drawLine(cellRect.x - 1, cellRect.y, cellRect.x - 1,
+							bottom);
+					// if (column == cMin) {
+					// g2d.drawLine(cellRect.x + cellRect.width - 1,
+					// cellRect.y, cellRect.x + cellRect.width - 1,
+					// bottom);
+					// }
+				}
+				cellRect.x += columnWidth;
+			}
+		}
+
+		g2d.dispose();
+	}
+
+	protected static Color getGridColor(JTableHeader header) {
+		ComponentState currState = (header.isEnabled() && header.getTable()
+				.isEnabled()) ? ComponentState.DEFAULT
+				: ComponentState.DISABLED_UNSELECTED;
+		SubstanceTheme currTheme = SubstanceThemeUtilities.getHighlightTheme(
+				header, currState);
+		Color gridColor = currTheme.getBorderTheme().getColorScheme()
+				.getDarkColor();
+
+		// // support for enhancement 256 - colorization of controls.
+		// boolean hasColorization = SubstanceCoreUtilities
+		// .hasColorization(header);
+		// if (hasColorization) {
+		// Color backgr = header.getBackground();
+		// if (!(backgr instanceof UIResource)) {
+		// double colorizationFactor = SubstanceCoreUtilities
+		// .getColorizationFactor(header);
+		// if (!header.isEnabled() || !header.getTable().isEnabled())
+		// colorizationFactor /= 2.0;
+		// Color toShiftTo = SubstanceColorUtilities.deriveByBrightness(
+		// backgr, gridColor);
+		// gridColor = SubstanceColorUtilities.getInterpolatedColor(
+		// toShiftTo, gridColor, colorizationFactor);
+		// }
+		// }
+		return gridColor;
+	}
+
 	/**
 	 * Paints cell.
 	 * 
@@ -438,29 +584,61 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 		g2d.setComposite(TransitionLayout.getAlphaComposite(header, g));
 
 		// paint default background
-		Component component = getHeaderRenderer(columnIndex);
-		SubstanceTableHeaderUI.backgroundDelegate.update(g, component,
-				cellRect, SubstanceCoreUtilities.getDefaultTheme(header, true),
-				0.5f);
+		final Component component = getHeaderRenderer(columnIndex);
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		final ComponentState prevState = getPrevColumnState(columnIndex);
+		final ComponentState currState = getColumnState(columnIndex);
 
-		ComponentState prevState = getPrevColumnState(columnIndex);
-		ComponentState currState = getColumnState(columnIndex);
-		float alphaForPrevBackground = 0.0f;
+		final ComponentState backgroundState = (this.header.isEnabled() && this.header
+				.getTable().isEnabled()) ? ComponentState.DEFAULT
+				: ComponentState.DISABLED_UNSELECTED;
+		final SubstanceTheme theme = SubstanceThemeUtilities.getTheme(
+				component, backgroundState);
+		if (textPainter.needsBackgroundImage()) {
+			final Rectangle offsetCellRect = new Rectangle(0, 0,
+					cellRect.width, cellRect.height);
+			textPainter.init(header, offsetCellRect, true);
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							Graphics2D g2d = (Graphics2D) g.create();
+							SubstanceHighlightUtils.paintHighlight(g2d,
+									component, offsetCellRect, 0.0f, null,
+									theme.getColorScheme(), theme
+											.getColorScheme(), 0.0f);
+							// SubstanceHighlightUtils.paintHighlight(g2d,
+							// component, backgroundState,
+							// backgroundState, offsetCellRect, 0.0f);
+							g2d.dispose();
+						}
+					});
+		} else {
+			SubstanceHighlightUtils.paintHighlight(g2d, component, cellRect,
+					0.0f, null, theme.getColorScheme(), theme.getColorScheme(),
+					0.0f);
+			// highlightDelegate.paintClassicHighlight(g, component, cellRect,
+			// SubstanceCoreUtilities.getTheme(header, backgroundState,
+			// true, true, true), 0.0f);
+		}
+
+		// float alphaForPrevBackground = 0.0f;
 
 		// Compute the alpha values for the animation.
-		float startAlpha = SubstanceCoreUtilities.getHighlightAlpha(header,
-				prevState, true);
-		float endAlpha = SubstanceCoreUtilities.getHighlightAlpha(header,
-				currState, true);
-		float alphaForCurrBackground = endAlpha;
+		float startAlpha = SubstanceThemeUtilities.getHighlightAlpha(header,
+				prevState);
+		float endAlpha = SubstanceThemeUtilities.getHighlightAlpha(header,
+				currState);
+		// float alphaForCurrBackground = endAlpha;
 
 		FadeState state = SubstanceFadeUtilities.getFadeState(header,
 				columnIndex, FadeKind.SELECTION, FadeKind.ROLLOVER);
+		float totalAlpha = endAlpha;
+		float fadeCoef = 0.0f;
 		if (state != null) {
-			float fadeCoef = state.getFadePosition();
+			fadeCoef = state.getFadePosition();
 
 			// compute the total alpha of the overlays.
-			float totalAlpha = 0.0f;
 			if (state.isFadingIn()) {
 				totalAlpha = startAlpha + (endAlpha - startAlpha) * fadeCoef
 						/ 10.0f;
@@ -473,37 +651,112 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 				fadeCoef = 10.0f - fadeCoef;
 
 			// compute the alpha for each one of the animation overlays
-			alphaForPrevBackground = totalAlpha * fadeCoef / 10.0f;
-			alphaForCurrBackground = totalAlpha * (10.0f - fadeCoef) / 10.0f;
+			// alphaForPrevBackground = totalAlpha * fadeCoef / 10.0f;
+			// alphaForCurrBackground = totalAlpha * (10.0f - fadeCoef) / 10.0f;
 		}
 
-		SubstanceTheme prevTheme = SubstanceCoreUtilities.getHighlightTheme(
-				header, prevState, true, false);
-		SubstanceTheme currTheme = SubstanceCoreUtilities.getHighlightTheme(
-				header, currState, true, false);
+		final SubstanceTheme prevTheme = SubstanceThemeUtilities
+				.getHighlightTheme(header, prevState);
+		final SubstanceTheme currTheme = SubstanceThemeUtilities
+				.getHighlightTheme(header, currState);
 
 		// System.out.println(row + ":" + prevTheme.getDisplayName() + "["
 		// + alphaForPrevBackground + "]:" + currTheme.getDisplayName()
 		// + "[" + alphaForCurrBackground + "]");
 
-		if (alphaForPrevBackground > 0.0f) {
-			g2d.setComposite(TransitionLayout.getAlphaComposite(header,
-					alphaForPrevBackground, g));
-			SubstanceTableHeaderUI.backgroundDelegate.update(g2d, component,
-					cellRect, prevTheme, 0.5f);
-			g2d.setComposite(TransitionLayout.getAlphaComposite(header, g));
-		}
-		if (alphaForCurrBackground > 0.0f) {
-			g2d.setComposite(TransitionLayout.getAlphaComposite(header,
-					alphaForCurrBackground, g));
-			SubstanceTableHeaderUI.backgroundDelegate.update(g2d, component,
-					cellRect, currTheme, 0.5f);
-			g2d.setComposite(TransitionLayout.getAlphaComposite(header, g));
+		if (textPainter.needsBackgroundImage()) {
+			// final float finalPrevAlpha = alphaForPrevBackground;
+			// final float finalCurrAlpha = alphaForCurrBackground;
+			final float finalTotalAlpha = totalAlpha;
+			final float finalFadeCoef = fadeCoef;
+
+			final Rectangle offsetCellRect = new Rectangle(0, 0,
+					cellRect.width, cellRect.height);
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							Graphics2D g2d = (Graphics2D) g.create();
+							if (finalTotalAlpha > 0.0f) {
+								g2d.setComposite(TransitionLayout
+										.getAlphaComposite(header,
+												finalTotalAlpha, g));
+								SubstanceHighlightUtils.paintHighlight(g2d,
+										component, offsetCellRect, 0.0f, null,
+										currTheme.getColorScheme(), prevTheme
+												.getColorScheme(),
+										finalFadeCoef);
+								g2d.setComposite(TransitionLayout
+										.getAlphaComposite(header, g));
+							}
+							// if (finalPrevAlpha > 0.0f) {
+							// g2d.setComposite(TransitionLayout
+							// .getAlphaComposite(header,
+							// finalPrevAlpha, g));
+							// SubstanceHighlightUtils.paintHighlight(g2d,
+							// component, prevState, prevState,
+							// offsetCellRect, 0.0f);
+							// // SubstanceTableHeaderUI.highlightDelegate
+							// // .paintClassicHighlight(g2d, component,
+							// // offsetCellRect, prevTheme, 0.0f);
+							// g2d.setComposite(TransitionLayout
+							// .getAlphaComposite(header, g));
+							// }
+							// if (finalCurrAlpha > 0.0f) {
+							// g2d.setComposite(TransitionLayout
+							// .getAlphaComposite(header,
+							// finalCurrAlpha, g));
+							// SubstanceHighlightUtils.paintHighlight(g2d,
+							// component, currState, currState,
+							// offsetCellRect, 0.0f);
+							// // SubstanceTableHeaderUI.highlightDelegate
+							// // .paintClassicHighlight(g2d, component,
+							// // offsetCellRect, currTheme, 0.0f);
+							// g2d.setComposite(TransitionLayout
+							// .getAlphaComposite(header, g));
+							// }
+							g2d.dispose();
+						}
+					});
+		} else {
+			if (totalAlpha > 0.0f) {
+				g2d.setComposite(TransitionLayout.getAlphaComposite(
+						this.header, totalAlpha, g));
+				SubstanceHighlightUtils.paintHighlight(g2d, component,
+						cellRect, 0.0f, null, currTheme.getColorScheme(),
+						prevTheme.getColorScheme(), fadeCoef);
+				g2d.setComposite(TransitionLayout.getAlphaComposite(
+						this.header, g));
+			}
+
+			// if (alphaForPrevBackground > 0.0f) {
+			// g2d.setComposite(TransitionLayout.getAlphaComposite(header,
+			// alphaForPrevBackground, g));
+			// SubstanceHighlightUtils.paintHighlight(g2d, component,
+			// prevState, prevState, cellRect, 0.0f);
+			// //
+			// SubstanceTableHeaderUI.highlightDelegate.paintClassicHighlight(
+			// // g2d, component, cellRect, prevTheme, 0.0f);
+			// g2d.setComposite(TransitionLayout.getAlphaComposite(header, g));
+			// }
+			// if (alphaForCurrBackground > 0.0f) {
+			// g2d.setComposite(TransitionLayout.getAlphaComposite(header,
+			// alphaForCurrBackground, g));
+			// SubstanceHighlightUtils.paintHighlight(g2d, component,
+			// currState, currState, cellRect, 0.0f);
+			// //
+			// SubstanceTableHeaderUI.highlightDelegate.paintClassicHighlight(
+			// // g2d, component, cellRect, currTheme, 0.0f);
+			// g2d.setComposite(TransitionLayout.getAlphaComposite(header, g));
+			// }
 		}
 
+		this.header.putClientProperty(
+				SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, Boolean.TRUE);
 		rendererPane.paintComponent(g2d, component, header, cellRect.x,
 				cellRect.y, cellRect.width, cellRect.height, true);
 
+		this.header.putClientProperty(
+				SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, null);
 		g2d.dispose();
 		//
 		// Component component = getHeaderRenderer(columnIndex);
@@ -543,10 +796,20 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 	public void update(Graphics g, JComponent c) {
 		// fix for issue 175 - table header under resize mode off
 		// was painted in theme-agnostic (gray) color.
-		SubstanceTheme theme = SubstanceCoreUtilities.getDefaultTheme(header,
-				true);
-		SubstanceTableHeaderUI.backgroundDelegate.update(g, c, c.getBounds(),
-				theme/* .getHighlightBackgroundTheme() */, 0.5f);
+		final ComponentState backgroundState = (this.header.isEnabled() && this.header
+				.getTable().isEnabled()) ? ComponentState.DEFAULT
+				: ComponentState.DISABLED_UNSELECTED;
+		// SubstanceTheme theme = SubstanceCoreUtilities.getTheme(c,
+		// backgroundState, true, true, true);
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(c,
+				backgroundState);
+
+		SubstanceHighlightUtils.paintHighlight(g, c, c.getBounds(), 0.0f, null,
+				theme.getColorScheme(), theme.getColorScheme(), 0.0f);
+
+		// SubstanceHighlightUtils.paintHighlight(g, c, backgroundState,
+		// backgroundState, c.getBounds(), 0.0f);
+		// paintClassicHighlight(g, c, c.getBounds(), theme, 0.0f);
 		paint(g, c);
 	}
 
@@ -570,6 +833,150 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 		super.uninstallUI(c);
 	}
 
+	/**
+	 * Returns the previous state for the specified column.
+	 * 
+	 * @param columnIndex
+	 *            Column index.
+	 * @return The previous state for the specified column.
+	 */
+	public ComponentState getPrevColumnState(int columnIndex) {
+		if (this.header.isEnabled()
+				&& this.header.getTable().isEnabled()
+				&& SubstanceDefaultTableHeaderCellRenderer.isColumnSorted(
+						this.header.getTable(), columnIndex))
+			return ComponentState.SELECTED;
+
+		if (prevStateMap.containsKey(columnIndex))
+			return prevStateMap.get(columnIndex);
+		return ComponentState.DEFAULT;
+	}
+
+	/**
+	 * Returns the current state for the specified cell.
+	 * 
+	 * @param columnIndex
+	 *            Column index.
+	 * @return The current state for the specified column.
+	 */
+	public ComponentState getColumnState(int columnIndex) {
+		ButtonModel synthModel = new DefaultButtonModel();
+		boolean toEnable = header.isEnabled();
+
+		// get the rollover column index from the table UI delegate
+		JTable table = this.header.getTable();
+		if (table != null) {
+			toEnable = toEnable && table.isEnabled();
+			TableUI ui = table.getUI();
+			if (ui instanceof SubstanceTableUI) {
+				SubstanceTableUI tableUI = (SubstanceTableUI) table.getUI();
+				int rolledOverIndex = tableUI.getRolloverColumnIndex();
+				synthModel.setRollover((rolledOverIndex >= 0)
+						&& (rolledOverIndex == columnIndex));
+			}
+		}
+
+		if (toEnable
+				&& SubstanceDefaultTableHeaderCellRenderer.isColumnSorted(
+						this.header.getTable(), columnIndex))
+			return ComponentState.SELECTED;
+
+		synthModel.setEnabled(toEnable);
+
+		// set selection on the model.
+		TableColumnModel columnModel = header.getColumnModel();
+		synthModel
+				.setSelected(columnModel.getColumnSelectionAllowed()
+						&& columnModel.getSelectionModel().isSelectedIndex(
+								columnIndex));
+		return ComponentState.getState(synthModel, null);
+	}
+
+	/**
+	 * Returns the animation callback for the specified column.
+	 * 
+	 * @param columnIndex
+	 *            Column index.
+	 * @return Animation callback for the specified column.
+	 */
+	public FadeTrackerCallback getCallback(int columnIndex) {
+		return new ColumnHeaderRepaintCallback(this.header, columnIndex);
+	}
+
+	public JComponent getScrollPaneCornerFiller() {
+		return new ScrollPaneCornerFiller(this.header);
+	}
+
+	protected static class ScrollPaneCornerFiller extends JComponent implements
+			UIResource {
+		protected JTableHeader header;
+
+		public ScrollPaneCornerFiller(JTableHeader header) {
+			this.header = header;
+			// if (this.header == null) {
+			// System.err
+			// .println("************** Header is null ******************");
+			// }
+		}
+
+		@Override
+		protected void paintComponent(Graphics g) {
+			Graphics2D g2d = (Graphics2D) g.create();
+			// System.err.println("Painting " + this.hashCode() + " from "
+			// + ((header == null) ? "null" : header.hashCode()));
+
+			boolean ltr = header.getComponentOrientation().isLeftToRight();
+			final ComponentState backgroundState = (header.isEnabled() && header
+					.getTable().isEnabled()) ? ComponentState.DEFAULT
+					: ComponentState.DISABLED_UNSELECTED;
+			SubstanceTheme theme = SubstanceThemeUtilities.getTheme(header,
+					backgroundState);
+			SubstanceHighlightUtils.paintHighlight(g2d, this.header,
+					new Rectangle(0, 0, this.getWidth(), this.getHeight()),
+					0.0f, null, theme.getColorScheme(), theme.getColorScheme(),
+					0.0f);
+
+			// SubstanceHighlightUtils.paintHighlight(g2d, header,
+			// backgroundState, backgroundState, new Rectangle(0, 0, this
+			// .getWidth(), this.getHeight()), 0.0f);
+			// paintClassicHighlight(g2d,
+			// header, new Rectangle(0, 0, this.getWidth(), this
+			// .getHeight()), theme, 0.0f);
+
+			g2d.setColor(getGridColor(this.header));
+			float strokeWidth = SubstanceSizeUtils
+					.getBorderStrokeWidth(SubstanceSizeUtils
+							.getComponentFontSize(header));
+			g2d.setStroke(new BasicStroke(strokeWidth, BasicStroke.CAP_ROUND,
+					BasicStroke.JOIN_BEVEL));
+			g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
+					RenderingHints.VALUE_ANTIALIAS_ON);
+			g2d.setComposite(TransitionLayout.getAlphaComposite(this.header,
+					0.7f, g));
+
+			int x = ltr ? (int) strokeWidth / 2 : getWidth() - 1
+					- (int) strokeWidth / 2;
+			g2d.drawLine(x, 0, x, getHeight());
+
+			g2d.dispose();
+		}
+	}
+
+	public void processColumnModelChangeEvent(TableColumnModel oldModel,
+			TableColumnModel newModel) {
+		if (oldModel != null) {
+			oldModel.getSelectionModel().removeListSelectionListener(
+					substanceFadeSelectionListener);
+		}
+		if (newModel != null) {
+			newModel.getSelectionModel().addListSelectionListener(
+					substanceFadeSelectionListener);
+		}
+		selectedIndices.clear();
+		prevStateMap.clear();
+		nextStateMap.clear();
+	}
+
 	/**
 	 * Repaints a single column header during the fade animation cycle.
 	 * 
@@ -678,7 +1085,9 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 							// need to retrieve the cell rectangle since the
 							// cells can be moved while animating
 							Rectangle rect = header.getHeaderRect(columnIndex);
-							header.repaint(rect);
+							Rectangle damaged = new Rectangle(rect.x - 5,
+									rect.y, rect.width + 10, rect.height);
+							header.repaint(damaged);
 						}
 					} catch (RuntimeException re) {
 						return;
@@ -687,57 +1096,4 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 			});
 		}
 	}
-
-	/**
-	 * Returns the previous state for the specified column.
-	 * 
-	 * @param columnIndex
-	 *            Column index.
-	 * @return The previous state for the specified column.
-	 */
-	public ComponentState getPrevColumnState(int columnIndex) {
-		if (prevStateMap.containsKey(columnIndex))
-			return prevStateMap.get(columnIndex);
-		return ComponentState.DEFAULT;
-	}
-
-	/**
-	 * Returns the current state for the specified cell.
-	 * 
-	 * @param columnIndex
-	 *            Column index.
-	 * @return The current state for the specified column.
-	 */
-	public ComponentState getColumnState(int columnIndex) {
-		ButtonModel synthModel = new DefaultButtonModel();
-		synthModel.setEnabled(header.isEnabled());
-
-		// get the rollover column index from the table UI delegate
-		JTable table = this.header.getTable();
-		if (table != null) {
-			TableUI ui = table.getUI();
-			if (ui instanceof SubstanceTableUI) {
-				SubstanceTableUI tableUI = (SubstanceTableUI) table.getUI();
-				int rolledOverIndex = tableUI.getRolloverColumnIndex();
-				synthModel.setRollover((rolledOverIndex >= 0)
-						&& (rolledOverIndex == columnIndex));
-			}
-		}
-
-		// set selection on the model.
-		synthModel.setSelected(header.getColumnModel().getSelectionModel()
-				.isSelectedIndex(columnIndex));
-		return ComponentState.getState(synthModel, null);
-	}
-
-	/**
-	 * Returns the animation callback for the specified column.
-	 * 
-	 * @param columnIndex
-	 *            Column index.
-	 * @return Animation callback for the specified column.
-	 */
-	public FadeTrackerCallback getCallback(int columnIndex) {
-		return new ColumnHeaderRepaintCallback(this.header, columnIndex);
-	}
 }
