@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -36,15 +36,17 @@ import java.beans.PropertyChangeListener;
 import java.util.*;
 
 import javax.swing.*;
-import javax.swing.event.ListSelectionEvent;
-import javax.swing.event.ListSelectionListener;
+import javax.swing.event.*;
 import javax.swing.plaf.ComponentUI;
+import javax.swing.plaf.UIResource;
 import javax.swing.plaf.basic.BasicTableUI;
 import javax.swing.table.*;
 
 import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
+import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.*;
 
@@ -90,25 +92,25 @@ public class SubstanceTableUI extends BasicTableUI {
 	protected Map<Class<?>, TableCellRenderer> defaultRenderers;
 
 	/**
-	 * Listener that listens to changes on
-	 * {@link SubstanceLookAndFeel#WATERMARK_TO_BLEED} property.
+	 * Listener that listens to changes on table properties.
 	 */
 	protected PropertyChangeListener substancePropertyChangeListener;
 
 	/**
 	 * Listener for fade animations on list selections.
 	 */
-	protected ListSelectionListener substanceFadeSelectionListener;
+	protected TableStateListener substanceFadeSelectionListener;
 
 	/**
 	 * Listener for fade animations on table rollovers.
 	 */
 	protected RolloverFadeListener substanceFadeRolloverListener;
 
-	/**
-	 * Delegate for painting the background.
-	 */
-	private static SubstanceGradientBackgroundDelegate backgroundDelegate = new SubstanceGradientBackgroundDelegate();
+	// /**
+	// * Delegate for painting the background of list cells.
+	// */
+	// private static SubstanceFillBackgroundDelegate backgroundDelegate = new
+	// SubstanceFillBackgroundDelegate();
 
 	/**
 	 * Map of previous fade states (for state-aware theme transitions).
@@ -247,7 +249,8 @@ public class SubstanceTableUI extends BasicTableUI {
 		}
 		if (areAllRenderersFromSubstance) {
 			Insets rendererInsets = SubstanceSizeUtils
-					.getTableCellRendererInsets(table.getFont().getSize());
+					.getTableCellRendererInsets(SubstanceSizeUtils
+							.getComponentFontSize(table));
 			JLabel dummy = new JLabel("dummy");
 			dummy.setFont(table.getFont());
 			int rowHeight = dummy.getPreferredSize().height
@@ -255,10 +258,18 @@ public class SubstanceTableUI extends BasicTableUI {
 			for (int i = 0; i < table.getRowCount(); i++) {
 				for (int j = 0; j < table.getColumnCount(); j++) {
 					TableCellRenderer renderer = table.getCellRenderer(i, j);
-					rowHeight = Math.max(rowHeight, renderer
-							.getTableCellRendererComponent(table,
-									table.getValueAt(i, j), false, false, i, j)
-							.getPreferredSize().height);
+					try {
+						rowHeight = Math.max(rowHeight, renderer
+								.getTableCellRendererComponent(table,
+										table.getValueAt(i, j), false, false,
+										i, j).getPreferredSize().height);
+					} catch (Throwable t) {
+						// Ignore - this happens when the renderer has a null
+						// font, such as in NetBeans palette table (in the
+						// getPreferredSize() call). Not much we can do since
+						// setting the font on the table or renderer will
+						// interfere with the application logic.
+					}
 				}
 			}
 			table.setRowHeight(rowHeight);
@@ -359,14 +370,22 @@ public class SubstanceTableUI extends BasicTableUI {
 
 				if ("columnSelectionAllowed".equals(evt.getPropertyName())
 						|| "rowSelectionAllowed".equals(evt.getPropertyName())) {
-					SubstanceTableUI.this.syncSelection(null);
+					SubstanceTableUI.this.syncSelection();
 				}
 
 				if ("model".equals(evt.getPropertyName())) {
+					TableModel old = (TableModel) evt.getOldValue();
+					if (old != null) {
+						old
+								.removeTableModelListener(substanceFadeSelectionListener);
+					}
+					// fix for defect 291 - track changes to the table.
+					table.getModel().addTableModelListener(
+							substanceFadeSelectionListener);
 					selectedIndices.clear();
 					prevStateMap.clear();
 					nextStateMap.clear();
-					SubstanceTableUI.this.syncSelection(null);
+					SubstanceTableUI.this.syncSelection();
 				}
 
 				if ("columnModel".equals(evt.getPropertyName())) {
@@ -381,7 +400,15 @@ public class SubstanceTableUI extends BasicTableUI {
 					selectedIndices.clear();
 					prevStateMap.clear();
 					nextStateMap.clear();
-					SubstanceTableUI.this.syncSelection(null);
+					SubstanceTableUI.this.syncSelection();
+
+					// fix for issue 309 - syncing animations on tables
+					// and table headers.
+					SubstanceTableHeaderUI headerUI = (SubstanceTableHeaderUI) table
+							.getTableHeader().getUI();
+					headerUI.processColumnModelChangeEvent(
+							(TableColumnModel) evt.getOldValue(),
+							(TableColumnModel) evt.getNewValue());
 				}
 
 				// fix for defect 243 - not tracking changes to selection
@@ -399,28 +426,48 @@ public class SubstanceTableUI extends BasicTableUI {
 					selectedIndices.clear();
 					prevStateMap.clear();
 					nextStateMap.clear();
-					SubstanceTableUI.this.syncSelection(null);
+					SubstanceTableUI.this.syncSelection();
 				}
 
 				if ("font".equals(evt.getPropertyName())) {
 					SwingUtilities.invokeLater(new Runnable() {
 						public void run() {
+							// fix for bug 341
+							if (table == null)
+								return;
 							table.updateUI();
 						}
 					});
 				}
+
+				if ("background".equals(evt.getPropertyName())) {
+					// propagate application-specific background color to the
+					// scroll bars.
+					Color newBackgr = (Color) evt.getNewValue();
+					if (!(newBackgr instanceof UIResource)) {
+						JTableHeader header = table.getTableHeader();
+						if (header != null) {
+							if (header.getBackground() instanceof UIResource) {
+								header.setBackground(newBackgr);
+							}
+						}
+					}
+				}
+
 			}
 		};
 		this.table
 				.addPropertyChangeListener(this.substancePropertyChangeListener);
 
 		// Add listener for the selection animation
-		this.substanceFadeSelectionListener = new TableSelectionListener();
+		this.substanceFadeSelectionListener = new TableStateListener();
 		this.table.getSelectionModel().addListSelectionListener(
 				this.substanceFadeSelectionListener);
 		TableColumnModel columnModel = this.table.getColumnModel();
 		columnModel.getSelectionModel().addListSelectionListener(
 				this.substanceFadeSelectionListener);
+		this.table.getModel().addTableModelListener(
+				this.substanceFadeSelectionListener);
 
 		// Add listener for the fade animation
 		this.substanceFadeRolloverListener = new RolloverFadeListener();
@@ -444,6 +491,8 @@ public class SubstanceTableUI extends BasicTableUI {
 		this.table.getColumnModel().getSelectionModel()
 				.removeListSelectionListener(
 						this.substanceFadeSelectionListener);
+		this.table.getModel().removeTableModelListener(
+				this.substanceFadeSelectionListener);
 		this.substanceFadeSelectionListener = null;
 
 		// Remove listener for the fade animation
@@ -522,13 +571,44 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * lines if <code>getShowHorizontalLines()</code> returns true.
 	 */
 	private void paintGrid(Graphics g, int rMin, int rMax, int cMin, int cMax) {
-		g.setColor(this.table.getGridColor());
+		Graphics2D g2d = (Graphics2D) g.create();
+		ComponentState currState = this.table.isEnabled() ? ComponentState.DEFAULT
+				: ComponentState.DISABLED_UNSELECTED;
+		float alpha = SubstanceThemeUtilities.getTheme(this.table)
+				.getThemeAlpha(this.table, currState);
+		g2d.setComposite(TransitionLayout.getAlphaComposite(this.table, alpha,
+				g));
+
+		Color gridColor = this.table.getGridColor();
+		if (gridColor instanceof UIResource) {
+			SubstanceTheme theme = SubstanceThemeUtilities.getTheme(this.table,
+					this.table.isEnabled() ? ComponentState.DEFAULT
+							: ComponentState.DISABLED_UNSELECTED);
+			gridColor = theme.getLineColor();
+
+			// // support for enhancement 256 - colorization of controls.
+			// boolean hasColorization = SubstanceCoreUtilities
+			// .hasColorization(this.table);
+			// if (hasColorization) {
+			// Color backgr = this.table.getBackground();
+			// if (!(backgr instanceof UIResource)) {
+			// double colorizationFactor = SubstanceCoreUtilities
+			// .getColorizationFactor(this.table);
+			// if (!table.isEnabled())
+			// colorizationFactor /= 2.0;
+			// Color toShiftTo = SubstanceColorUtilities
+			// .deriveByBrightness(backgr, gridColor);
+			// gridColor = SubstanceColorUtilities.getInterpolatedColor(
+			// toShiftTo, gridColor, colorizationFactor);
+			// }
+			// }
+		}
+		g2d.setColor(gridColor);
 
 		Rectangle minCell = this.table.getCellRect(rMin, cMin, true);
 		Rectangle maxCell = this.table.getCellRect(rMax, cMax, true);
 		Rectangle damagedArea = minCell.union(maxCell);
 
-		Graphics2D g2d = (Graphics2D) g.create();
 		float strokeWidth = SubstanceSizeUtils
 				.getBorderStrokeWidth(SubstanceSizeUtils
 						.getComponentFontSize(this.table));
@@ -554,7 +634,9 @@ public class SubstanceTableUI extends BasicTableUI {
 				for (int column = cMin; column <= cMax; column++) {
 					int w = cm.getColumn(column).getWidth();
 					x += w;
-					g2d.drawLine(x - 1, 0, x - 1, tableHeight - 1);
+					if (column != (cm.getColumnCount() - 1)) {
+						g2d.drawLine(x - 1, 0, x - 1, tableHeight - 1);
+					}
 				}
 			} else {
 				x = damagedArea.x + damagedArea.width;
@@ -590,17 +672,27 @@ public class SubstanceTableUI extends BasicTableUI {
 		int columnMargin = cm.getColumnMargin();
 
 		Rectangle cellRect;
+		Rectangle highlightCellRect;
 		TableColumn aColumn;
 		int columnWidth;
 		if (this.table.getComponentOrientation().isLeftToRight()) {
 			for (int row = rMin; row <= rMax; row++) {
 				cellRect = this.table.getCellRect(row, cMin, false);
+				highlightCellRect = new Rectangle(cellRect);
+				highlightCellRect.y -= this.table.getRowMargin() / 2;
+				highlightCellRect.height += this.table.getRowMargin();
+
 				for (int column = cMin; column <= cMax; column++) {
 					aColumn = cm.getColumn(column);
 					columnWidth = aColumn.getWidth();
 					cellRect.width = columnWidth - columnMargin;
+
+					highlightCellRect.x = cellRect.x - columnMargin / 2;
+					highlightCellRect.width = columnWidth;
+
 					if (aColumn != draggedColumn) {
-						this.paintCell(g, cellRect, row, column);
+						this.paintCell(g, cellRect, highlightCellRect, row,
+								column);
 					}
 					cellRect.x += columnWidth;
 				}
@@ -608,19 +700,32 @@ public class SubstanceTableUI extends BasicTableUI {
 		} else {
 			for (int row = rMin; row <= rMax; row++) {
 				cellRect = this.table.getCellRect(row, cMin, false);
+				highlightCellRect = new Rectangle(cellRect);
+				highlightCellRect.y -= this.table.getRowMargin() / 2;
+				highlightCellRect.height += this.table.getRowMargin();
+
 				aColumn = cm.getColumn(cMin);
+
 				if (aColumn != draggedColumn) {
 					columnWidth = aColumn.getWidth();
 					cellRect.width = columnWidth - columnMargin;
-					this.paintCell(g, cellRect, row, cMin);
+
+					highlightCellRect.x = cellRect.x - columnMargin / 2;
+					highlightCellRect.width = columnWidth;
+
+					this.paintCell(g, cellRect, highlightCellRect, row, cMin);
 				}
 				for (int column = cMin + 1; column <= cMax; column++) {
 					aColumn = cm.getColumn(column);
 					columnWidth = aColumn.getWidth();
 					cellRect.width = columnWidth - columnMargin;
 					cellRect.x -= columnWidth;
+
+					highlightCellRect.x = cellRect.x - columnMargin / 2;
+					highlightCellRect.width = columnWidth;
 					if (aColumn != draggedColumn) {
-						this.paintCell(g, cellRect, row, column);
+						this.paintCell(g, cellRect, highlightCellRect, row,
+								column);
 					}
 				}
 			}
@@ -628,8 +733,13 @@ public class SubstanceTableUI extends BasicTableUI {
 
 		// Paint the dragged column if we are dragging.
 		if (draggedColumn != null) {
-			this.paintDraggedArea(g, rMin, rMax, draggedColumn, header
+			Graphics2D g2d = (Graphics2D) g.create();
+			// enhancement 331 - translucent dragged column
+			g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
+					0.65f, g));
+			this.paintDraggedArea(g2d, rMin, rMax, draggedColumn, header
 					.getDraggedDistance());
+			g2d.dispose();
 		}
 
 		// Remove any renderers that may be left in the rendererPane.
@@ -678,7 +788,7 @@ public class SubstanceTableUI extends BasicTableUI {
 			Rectangle r = this.table
 					.getCellRect(row, draggedColumnIndex, false);
 			r.x += distance;
-			this.paintCell(g, r, row, draggedColumnIndex);
+			this.paintCell(g, r, r, row, draggedColumnIndex);
 
 			// Paint the (lower) horizontal grid line if necessary.
 			if (this.table.getShowHorizontalLines()) {
@@ -695,7 +805,8 @@ public class SubstanceTableUI extends BasicTableUI {
 		}
 	}
 
-	protected void paintCell(Graphics g, Rectangle cellRect, int row, int column) {
+	protected void paintCell(Graphics g, Rectangle cellRect,
+			Rectangle highlightCellRect, int row, int column) {
 		Graphics2D g2d = (Graphics2D) g.create();
 		// fix for issue 183 - passing the original Graphics context
 		// to compute the alpha composite. If the table is in a JXPanel
@@ -705,35 +816,30 @@ public class SubstanceTableUI extends BasicTableUI {
 		g2d.setComposite(TransitionLayout.getAlphaComposite(this.table, g));
 
 		TableCellId cellId = new TableCellId(row, column);
-		// boolean isSelectedAnim = FadeTracker.getInstance().isTracked(
-		// this.table, cellId, FadeKind.SELECTION);
-		// boolean isRolloverAnim = FadeTracker.getInstance().isTracked(
-		// this.table, cellId, FadeKind.ROLLOVER);
-		// Comparable currRoIndex = (Comparable) table
-		// .getClientProperty(ROLLED_OVER_INDEX);
 		boolean isRollover = ((this.rolledOverId != null) && this.rolledOverId
 				.equals(cellId));
 
-		ComponentState prevState = this.getPrevCellState(cellId);
-		ComponentState currState = this.getCellState(cellId);
+		final ComponentState prevState = this.getPrevCellState(cellId);
+		final ComponentState currState = this.getCellState(cellId);
 		// System.out.println(cellId + ":" + prevState.name() + "->"
 		// + currState.name());
-		float alphaForPrevBackground = 0.0f;
+		// float alphaForPrevBackground = 0.0f;
 
-		SubstanceTheme prevTheme = SubstanceCoreUtilities.getHighlightTheme(
-				this.table, prevState, true, false);
-		SubstanceTheme currTheme = SubstanceCoreUtilities.getHighlightTheme(
-				this.table, currState, true, false);
+		final SubstanceTheme prevTheme = SubstanceThemeUtilities
+				.getHighlightTheme(this.table, prevState);
+		final SubstanceTheme currTheme = SubstanceThemeUtilities
+				.getHighlightTheme(this.table, currState);
 
 		// Compute the alpha values for the animation.
-		float startAlpha = SubstanceCoreUtilities.getHighlightAlpha(this.table,
-				prevState, true);
-		float endAlpha = SubstanceCoreUtilities.getHighlightAlpha(this.table,
-				currState, true);
-		float alphaForCurrBackground = endAlpha;
+		float startAlpha = SubstanceThemeUtilities.getHighlightAlpha(
+				this.table, prevState);
+		float endAlpha = SubstanceThemeUtilities.getHighlightAlpha(this.table,
+				currState);
 
 		FadeState state = SubstanceFadeUtilities.getFadeState(this.table,
 				cellId, FadeKind.SELECTION, FadeKind.ROLLOVER);
+		float totalAlpha = endAlpha;
+		float fadeCoef = 0.0f;
 		// System.out.println("-------- (" + System.currentTimeMillis() % 10000
 		// + ") " + row + ":" + column + " [" + cellRect + "] --------");
 		if (state != null) {
@@ -741,10 +847,9 @@ public class SubstanceTableUI extends BasicTableUI {
 			// ":"
 			// + state.fadeKind + ":" + state.getFadePosition());
 			// System.out.println("States : " + prevState + "->" + currState);
-			float fadeCoef = state.getFadePosition();
+			fadeCoef = state.getFadePosition();
 
 			// compute the total alpha of the overlays.
-			float totalAlpha = 0.0f;
 			if (state.isFadingIn()) {
 				totalAlpha = startAlpha + (endAlpha - startAlpha) * fadeCoef
 						/ 10.0f;
@@ -756,10 +861,6 @@ public class SubstanceTableUI extends BasicTableUI {
 			if (state.isFadingIn())
 				fadeCoef = 10.0f - fadeCoef;
 
-			// compute the alpha for each one of the animation overlays
-			alphaForPrevBackground = totalAlpha * fadeCoef / 10.0f;
-			alphaForCurrBackground = totalAlpha * (10.0f - fadeCoef) / 10.0f;
-
 			// System.out.println("prev alpha " + alphaForPrevBackground
 			// + ", curr alpha " + alphaForCurrBackground);
 			// System.out.println("from " + prevTheme.getDisplayName() + " to "
@@ -775,29 +876,83 @@ public class SubstanceTableUI extends BasicTableUI {
 				&& (prevState.isKindActive(FadeKind.SELECTION) || currState
 						.isKindActive(FadeKind.SELECTION))) {
 			// no animations on selected cells in big tables - bug 209
-			alphaForPrevBackground = 0.0f;
-			alphaForCurrBackground = endAlpha;
+			fadeCoef = 0.0f;
+		}
+
+		final Set<SubstanceConstants.Side> highlightOpenSides = new HashSet<SubstanceConstants.Side>();
+		// show highlight border only when the table grid is not shown
+		final float highlightBorderAlpha = (table.getShowHorizontalLines() || table
+				.getShowVerticalLines()) ? 0.0f : 0.8f;
+		if (!table.getColumnSelectionAllowed()
+				&& table.getRowSelectionAllowed()) {
+			// if row selection is on and column selection is off, we will
+			// show the highlight for the entire row
+
+			// all cells have open left side
+			highlightOpenSides.add(SubstanceConstants.Side.LEFT);
+			// all cells have open right side
+			highlightOpenSides.add(SubstanceConstants.Side.RIGHT);
+		}
+		if (table.getColumnSelectionAllowed()
+				&& !table.getRowSelectionAllowed()) {
+			// if row selection is off and column selection is on, we will
+			// show the highlight for the entire column
+
+			// if (table.getTableHeader().isVisible() || (row > 0)) {
+			// the top side is open for all rows except the
+			// first, or when the table header is visible
+			highlightOpenSides.add(SubstanceConstants.Side.TOP);
+			// }
+			// if (row < (this.table.getRowCount() - 1)) {
+			// all cells but the last have open bottom side
+			highlightOpenSides.add(SubstanceConstants.Side.BOTTOM);
+			// }
+		}
+		if (row > 1) {
+			ComponentState upperNeighbourState = this
+					.getCellState(new TableCellId(row - 1, column));
+			if (currState == upperNeighbourState) {
+				// the cell above it is in the same state
+				highlightOpenSides.add(SubstanceConstants.Side.TOP);
+			}
+		}
+		if (column > 1) {
+			ComponentState leftNeighbourState = this
+					.getCellState(new TableCellId(row, column - 1));
+			if (currState == leftNeighbourState) {
+				// the cell to the left is in the same state
+				highlightOpenSides.add(SubstanceConstants.Side.LEFT);
+			}
+		}
+		// if ((row == 0) && table.getTableHeader().isVisible()) {
+		// // special case for a selected cell in first row when the
+		// // table header is visible - open top side
+		// highlightOpenSides.add(SubstanceConstants.Side.TOP);
+		// }
+		if (row == 0) {
+			highlightOpenSides.add(SubstanceConstants.Side.TOP);
+		}
+		if (row == (table.getRowCount() - 1)) {
+			highlightOpenSides.add(SubstanceConstants.Side.BOTTOM);
+		}
+		if (column == 0) {
+			highlightOpenSides.add(SubstanceConstants.Side.LEFT);
+		}
+		if (column == (table.getColumnCount() - 1)) {
+			highlightOpenSides.add(SubstanceConstants.Side.RIGHT);
 		}
 
 		if (this.table.isEditing() && this.table.getEditingRow() == row
 				&& this.table.getEditingColumn() == column) {
 			Component component = this.table.getEditorComponent();
 
-			if (alphaForPrevBackground > 0.0f) {
+			if (totalAlpha > 0.0f) {
 				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
-						alphaForPrevBackground, g));
-				backgroundDelegate.update(g2d, component,
-						new Rectangle(cellRect.x, cellRect.y, cellRect.width,
-								cellRect.height), prevTheme, 0.0f);
-				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
-						g));
-			}
-			if (alphaForCurrBackground > 0.0f) {
-				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
-						alphaForCurrBackground, g));
-				backgroundDelegate.update(g2d, component,
-						new Rectangle(cellRect.x, cellRect.y, cellRect.width,
-								cellRect.height), currTheme, 0.0f);
+						totalAlpha, g));
+				SubstanceHighlightUtils.paintHighlight(g2d, component,
+						highlightCellRect, highlightBorderAlpha,
+						highlightOpenSides, currTheme.getColorScheme(),
+						prevTheme.getColorScheme(), fadeCoef);
 				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
 						g));
 			}
@@ -857,49 +1012,138 @@ public class SubstanceTableUI extends BasicTableUI {
 		} else {
 			TableCellRenderer renderer = this.table
 					.getCellRenderer(row, column);
-			Component component = this.table.prepareRenderer(renderer, row,
-					column);
+			final Component rendererComponent = this.table.prepareRenderer(
+					renderer, row, column);
 
-			boolean isWatermarkBleed = SubstanceCoreUtilities
+			SubstanceTextPainter textPainter = SubstanceLookAndFeel
+					.getCurrentTextPainter();
+			final boolean isWatermarkBleed = SubstanceCoreUtilities
 					.toBleedWatermark(this.table);
-			if (!isWatermarkBleed) {
-				if (component != null) {
-					g2d.setColor(component.getBackground());
-					// System.out
-					// .println("Fill with " + component.getBackground());
-					g2d.fillRect(cellRect.x, cellRect.y, cellRect.width,
-							cellRect.height);
+			if (rendererComponent != null) {
+				if (textPainter.needsBackgroundImage()) {
+					final Rectangle offsetHighlightRect = new Rectangle(0, 0,
+							cellRect.width, cellRect.height);
+					textPainter.init(table, offsetHighlightRect, true);
+					textPainter.setBackgroundFill(table, null, false, 0, 0);
+					textPainter
+							.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+								public void paintBackground(Graphics g) {
+									Graphics2D g2d = (Graphics2D) g.create();
+									if (!isWatermarkBleed) {
+										// fill with the renderer
+										// background color
+										g2d.setColor(rendererComponent
+												.getBackground());
+										g2d.fillRect(0, 0,
+												offsetHighlightRect.width,
+												offsetHighlightRect.height);
+									} else {
+										SubstanceFillBackgroundDelegate.GLOBAL_INSTANCE
+												.fillAndWatermark(
+														g2d,
+														table,
+														rendererComponent
+																.getBackground(),
+														offsetHighlightRect);
+									}
+									g2d.dispose();
+								}
+							});
+				} else {
+					if (!isWatermarkBleed) {
+						// fill with the renderer background color
+						g2d.setColor(rendererComponent.getBackground());
+						g2d.fillRect(highlightCellRect.x, highlightCellRect.y,
+								highlightCellRect.width,
+								highlightCellRect.height);
+					} else {
+						SubstanceFillBackgroundDelegate.GLOBAL_INSTANCE
+								.fillAndWatermark(g2d, this.table,
+										rendererComponent.getBackground(),
+										highlightCellRect);
+					}
 				}
 			}
-			if ((alphaForPrevBackground > 0.0f)
-					&& (prevState != ComponentState.DEFAULT)) {
-				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
-						alphaForPrevBackground, g));
-				backgroundDelegate.update(g2d, component,
-						new Rectangle(cellRect.x, cellRect.y, cellRect.width,
-								cellRect.height), prevTheme, 0.0f);
-				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
-						g));
-				// System.out.println("Used " + prevTheme.getDisplayName()
-				// + " at " + alphaForPrevBackground);
-			}
-			if ((alphaForCurrBackground > 0.0f)
-					&& (currState != ComponentState.DEFAULT)) {
-				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
-						alphaForCurrBackground, g));
-				backgroundDelegate.update(g2d, component,
-						new Rectangle(cellRect.x, cellRect.y, cellRect.width,
-								cellRect.height), currTheme, 0.0f);
-				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
-						g));
-				// System.out.println("Used " + currTheme.getDisplayName()
-				// + " at " + alphaForCurrBackground);
+			if (textPainter.needsBackgroundImage()) {
+				final float finalTotalAlpha = totalAlpha;
+				final float finalFadeCoef = fadeCoef;
+
+				// The text GC will be clipped to the renderer bound. To have
+				// the background painted in the correct location, enforce this
+				// by painting the background in 0, 0 position.
+				final Rectangle zeroCellRect = new Rectangle(0, 0,
+						cellRect.width, cellRect.height);
+				textPainter.setBackgroundFill(table, rendererComponent
+						.getBackground(), true, cellRect.x, cellRect.y);
+				textPainter
+						.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+							public void paintBackground(Graphics g) {
+								Graphics2D g2d = (Graphics2D) g.create();
+
+								if (finalTotalAlpha > 0.0f) {
+									g2d.setComposite(TransitionLayout
+											.getAlphaComposite(table,
+													finalTotalAlpha, g));
+									SubstanceHighlightUtils.paintHighlight(g2d,
+											rendererComponent, zeroCellRect,
+											highlightBorderAlpha,
+											highlightOpenSides, currTheme
+													.getColorScheme(),
+											prevTheme.getColorScheme(),
+											finalFadeCoef);
+									g2d.setComposite(TransitionLayout
+											.getAlphaComposite(table, g));
+								}
+							}
+						});
+			} else {
+				if (totalAlpha > 0.0f) {
+					g2d.setComposite(TransitionLayout.getAlphaComposite(
+							this.table, totalAlpha, g));
+					float extra = SubstanceSizeUtils
+							.getBorderStrokeWidth(SubstanceSizeUtils
+									.getComponentFontSize(this.table
+											.getTableHeader()));
+					float extraWidth = highlightOpenSides
+							.contains(SubstanceConstants.Side.LEFT) ? 0.0f
+							: extra;
+					float extraHeight = highlightOpenSides
+							.contains(SubstanceConstants.Side.TOP) ? 0.0f
+							: extra;
+					// if ((column > 0) && isFocusedCell(row, column - 1))
+					// extraWidth = 0.0f;
+					// if ((row > 0) && isFocusedCell(row - 1, column))
+					// extraHeight = 0.0f;
+					// System.out
+					// .println(row + ":" + column + ":"
+					// + rendererComponent.getBackground() + ":"
+					// + totalAlpha + ":"
+					// + currTheme.getDisplayName() + ":"
+					// + prevTheme.getDisplayName() + ":"
+					// + fadeCoef);
+					SubstanceHighlightUtils.paintHighlight(g2d,
+							rendererComponent, new Rectangle(
+									highlightCellRect.x - (int) extraWidth,
+									highlightCellRect.y - (int) extraHeight,
+									highlightCellRect.width + (int) extraWidth,
+									highlightCellRect.height
+											+ (int) extraHeight),
+							highlightBorderAlpha, highlightOpenSides, currTheme
+									.getColorScheme(), prevTheme
+									.getColorScheme(), fadeCoef);
+					g2d.setComposite(TransitionLayout.getAlphaComposite(
+							this.table, g));
+				}
 			}
 
-			if (component instanceof JComponent) {
+			this.table
+					.putClientProperty(
+							SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND,
+							Boolean.TRUE);
+			if (rendererComponent instanceof JComponent) {
 				// Play with opaqueness to make our own gradient background
 				// on selected elements to show.
-				JComponent jRenderer = (JComponent) component;
+				JComponent jRenderer = (JComponent) rendererComponent;
 				synchronized (jRenderer) {
 					// Compute the selection status to prevent flicker - JTable
 					// registers a listener on selection changes and repaints
@@ -925,11 +1169,13 @@ public class SubstanceTableUI extends BasicTableUI {
 					if (!newOpaque)
 						SubstanceCoreUtilities
 								.makeNonOpaque(jRenderer, opacity);
-					// System.out.println("Painting at index " + row + " [" +
-					// value
-					// + "] " + (newOpaque ? "opaque" : "transparent")
+					// System.out.println("Painting "
+					// + rendererComponent.getClass().getSimpleName()
+					// + " at " + row + ":" + column + " "
+					// + (newOpaque ? "opaque" : "transparent")
 					// + " with bg " + rendererComponent.getBackground());
-					this.rendererPane.paintComponent(g2d, component,
+					// System.out.println(row + ":" + column + ":" + cellRect);
+					this.rendererPane.paintComponent(g2d, rendererComponent,
 							this.table, cellRect.x, cellRect.y, cellRect.width,
 							cellRect.height, true);
 					// System.out.println("Painting at index " + row + " [" +
@@ -947,10 +1193,12 @@ public class SubstanceTableUI extends BasicTableUI {
 					// + " with bg " + rendererComponent.getBackground());
 				}
 			} else {
-				this.rendererPane.paintComponent(g2d, component, this.table,
-						cellRect.x, cellRect.y, cellRect.width,
+				this.rendererPane.paintComponent(g2d, rendererComponent,
+						this.table, cellRect.x, cellRect.y, cellRect.width,
 						cellRect.height, true);
 			}
+			this.table.putClientProperty(
+					SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, null);
 		}
 		// System.out
 		// .println("------------------------------------------------------------------------------");
@@ -1101,6 +1349,18 @@ public class SubstanceTableUI extends BasicTableUI {
 								.getCellRect(CellRepaintCallback.this.rowIndex,
 										CellRepaintCallback.this.columnIndex,
 										true);
+
+						if (!table.getShowHorizontalLines()
+								&& !table.getShowVerticalLines()) {
+							float extra = SubstanceSizeUtils
+									.getBorderStrokeWidth(SubstanceSizeUtils
+											.getComponentFontSize(table
+													.getTableHeader()));
+							rect.x -= (int) extra;
+							rect.width += 2 * (int) extra;
+							rect.y -= (int) extra;
+							rect.height += 2 * (int) extra;
+						}
 						// System.out.println("Cell Repainting " + rowIndex +
 						// ":"
 						// + columnIndex + ":" + rect);
@@ -1253,6 +1513,15 @@ public class SubstanceTableUI extends BasicTableUI {
 											RowRepaintCallback.this.rowIndex,
 											i, true));
 						}
+						if (!table.getShowHorizontalLines()
+								&& !table.getShowVerticalLines()) {
+							float extra = SubstanceSizeUtils
+									.getBorderStrokeWidth(SubstanceSizeUtils
+											.getComponentFontSize(table
+													.getTableHeader()));
+							rect.y -= (int) extra;
+							rect.height += 2 * (int) extra;
+						}
 						// System.out.println("Repainting row " + rowIndex
 						// + " at " + rect);
 						RowRepaintCallback.this.table.repaint(rect);
@@ -1403,6 +1672,15 @@ public class SubstanceTableUI extends BasicTableUI {
 													ColumnRepaintCallback.this.columnIndex,
 													true));
 						}
+						if (!table.getShowHorizontalLines()
+								&& !table.getShowVerticalLines()) {
+							float extra = SubstanceSizeUtils
+									.getBorderStrokeWidth(SubstanceSizeUtils
+											.getComponentFontSize(table
+													.getTableHeader()));
+							rect.x -= (int) extra;
+							rect.width += 2 * (int) extra;
+						}
 						ColumnRepaintCallback.this.table.repaint(rect);
 					}
 				}
@@ -1650,11 +1928,12 @@ public class SubstanceTableUI extends BasicTableUI {
 	}
 
 	/**
-	 * Selection listener for selection animation effects.
+	 * State listener for tracking the selection changes.
 	 * 
 	 * @author Kirill Grouchnikov
 	 */
-	protected class TableSelectionListener implements ListSelectionListener {
+	protected class TableStateListener implements ListSelectionListener,
+			TableModelListener {
 		/*
 		 * (non-Javadoc)
 		 * 
@@ -1664,7 +1943,37 @@ public class SubstanceTableUI extends BasicTableUI {
 		public void valueChanged(final ListSelectionEvent e) {
 			SwingUtilities.invokeLater(new Runnable() {
 				public void run() {
-					SubstanceTableUI.this.syncSelection(e);
+					syncSelection();
+				}
+			});
+		}
+
+		/*
+		 * (non-Javadoc)
+		 * 
+		 * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
+		 */
+		public void tableChanged(final TableModelEvent e) {
+			// fix for defect 291 - tracking changes to the table.
+			SwingUtilities.invokeLater(new Runnable() {
+				public void run() {
+					// fix for defect 350 - font might have been
+					// switched in the middle of update
+					if (table == null)
+						return;
+
+					// fix for defect 328 - do not clear the
+					// internal selection and focus tracking
+					// when the event is table update.
+					if (e.getType() != TableModelEvent.UPDATE) {
+						selectedIndices.clear();
+						prevStateMap.clear();
+						nextStateMap.clear();
+						focusedColumn = -1;
+						focusedRow = -1;
+					}
+					syncSelection();
+					table.repaint();
 				}
 			});
 		}
@@ -1930,7 +2239,7 @@ public class SubstanceTableUI extends BasicTableUI {
 	 *            Selection event.
 	 */
 	// @SuppressWarnings("unchecked")
-	protected void syncSelection(ListSelectionEvent e) {
+	protected void syncSelection(/* ListSelectionEvent e */) {
 		if (this.table == null) {
 			// fix for defect 270 - if the UI delegate is updated
 			// by another selection listener, ignore this
@@ -2191,7 +2500,7 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @return <code>true</code> if the table has animations,
 	 *         <code>false</code> otherwise.
 	 */
-	protected boolean hasAnimations2() {
+	protected boolean hasAnimations() {
 		// fix for defects 164 and 209 - selection
 		// and deletion are very slow on large tables.
 		int rowCount = this.table.getRowCount();
@@ -2199,11 +2508,19 @@ public class SubstanceTableUI extends BasicTableUI {
 		if (rowCount * colCount >= 500)
 			return false;
 		if (this.table.getColumnSelectionAllowed()
-				&& !this.table.getRowSelectionAllowed())
+				&& !this.table.getRowSelectionAllowed()) {
+			if (!this.table.getShowHorizontalLines()
+					&& !this.table.getShowVerticalLines())
+				return rowCount <= 8;
 			return rowCount <= 25;
+		}
 		if (!this.table.getColumnSelectionAllowed()
-				&& this.table.getRowSelectionAllowed())
+				&& this.table.getRowSelectionAllowed()) {
+			if (!this.table.getShowHorizontalLines()
+					&& !this.table.getShowVerticalLines())
+				return colCount <= 8;
 			return colCount <= 25;
+		}
 		return true;
 	}
 
@@ -2213,8 +2530,8 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @return <code>true</code> if the table has selection animations,
 	 *         <code>false</code> otherwise.
 	 */
-	protected boolean hasSelectionAnimations() {
-		return this.hasAnimations2()
+	public boolean hasSelectionAnimations() {
+		return this.hasAnimations()
 				&& !LafWidgetUtilities.hasNoFades(this.table,
 						FadeKind.SELECTION);
 	}
@@ -2225,8 +2542,8 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @return <code>true</code> if the table has rollover animations,
 	 *         <code>false</code> otherwise.
 	 */
-	protected boolean hasRolloverAnimations() {
-		return this.hasAnimations2()
+	public boolean hasRolloverAnimations() {
+		return this.hasAnimations()
 				&& !LafWidgetUtilities
 						.hasNoFades(this.table, FadeKind.ROLLOVER);
 	}
@@ -2253,4 +2570,29 @@ public class SubstanceTableUI extends BasicTableUI {
 	public boolean isFocusedCell(int row, int column) {
 		return (this.focusedRow == row) && (this.focusedColumn == column);
 	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.ComponentUI#update(java.awt.Graphics,
+	 *      javax.swing.JComponent)
+	 */
+	@Override
+	public void update(Graphics g, JComponent c) {
+		SubstanceFillBackgroundDelegate.GLOBAL_INSTANCE.updateIfOpaque(g, c);
+
+		Graphics2D g2d = (Graphics2D) g.create();
+		// install state-aware alpha channel (support for skins that use
+		// translucency on disabled states).
+		// SubstanceTheme theme = SubstanceCoreUtilities.getTheme(this.table,
+		// true,
+		// true);
+		// float themeAlpha = theme.getThemeAlpha(this.table,
+		// this.table.isEnabled() ? ComponentState.DEFAULT
+		// : ComponentState.DISABLED_UNSELECTED);
+		// g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
+		// themeAlpha));
+		this.paint(g2d, c);
+		g2d.dispose();
+	}
 }
