@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -46,8 +46,11 @@ import javax.swing.tree.*;
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
 import org.jvnet.lafwidget.utils.LookUtils;
+import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.*;
+import org.jvnet.substance.utils.icon.SubstanceIconFactory;
 
 /**
  * UI for lists in <b>Substance</b> look and feel.
@@ -66,13 +69,12 @@ public class SubstanceTreeUI extends BasicTreeUI {
 	protected TreePathId currRolloverPathId;
 
 	/**
-	 * Delegate for painting the background.
+	 * Delegate for painting the background of list rows.
 	 */
-	private static SubstanceGradientBackgroundDelegate backgroundDelegate = new SubstanceGradientBackgroundDelegate();
+	private static SubstanceFillBackgroundDelegate backgroundDelegate = new SubstanceFillBackgroundDelegate();
 
 	/**
-	 * Listener that listens to changes on
-	 * {@link SubstanceLookAndFeel#WATERMARK_TO_BLEED} property.
+	 * Listener that listens to changes on tree properties.
 	 */
 	protected PropertyChangeListener substancePropertyChangeListener;
 
@@ -137,7 +139,6 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		if (SubstanceCoreUtilities.toBleedWatermark(this.tree))
 			this.tree.setOpaque(false);
 
-		// Map<TreePathId, Object> selected = new HashMap<TreePathId, Object>();
 		if (this.tree.getSelectionPaths() != null) {
 			for (TreePath selectionPath : this.tree.getSelectionPaths()) {
 				TreePathId pathId = new TreePathId(selectionPath);
@@ -146,19 +147,10 @@ public class SubstanceTreeUI extends BasicTreeUI {
 			}
 		}
 
-		SubstanceTheme theme = SubstanceCoreUtilities.getTheme(this.tree, true);
-		setExpandedIcon(new IconUIResource(new ImageIcon(SubstanceImageCreator
-				.getTreeIcon(
-						SubstanceSizeUtils.getComponentFontSize(this.tree),
-						this.tree, theme.getDefaultTheme().getColorScheme(),
-						SubstanceCoreUtilities.isThemeDark(theme), false))));
-		setCollapsedIcon(new IconUIResource(new ImageIcon(SubstanceImageCreator
-				.getTreeIcon(
-						SubstanceSizeUtils.getComponentFontSize(this.tree),
-						this.tree, theme.getDefaultTheme().getColorScheme(),
-						SubstanceCoreUtilities.isThemeDark(theme), true))));
-
-		// this.tree.putClientProperty(SELECTED_INDICES, selected);
+		setExpandedIcon(new IconUIResource(SubstanceIconFactory.getTreeIcon(
+				this.tree, false)));
+		setCollapsedIcon(new IconUIResource(SubstanceIconFactory.getTreeIcon(
+				this.tree, true)));
 	}
 
 	/*
@@ -168,7 +160,6 @@ public class SubstanceTreeUI extends BasicTreeUI {
 	 */
 	@Override
 	protected void uninstallDefaults() {
-		// this.tree.putClientProperty(SELECTED_INDICES, null);
 		this.selectedPaths.clear();
 		super.uninstallDefaults();
 	}
@@ -197,34 +188,21 @@ public class SubstanceTreeUI extends BasicTreeUI {
 			leadIndex = -1;
 		}
 
-		Component component;
+		final Component renderer = this.currentCellRenderer
+				.getTreeCellRendererComponent(this.tree, path
+						.getLastPathComponent(), this.tree.isRowSelected(row),
+						isExpanded, isLeaf, row, (leadIndex == row));
 
-		component = this.currentCellRenderer.getTreeCellRendererComponent(
-				this.tree, path.getLastPathComponent(), this.tree
-						.isRowSelected(row), isExpanded, isLeaf, row,
-				(leadIndex == row));
-
-		// second part - fix for defect 214 (rollover effects on non-opaque
-		// trees resulted in inconsistent behaviour)
-		boolean isWatermarkBleed = SubstanceCoreUtilities
-				.toBleedWatermark(tree)
-				|| !tree.isOpaque();
+		// // second part - fix for defect 214 (rollover effects on non-opaque
+		// // trees resulted in inconsistent behaviour)
+		// boolean isWatermarkBleed = SubstanceCoreUtilities
+		// .toBleedWatermark(tree)
+		// || !tree.isOpaque();
 
 		TreePathId pathId = new TreePathId(path);
-		// boolean isSelectedAnim =
-		// FadeTracker.getInstance().isTracked(this.tree,
-		// pathId, FadeKind.SELECTION);
-		// boolean isRolloverAnim =
-		// FadeTracker.getInstance().isTracked(this.tree,
-		// pathId, FadeKind.ROLLOVER);
-		// TreePathId currRoPath = (TreePathId) tree
-		// .getClientProperty(ROLLED_OVER_INDEX);
-		// boolean isRollover = ((currRolloverPathId != null) &&
-		// (currRolloverPathId.path
-		// .equals(path)));
 
 		// Respect the current composite set on the graphics - for
-		// JXPanel alpah channel
+		// JXPanel alpha channel
 		float currFactor = 1.0f;
 		Composite currComposite = ((Graphics2D) g).getComposite();
 		if (currComposite instanceof AlphaComposite) {
@@ -241,67 +219,33 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		// alpha composite applied to it.
 		g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree,
 				currFactor));
-		if ((!isWatermarkBleed) && (component.getBackground() != null)) {
-			g2d.setColor(component.getBackground());
-			if (this.tree.getComponentOrientation().isLeftToRight()) {
-				g2d.fillRect(bounds.x, bounds.y, clipBounds.width,
-						bounds.height);
-			} else {
-				g2d.fillRect(clipBounds.x, bounds.y, bounds.width,
-						bounds.height);
-			}
-		}
-		// float alphaForBackground = 0.0f;
-		//
-		// // Support for selection animations (from 3.1)
-		// if (this.tree.isRowSelected(row) || isSelectedAnim) {
-		// if (isSelectedAnim) {
-		// // set the alpha for selection animation
-		// float fadeCoef = FadeTracker.getInstance().getFade10(this.tree,
-		// pathId, FadeKind.SELECTION);
-		// alphaForBackground = 0.7f * fadeCoef / 10.0f;
-		// } else {
-		// alphaForBackground = 0.7f;
-		// }
-		// }
-		// // Support for rollover animations (from 3.1)
-		// if (isRolloverAnim) {
-		// // set the alpha for rollover animation
-		// float fadeCoef = FadeTracker.getInstance().getFade10(this.tree,
-		// pathId, FadeKind.ROLLOVER);
-		// // System.out.println("Has rollover anim on " + row + "["
-		// // + fadeCoef + "] : " + cx + ":" + cy + "-" + cw + ":"
-		// // + ch);
-		// alphaForBackground = Math.max(alphaForBackground,
-		// 0.4f * fadeCoef / 10.0f);
-		// } else {
-		// if (isRollover) {
-		// alphaForBackground = Math.max(alphaForBackground, 0.4f);
-		// }
-		// }
 
-		ComponentState prevState = this.getPrevPathState(pathId);
-		ComponentState currState = this.getPathState(pathId);
-		float alphaForPrevBackground = 0.0f;
+		Color background = renderer.getBackground();
+		if (background == null)
+			background = tree.getBackground();
 
-		// Compute the alpha values for the animation (the highlights
-		// have separate alpha channels, so that rollover animation starts
-		// from 0.0 alpha and goes to 0.4 alpha on non-selected cells,
-		// but on selected cells goes from 0.7 to 0.9). We need to respect
-		// these border values for proper visual transitions
-		float startAlpha = SubstanceCoreUtilities.getHighlightAlpha(this.tree,
-				prevState, true);
-		float endAlpha = SubstanceCoreUtilities.getHighlightAlpha(this.tree,
-				currState, true);
-		float alphaForCurrBackground = endAlpha;
+		final ComponentState prevState = this.getPrevPathState(pathId);
+		final ComponentState currState = this.getPathState(pathId);
+
+		final SubstanceTheme prevTheme = SubstanceThemeUtilities
+				.getHighlightTheme(tree, prevState);
+		final SubstanceTheme currTheme = SubstanceThemeUtilities
+				.getHighlightTheme(tree, currState);
+
+		// Compute the alpha values for the animation.
+		float startAlpha = SubstanceThemeUtilities.getHighlightAlpha(this.tree,
+				prevState);
+		float endAlpha = SubstanceThemeUtilities.getHighlightAlpha(this.tree,
+				currState);
 
 		FadeState state = SubstanceFadeUtilities.getFadeState(this.tree,
 				pathId, FadeKind.SELECTION, FadeKind.ROLLOVER);
+		float totalAlpha = endAlpha;
+		float fadeCoef = 0.0f;
 		if (state != null) {
-			float fadeCoef = state.getFadePosition();
+			fadeCoef = state.getFadePosition();
 
 			// compute the total alpha of the overlays.
-			float totalAlpha = 0.0f;
 			if (state.isFadingIn()) {
 				totalAlpha = startAlpha + (endAlpha - startAlpha) * fadeCoef
 						/ 10.0f;
@@ -312,17 +256,8 @@ public class SubstanceTreeUI extends BasicTreeUI {
 
 			if (state.isFadingIn())
 				fadeCoef = 10.0f - fadeCoef;
-
-			// compute the alpha for each one of the animation overlays
-			alphaForPrevBackground = totalAlpha * fadeCoef / 10.0f;
-			alphaForCurrBackground = totalAlpha * (10.0f - fadeCoef) / 10.0f;
 		}
 
-		SubstanceTheme prevTheme = SubstanceCoreUtilities.getHighlightTheme(
-				this.tree, prevState, true, false);
-		SubstanceTheme currTheme = SubstanceCoreUtilities.getHighlightTheme(
-				this.tree, currState, true, false);
-
 		// System.out.println(row + ":" + prevTheme.getDisplayName() + "["
 		// + alphaForPrevBackground + "]:" + currTheme.getDisplayName()
 		// + "[" + alphaForCurrBackground + "]");
@@ -330,53 +265,90 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		// The DefaultTreeCellRenderer overrides the isOpaque method
 		// so that there is no point in trying to make it non-opaque.
 		// Fix for defect 181.
-		boolean canHaveSubstanceEffects = !(component instanceof DefaultTreeCellRenderer);
-		if (canHaveSubstanceEffects && (alphaForPrevBackground > 0.0f)) {
+		final boolean canHaveSubstanceEffects = !(renderer instanceof DefaultTreeCellRenderer);
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		if (canHaveSubstanceEffects && (totalAlpha > 0.0f)) {
 			g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree,
-					currFactor * alphaForPrevBackground, g));
-			// Fix for defect 180 (old code in comments) - painting the
+					currFactor * totalAlpha, g));
+			// Fix for defect 180 - painting the
 			// highlight beneath the entire row
-			backgroundDelegate.update(g2d, component, new Rectangle(this.tree
-					.getInsets().left /* bounds.x */, bounds.y,
-					this.tree.getWidth() - this.tree.getInsets().right
-							- this.tree.getInsets().left/* bounds.x */,
-					bounds.height), prevTheme, 0.8f);
+			SubstanceHighlightUtils
+					.paintHighlight(g2d, renderer, new Rectangle(this.tree
+							.getInsets().left, bounds.y, this.tree.getWidth()
+							- this.tree.getInsets().right
+							- this.tree.getInsets().left, bounds.height), 0.8f,
+							null, currTheme.getColorScheme(), prevTheme
+									.getColorScheme(), fadeCoef);
 			g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree,
 					currFactor));
 		}
 
-		if (canHaveSubstanceEffects && (alphaForCurrBackground > 0.0f)) {
-			g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree,
-					currFactor * alphaForCurrBackground, g));
-			// Fix for defect 180 (old code in comments) - painting the
-			// highlight beneath the entire row
-			backgroundDelegate.update(g2d, component, new Rectangle(this.tree
-					.getInsets().left /* bounds.x */, bounds.y,
-					this.tree.getWidth() - this.tree.getInsets().right
-							- this.tree.getInsets().left/* bounds.x */,
-					bounds.height), currTheme, 0.8f);
-			g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree,
-					currFactor));
+		// System.out.println("Painting row " + row);
+		final boolean isWatermarkBleed = SubstanceCoreUtilities
+				.toBleedWatermark(this.tree);
+		if (textPainter.needsBackgroundImage()) {
+			final float finalTotalAlpha = totalAlpha;
+			final float finalCurrFactor = currFactor;
+			final float finalFadeCoef = fadeCoef;
+			final Rectangle cellRect = new Rectangle(this.tree.getInsets().left
+					- bounds.x, 0, this.tree.getWidth()
+					- this.tree.getInsets().right - this.tree.getInsets().left,
+					bounds.height);
+			textPainter.init(this.tree, cellRect, true);
+			textPainter.setBackgroundFill(this.tree, background, true,
+					bounds.x, bounds.y);
+
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							Graphics2D g2d = (Graphics2D) g.create();
+							if (!isWatermarkBleed) {
+								// fill with the renderer
+								// background color
+								g2d.setColor(renderer.getBackground());
+								g2d.fillRect(0, 0, cellRect.width,
+										cellRect.height);
+							} else {
+								backgroundDelegate.fillAndWatermark(g2d, tree,
+										renderer.getBackground(), cellRect);
+							}
+							g2d.dispose();
+						}
+					});
+
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							Graphics2D g2d = (Graphics2D) g.create();
+							if (canHaveSubstanceEffects
+									&& (finalTotalAlpha > 0.0f)) {
+								g2d.setComposite(TransitionLayout
+										.getAlphaComposite(tree,
+												finalCurrFactor
+														* finalTotalAlpha, g));
+								// Fix for defect 180 - painting the
+								// highlight beneath the entire row
+								SubstanceHighlightUtils.paintHighlight(g2d,
+										renderer, cellRect, 0.8f, null,
+										currTheme.getColorScheme(), prevTheme
+												.getColorScheme(),
+										finalFadeCoef);
+								g2d.setComposite(TransitionLayout
+										.getAlphaComposite(tree,
+												finalCurrFactor));
+							}
+						}
+					});
+		} else {
 		}
 
-		// if (canHaveSubstanceEffects && (alphaForBackground > 0.0f)) {
-		// g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree,
-		// alphaForBackground, g));
-		// // Fix for defect 180 (old code in comments) - painting the
-		// // highlight beneath the entire row
-		// backgroundDelegate.update(g2d, component, new Rectangle(this.tree
-		// .getInsets().left /* bounds.x */, bounds.y,
-		// this.tree.getWidth() - this.tree.getInsets().right
-		// - this.tree.getInsets().left/* bounds.x */,
-		// bounds.height), SubstanceCoreUtilities.getHighlightTheme(
-		// this.tree, ComponentState.ACTIVE, true, true), true);
-		// g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree, g));
-		// }
-
-		if (component instanceof JComponent) {
+		this.tree.putClientProperty(
+				SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, Boolean.TRUE);
+		if (renderer instanceof JComponent) {
 			// Play with opacity to make our own gradient background
 			// on selected elements to show.
-			JComponent jRenderer = (JComponent) component;
+			JComponent jRenderer = (JComponent) renderer;
 			synchronized (jRenderer) {
 				boolean newOpaque = !this.tree.isRowSelected(row);
 				if (SubstanceCoreUtilities.toBleedWatermark(this.tree))
@@ -389,7 +361,7 @@ public class SubstanceTreeUI extends BasicTreeUI {
 				Map<Component, Boolean> opacity = new HashMap<Component, Boolean>();
 				if (!newOpaque)
 					SubstanceCoreUtilities.makeNonOpaque(jRenderer, opacity);
-				this.rendererPane.paintComponent(g2d, component, this.tree,
+				this.rendererPane.paintComponent(g2d, renderer, this.tree,
 						bounds.x, bounds.y, Math.max(this.tree.getWidth()
 								- this.tree.getInsets().right
 								- this.tree.getInsets().left - bounds.x,
@@ -398,26 +370,27 @@ public class SubstanceTreeUI extends BasicTreeUI {
 					SubstanceCoreUtilities.restoreOpaque(jRenderer, opacity);
 			}
 		} else {
-			this.rendererPane.paintComponent(g2d, component, this.tree,
+			this.rendererPane.paintComponent(g2d, renderer, this.tree,
 					bounds.x, bounds.y, Math
 							.max(clipBounds.width, bounds.width),
 					bounds.height, true);
 		}
+		this.tree.putClientProperty(
+				SubstanceCoreUtilities.DO_NOT_FILL_BACKGROUND, null);
 
-		// Paint the expand control once again since it has been overlayed
-		// by the highlight background on selected and rolled over rows.
+		// Paint the expand control now after the row background has been
+		// overlayed by the highlight background on selected and rolled over
+		// rows. See comments on paintExpandControlEnforce().
 		if (shouldPaintExpandControl(path, row, isExpanded, hasBeenExpanded,
 				isLeaf)) {
 			if (!this.tree.getComponentOrientation().isLeftToRight()
 					&& LookUtils.IS_JAVA_5) {
 				bounds.x -= 4;
 			}
-			paintExpandControl(g2d, clipBounds, insets, bounds, path, row,
-					isExpanded, hasBeenExpanded, isLeaf);
+			paintExpandControlEnforce(g2d, clipBounds, insets, bounds, path,
+					row, isExpanded, hasBeenExpanded, isLeaf);
 		}
 
-		// g2d.setColor(Color.blue);
-		// g2d.draw(bounds);
 		g2d.dispose();
 	}
 
@@ -432,31 +405,46 @@ public class SubstanceTreeUI extends BasicTreeUI {
 	protected void paintExpandControl(Graphics g, Rectangle clipBounds,
 			Insets insets, Rectangle bounds, TreePath path, int row,
 			boolean isExpanded, boolean hasBeenExpanded, boolean isLeaf) {
+		// This does nothing. The base implementation of paint() paints
+		// the tree lines and tree expand controls *before* painting the
+		// renderer. In Substance, the highlights are painted in the
+		// paintRow, and thus would overlay the expand controls. This results
+		// in expand controls being much less visible under most of the skins.
+		// So, Substance paints the expand controls *after* painting the
+		// highlights (and the renderer which doesn't overlap with the expand
+		// controls in any case). This is done in paintRow() by calling
+		// the paintExpandControlEnforce() instead (that eventually calls the
+		// super implementation of paintExpandControl().
+	}
+
+	protected void paintExpandControlEnforce(Graphics g, Rectangle clipBounds,
+			Insets insets, Rectangle bounds, TreePath path, int row,
+			boolean isExpanded, boolean hasBeenExpanded, boolean isLeaf) {
 		boolean toPaint = (!this.tree.isEnabled())
 				|| this.isInside
 				|| !FadeConfigurationManager.getInstance().fadeAllowed(
 						SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND,
 						tree);
+
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(this.tree);
+		float alpha = theme.getThemeAlpha(this.tree,
+				this.tree.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED);
+
+		Graphics2D graphics = (Graphics2D) g.create();
 		if (FadeTracker.getInstance().isTracked(this.tree,
 				SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND)) {
-			Graphics2D graphics = (Graphics2D) g.create();
-			graphics
-					.setComposite(TransitionLayout
-							.getAlphaComposite(
-									this.tree,
-									FadeTracker
-											.getInstance()
-											.getFade10(
-													this.tree,
-													SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND) / 10.0f,
-									g));
+			toPaint = true;
+			alpha *= FadeTracker.getInstance().getFade10(this.tree,
+					SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND) / 10.0f;
+		}
+		if (toPaint) {
+			graphics.setComposite(TransitionLayout.getAlphaComposite(this.tree,
+					alpha, g));
 			super.paintExpandControl(graphics, clipBounds, insets, bounds,
 					path, row, isExpanded, hasBeenExpanded, isLeaf);
-			graphics.dispose();
-		} else if (toPaint) {
-			super.paintExpandControl(g, clipBounds, insets, bounds, path, row,
-					isExpanded, hasBeenExpanded, isLeaf);
 		}
+		graphics.dispose();
 	}
 
 	/*
@@ -484,21 +472,21 @@ public class SubstanceTreeUI extends BasicTreeUI {
 				|| !FadeConfigurationManager.getInstance().fadeAllowed(
 						SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND,
 						tree);
+
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(this.tree);
+		float alpha = theme.getThemeAlpha(this.tree,
+				this.tree.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED);
+
 		if (FadeTracker.getInstance().isTracked(this.tree,
 				SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND)) {
-			graphics
-					.setComposite(TransitionLayout
-							.getAlphaComposite(
-									this.tree,
-									FadeTracker
-											.getInstance()
-											.getFade10(
-													this.tree,
-													SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND) / 10.0f,
-									g));
-			super.paintHorizontalPartOfLeg(graphics, clipBounds, insets,
-					bounds, path, row, isExpanded, hasBeenExpanded, isLeaf);
-		} else if (toPaint) {
+			toPaint = true;
+			alpha *= FadeTracker.getInstance().getFade10(this.tree,
+					SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND) / 10.0f;
+		}
+		if (toPaint) {
+			graphics.setComposite(TransitionLayout.getAlphaComposite(this.tree,
+					alpha, g));
 			super.paintHorizontalPartOfLeg(graphics, clipBounds, insets,
 					bounds, path, row, isExpanded, hasBeenExpanded, isLeaf);
 		}
@@ -528,20 +516,21 @@ public class SubstanceTreeUI extends BasicTreeUI {
 				|| !FadeConfigurationManager.getInstance().fadeAllowed(
 						SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND,
 						tree);
+
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(this.tree);
+		float alpha = theme.getThemeAlpha(this.tree,
+				this.tree.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED);
+
 		if (FadeTracker.getInstance().isTracked(this.tree,
 				SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND)) {
-			graphics
-					.setComposite(TransitionLayout
-							.getAlphaComposite(
-									this.tree,
-									FadeTracker
-											.getInstance()
-											.getFade10(
-													this.tree,
-													SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND) / 10.0f,
-									g));
-			super.paintVerticalPartOfLeg(graphics, clipBounds, insets, path);
-		} else if (toPaint) {
+			toPaint = true;
+			alpha *= FadeTracker.getInstance().getFade10(this.tree,
+					SubstanceLookAndFeel.TREE_DECORATIONS_ANIMATION_KIND) / 10.0f;
+		}
+		if (toPaint) {
+			graphics.setComposite(TransitionLayout.getAlphaComposite(this.tree,
+					alpha, g));
 			super.paintVerticalPartOfLeg(graphics, clipBounds, insets, path);
 		}
 		graphics.dispose();
@@ -741,28 +730,6 @@ public class SubstanceTreeUI extends BasicTreeUI {
 						new PathRepaintCallback(tree, entry.getKey().path));
 				it.remove();
 			}
-
-			//
-			//
-			//
-			// // check if was selected before and still points to
-			// // the same element
-			// if (currSelected.containsKey(cellId)) {
-			// if (currSelected.get(cellId).equals(
-			// table.getValueAt(i, j))) {
-			// // start fading out
-			// // System.out.println("Fade out on index " + i
-			// // + ":" + j);
-			// FadeTracker.getInstance().trackFadeOut(
-			// FadeKind.SELECTION, table, cellId,
-			// false,
-			// new CellRepaintCallback(table, i, j));
-			// }
-			// currSelected.remove(cellId);
-			// }
-			// }
-			// }
-			// }
 		}
 	}
 
@@ -922,9 +889,6 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		}
 
 		public void mouseExited(MouseEvent e) {
-			// if (SubstanceCoreUtilities.toBleedWatermark(list))
-			// return;
-
 			if (!tree.isEnabled())
 				return;
 			isInside = false;
@@ -937,13 +901,9 @@ public class SubstanceTreeUI extends BasicTreeUI {
 			this.fadeOut();
 			// System.out.println("Nulling RO index");
 			currRolloverPathId = null;
-			// tree.putClientProperty(ROLLED_OVER_INDEX, null);
 		}
 
 		public void mouseMoved(MouseEvent e) {
-			// if (SubstanceCoreUtilities.toBleedWatermark(list))
-			// return;
-
 			if (!tree.isEnabled())
 				return;
 			isInside = true;
@@ -951,9 +911,6 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		}
 
 		public void mouseDragged(MouseEvent e) {
-			// if (SubstanceCoreUtilities.toBleedWatermark(list))
-			// return;
-
 			if (!tree.isEnabled())
 				return;
 			handleMove(e);
@@ -973,20 +930,16 @@ public class SubstanceTreeUI extends BasicTreeUI {
 			if (bounds == null) {
 				this.fadeOut();
 				currRolloverPathId = null;
-				// tree.putClientProperty(ROLLED_OVER_INDEX, null);
 				return;
 			}
 			if ((e.getY() < bounds.y)
 					|| (e.getY() > (bounds.y + bounds.height))) {
 				this.fadeOut();
 				currRolloverPathId = null;
-				// tree.putClientProperty(ROLLED_OVER_INDEX, null);
 				return;
 			}
 			// check if this is the same index
 			TreePathId newPathId = new TreePathId(closestPath);
-			// TreePathId currPathId = (TreePathId) tree
-			// .getClientProperty(ROLLED_OVER_INDEX);
 			if ((currRolloverPathId != null)
 					&& newPathId.equals(currRolloverPathId)) {
 				// System.out.println("Same location " +
@@ -1010,15 +963,12 @@ public class SubstanceTreeUI extends BasicTreeUI {
 					new PathRepaintCallback(tree, closestPath));
 			// System.out.println("Setting RO index to " + roIndex);
 			currRolloverPathId = newPathId;
-			// tree.putClientProperty(ROLLED_OVER_INDEX, newPathId);
 		}
 
 		/**
 		 * Initiates the fade out effect.
 		 */
 		private void fadeOut() {
-			// TreePathId prevRoPath = (TreePathId) tree
-			// .getClientProperty(ROLLED_OVER_INDEX);
 			if (currRolloverPathId == null)
 				return;
 
@@ -1108,8 +1058,11 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		}
 		if (count == 0)
 			return -1;
-		return totalY / count - 2
-				* SubstanceSizeUtils.getTreeIconSize(tree.getFont().getSize());
+		return totalY
+				/ count
+				- 2
+				* SubstanceSizeUtils.getTreeIconSize(SubstanceSizeUtils
+						.getComponentFontSize(tree));
 	}
 
 	/**
@@ -1141,4 +1094,125 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		synthModel.setSelected(this.tree.isRowSelected(rowIndex));
 		return ComponentState.getState(synthModel, null);
 	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.ComponentUI#update(java.awt.Graphics,
+	 *      javax.swing.JComponent)
+	 */
+	@Override
+	public void update(Graphics g, JComponent c) {
+		SubstanceFillBackgroundDelegate.GLOBAL_INSTANCE.updateIfOpaque(g, c);
+
+		// Should never happen if installed for a UI
+		if (treeState == null) {
+			return;
+		}
+
+		Rectangle paintBounds = g.getClipBounds();
+		Insets insets = tree.getInsets();
+
+		TreePath initialPath = getClosestPathForLocation(tree, 0, paintBounds.y);
+		Enumeration paintingEnumerator = treeState
+				.getVisiblePathsFrom(initialPath);
+		int row = treeState.getRowForPath(initialPath);
+		int endY = paintBounds.y + paintBounds.height;
+
+		// second part - fix for defect 214 (rollover effects on non-opaque
+		// trees resulted in inconsistent behaviour)
+		boolean isWatermarkBleed = SubstanceCoreUtilities
+				.toBleedWatermark(tree)
+				|| !tree.isOpaque();
+
+		Graphics2D g2d = (Graphics2D) g.create();
+
+		if (initialPath != null && paintingEnumerator != null) {
+			boolean done = false;
+			Rectangle boundsBuffer = new Rectangle();
+			Rectangle bounds;
+			TreePath path;
+
+			while (!done && paintingEnumerator.hasMoreElements()) {
+				path = (TreePath) paintingEnumerator.nextElement();
+				if (path != null) {
+					// respect the background color of the renderer.
+					boolean isLeaf = treeModel.isLeaf(path
+							.getLastPathComponent());
+					boolean isExpanded = isLeaf ? false : treeState
+							.getExpandedState(path);
+					Component renderer = this.currentCellRenderer
+							.getTreeCellRendererComponent(this.tree, path
+									.getLastPathComponent(), this.tree
+									.isRowSelected(row), isExpanded, isLeaf,
+									row, tree.hasFocus() ? (tree
+											.getLeadSelectionRow() == row)
+											: false);
+					Color background = renderer.getBackground();
+					if (background == null)
+						background = tree.getBackground();
+					bounds = treeState.getBounds(path, boundsBuffer);
+					bounds.x += insets.left;
+					bounds.y += insets.top;
+					if (!isWatermarkBleed) {
+						g2d.setColor(background);
+						g2d.fillRect(paintBounds.x, bounds.y,
+								paintBounds.width, bounds.height);
+					} else {
+						if (this.tree.getComponentOrientation().isLeftToRight()) {
+							backgroundDelegate.fillAndWatermark(g2d, this.tree,
+									background, new Rectangle(paintBounds.x,
+											bounds.y, paintBounds.width,
+											bounds.height));
+						} else {
+							backgroundDelegate.fillAndWatermark(g2d, this.tree,
+									background, new Rectangle(paintBounds.x,
+											bounds.y, paintBounds.width,
+											bounds.height));
+						}
+					}
+					if ((bounds.y + bounds.height) >= endY)
+						done = true;
+				} else {
+					done = true;
+				}
+				row++;
+			}
+		}
+
+		this.paint(g2d, c);
+		g2d.dispose();
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.plaf.basic.BasicTreeUI#getHashColor()
+	 */
+	@Override
+	protected Color getHashColor() {
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(this.tree,
+				this.tree.isEnabled() ? ComponentState.DEFAULT
+						: ComponentState.DISABLED_UNSELECTED);
+		Color curr = theme.getLineColor();
+
+		// // support for enhancement 256 - colorization of controls.
+		// boolean hasColorization = SubstanceCoreUtilities
+		// .hasColorization(this.tree);
+		// if (hasColorization) {
+		// Color backgr = this.tree.getBackground();
+		// if (!(backgr instanceof UIResource)) {
+		// double colorizationFactor = SubstanceCoreUtilities
+		// .getColorizationFactor(this.tree);
+		// if (!tree.isEnabled())
+		// colorizationFactor /= 2.0;
+		// Color toShiftTo = SubstanceColorUtilities.deriveByBrightness(
+		// backgr, curr);
+		// curr = SubstanceColorUtilities.getInterpolatedColor(toShiftTo,
+		// curr, colorizationFactor);
+		// }
+		// }
+
+		return curr;
+	}
 }
