@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -36,11 +36,11 @@ import org.jvnet.substance.button.ClassicButtonShaper;
 import org.jvnet.substance.color.ColorScheme;
 import org.jvnet.substance.painter.SimplisticSoftBorderReverseGradientPainter;
 import org.jvnet.substance.painter.SubduedGradientPainter;
+import org.jvnet.substance.painter.decoration.*;
+import org.jvnet.substance.painter.highlight.ClassicHighlightPainter;
 import org.jvnet.substance.theme.SubstanceComplexTheme;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.theme.SubstanceTheme.ThemeKind;
-import org.jvnet.substance.title.ArcHeaderPainter;
-import org.jvnet.substance.title.MarbleNoiseHeaderPainter;
 import org.jvnet.substance.utils.ComponentState;
 import org.jvnet.substance.watermark.SubstanceNullWatermark;
 
@@ -739,24 +739,32 @@ public class NebulaSkin extends SubstanceAbstractSkin {
 		theme.registerComponentHighlightStateTheme(pressedTheme, 0.8f,
 				ComponentState.ARMED, ComponentState.ROLLOVER_ARMED);
 
+		theme.registerDecorationAreaTheme(activeTitleTheme, false,
+				DecorationAreaType.PRIMARY_TITLE_PANE,
+				DecorationAreaType.SECONDARY_TITLE_PANE,
+				DecorationAreaType.HEADER, DecorationAreaType.GENERAL);
+
 		theme
 				.setNonActivePainter(new SimplisticSoftBorderReverseGradientPainter());
 		theme.setSelectedTabFadeStart(0.1);
 		theme.setSelectedTabFadeEnd(0.3);
+
+		theme.setPaintingToolbarDropShadows();
+
 		this.theme = theme;
 
-		this.shaper = new ClassicButtonShaper();
+		this.buttonShaper = new ClassicButtonShaper();
 		this.watermark = new SubstanceNullWatermark();
 		this.gradientPainter = new SubduedGradientPainter();
 		// this.titlePainter = new FlatTitlePainter();
 
-		MarbleNoiseHeaderPainter titlePainter = new MarbleNoiseHeaderPainter(
-				false, true);
-		titlePainter.setPaintingToolbarDropShadows(true);
-		titlePainter.setPaintingSeparators(true);
-		titlePainter.setBaseTitlePainter(new ArcHeaderPainter());
-		titlePainter.setTextureAlpha(0.3f);
-		this.titlePainter = titlePainter;
+		MarbleNoiseDecorationPainter decorationPainter = new MarbleNoiseDecorationPainter();
+		decorationPainter.setPaintingSeparators(true);
+		decorationPainter.setBaseDecorationPainter(new ArcDecorationPainter());
+		decorationPainter.setTextureAlpha(0.3f);
+		this.decorationPainter = decorationPainter;
+
+		this.highlightPainter = new ClassicHighlightPainter();
 
 		this.borderPainter = new FlatBorderPainter();
 		// this.tabBackgroundComposite = new
