@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -31,15 +31,16 @@ package org.jvnet.substance.theme;
 
 import java.awt.Color;
 import java.awt.Component;
-import java.util.HashMap;
-import java.util.Map;
+import java.util.*;
 
 import javax.swing.JTabbedPane;
 
 import org.jvnet.lafwidget.animation.FadeKind;
 import org.jvnet.substance.painter.SubstanceGradientPainter;
-import org.jvnet.substance.utils.ComponentState;
-import org.jvnet.substance.utils.SubstanceHighlightManager;
+import org.jvnet.substance.painter.decoration.DecorationAreaType;
+import org.jvnet.substance.theme.transform.ShiftTransform;
+import org.jvnet.substance.theme.transform.ThemeTransform;
+import org.jvnet.substance.utils.*;
 
 /**
  * Complex theme. A complex theme is defined by four separate themes:
@@ -111,6 +112,12 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 */
 	protected Map<ComponentState, Float> stateHighlightThemeAlphaMap;
 
+	protected Map<DecorationAreaType, SubstanceTheme> decorationThemeMap;
+
+	protected Set<DecorationAreaType> decoratedAreaSet;
+
+	protected Set<DecorationAreaType> toUseDecorationOnActiveControls;
+
 	/**
 	 * Painter for painting the non-active controls.
 	 */
@@ -130,6 +137,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 */
 	protected double selectedTabFadeEnd;
 
+	protected boolean isPaintingToolbarDropShadows;
+
 	/**
 	 * If <code>true</code>, the light background color and the background
 	 * color are the same.
@@ -138,6 +147,27 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 */
 	protected boolean useSameBackgroundColor;
 
+	/**
+	 * Creates a new complex theme.
+	 * 
+	 * @param displayName
+	 *            Theme display name.
+	 * @param themeKind
+	 *            Theme kind.
+	 * @param activeTheme
+	 *            Active theme.
+	 * @param defaultTheme
+	 *            Default theme.
+	 * @param disabledTheme
+	 *            Disabled theme.
+	 */
+	public SubstanceComplexTheme(String displayName, ThemeKind themeKind,
+			SubstanceTheme activeTheme, SubstanceTheme defaultTheme,
+			SubstanceTheme disabledTheme) {
+		this(displayName, themeKind, activeTheme, defaultTheme, disabledTheme,
+				activeTheme, defaultTheme);
+	}
+
 	/**
 	 * Creates a new complex theme.
 	 * 
@@ -184,11 +214,16 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 			SubstanceTheme disabledTheme, SubstanceTheme activeTitlePaneTheme,
 			SubstanceTheme watermarkTheme) {
 		super(defaultTheme.getColorScheme(), displayName, themeKind);
-		this.activeTheme = activeTheme;
-		this.defaultTheme = defaultTheme;
-		this.disabledTheme = disabledTheme;
-		this.activeTitlePaneTheme = activeTitlePaneTheme;
-		this.watermarkTheme = watermarkTheme;
+		this.activeTheme = SubstanceThemeUtilities
+				.getConstantTheme(activeTheme);
+		this.defaultTheme = SubstanceThemeUtilities
+				.getConstantTheme(defaultTheme);
+		this.disabledTheme = SubstanceThemeUtilities
+				.getConstantTheme(disabledTheme);
+		this.activeTitlePaneTheme = SubstanceThemeUtilities
+				.getConstantTheme(activeTitlePaneTheme);
+		this.watermarkTheme = SubstanceThemeUtilities
+				.getConstantTheme(watermarkTheme);
 		this.selectedTabFadeStart = 1.0;
 		this.selectedTabFadeEnd = 1.0;
 		this.useSameBackgroundColor = false;
@@ -197,6 +232,16 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 		this.stateThemeAlphaMap = new HashMap<ComponentState, Float>();
 		this.stateHighlightThemeMap = new HashMap<ComponentState, SubstanceTheme>();
 		this.stateHighlightThemeAlphaMap = new HashMap<ComponentState, Float>();
+		this.decoratedAreaSet = new HashSet<DecorationAreaType>();
+		this.decorationThemeMap = new HashMap<DecorationAreaType, SubstanceTheme>();
+		this.toUseDecorationOnActiveControls = new HashSet<DecorationAreaType>();
+
+		this.decorationThemeMap.put(DecorationAreaType.PRIMARY_TITLE_PANE,
+				this.activeTitlePaneTheme);
+		this.decorationThemeMap.put(DecorationAreaType.SECONDARY_TITLE_PANE,
+				this.activeTitlePaneTheme);
+		this.decoratedAreaSet.add(DecorationAreaType.PRIMARY_TITLE_PANE);
+		this.decoratedAreaSet.add(DecorationAreaType.SECONDARY_TITLE_PANE);
 	}
 
 	/*
@@ -275,7 +320,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 				.getFirstTheme(), this.getDefaultTheme().getFirstTheme(), this
 				.getDisabledTheme().getFirstTheme(), this
 				.getActiveTitlePaneTheme().getFirstTheme());
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -291,7 +336,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 				.getSecondTheme(), this.getDefaultTheme().getSecondTheme(),
 				this.getDisabledTheme().getSecondTheme(), this
 						.getActiveTitlePaneTheme().getSecondTheme());
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -316,7 +361,34 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 						saturateFactor, true) : this.getActiveTitlePaneTheme(),
 				toSaturateEverything ? this.getWatermarkTheme().saturate(
 						saturateFactor, true) : this.getWatermarkTheme());
-		this.copyTo(result);
+		this.copyTo(result, null);
+		return result;
+	}
+
+	@Override
+	public SubstanceTheme shift(Color backgroundShiftColor,
+			double backgroundShiftFactor, Color foregroundShiftColor,
+			double foregroundShiftFactor) {
+		ThemeTransform shiftTransform = new ShiftTransform(
+				backgroundShiftColor, backgroundShiftFactor,
+				foregroundShiftColor, foregroundShiftFactor);
+		ThemeTransform shiftDisabledTransform = new ShiftTransform(
+				backgroundShiftColor, backgroundShiftFactor / 2.0,
+				foregroundShiftColor, foregroundShiftFactor / 2.0);
+		// ThemeTransform shiftDisabledTransform = new ShiftDisabledTransform(
+		// backgroundShiftColor, backgroundShiftFactor,
+		// foregroundShiftColor, foregroundShiftFactor);
+		SubstanceComplexTheme result = new SubstanceComplexTheme("Shift "
+				+ this.getDisplayName() + " to [" + backgroundShiftColor + "] "
+				+ (int) (100 * backgroundShiftFactor) + "% * ["
+				+ foregroundShiftColor + "]"
+				+ (int) (100 * foregroundShiftFactor) + "%", this.getKind(),
+				shiftTransform.transform(this.getActiveTheme()), shiftTransform
+						.transform(this.getDefaultTheme()),
+				shiftDisabledTransform.transform(this.getDisabledTheme()),
+				shiftTransform.transform(this.getActiveTitlePaneTheme()),
+				shiftTransform.transform(this.getWatermarkTheme()));
+		this.copyTo(result, shiftTransform);
 		return result;
 	}
 
@@ -335,7 +407,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 						.getDisabledTheme().shade(shadeFactor), this
 						.getActiveTitlePaneTheme().shade(shadeFactor), this
 						.getWatermarkTheme().shade(shadeFactor));
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -353,7 +425,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 						.getDisabledTheme().tint(tintFactor), this
 						.getActiveTitlePaneTheme().tint(tintFactor), this
 						.getWatermarkTheme().tint(tintFactor));
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -371,7 +443,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 						.getDisabledTheme().tone(toneFactor), this
 						.getActiveTitlePaneTheme().tone(toneFactor), this
 						.getWatermarkTheme().tone(toneFactor));
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -387,7 +459,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 				.invert(), this.getDefaultTheme().invert(), this
 				.getDisabledTheme().invert(), this.getActiveTitlePaneTheme()
 				.invert(), this.getWatermarkTheme().invert());
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -403,7 +475,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 				.negate(), this.getDefaultTheme().negate(), this
 				.getDisabledTheme().negate(), this.getActiveTitlePaneTheme()
 				.negate(), this.getWatermarkTheme().negate());
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -422,7 +494,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 				hueShiftFactor), this.getActiveTitlePaneTheme().hueShift(
 				hueShiftFactor), this.getWatermarkTheme().hueShift(
 				hueShiftFactor));
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -439,7 +511,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 				.getDisabledTheme().protanopia(), this
 				.getActiveTitlePaneTheme().protanopia(), this
 				.getWatermarkTheme().protanopia());
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -456,7 +528,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 				.getDisabledTheme().tritanopia(), this
 				.getActiveTitlePaneTheme().tritanopia(), this
 				.getWatermarkTheme().tritanopia());
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -474,7 +546,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 						.getDisabledTheme().deuteranopia(), this
 						.getActiveTitlePaneTheme().deuteranopia(), this
 						.getWatermarkTheme().deuteranopia());
-		this.copyTo(result);
+		this.copyTo(result, null);
 		return result;
 	}
 
@@ -505,6 +577,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * 
 	 * @param nonActivePainter
 	 *            Painter for painting the non-active controls.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setNonActivePainter(SubstanceGradientPainter nonActivePainter) {
 		this.nonActivePainter = nonActivePainter;
@@ -516,6 +590,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param defaultTitlePaneTheme
 	 *            Theme for title panes of inactive windows.
 	 * @since version 3.3
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setDefaultTitlePaneTheme(SubstanceTheme defaultTitlePaneTheme) {
 		this.defaultTitlePaneTheme = defaultTitlePaneTheme;
@@ -527,6 +603,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param defaultTheme
 	 *            The theme for the default visual state.
 	 * @since version 4.0
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setDefaultTheme(SubstanceTheme defaultTheme) {
 		this.defaultTheme = defaultTheme;
@@ -538,6 +616,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param disabledTheme
 	 *            The theme for the disabled visual state.
 	 * @since version 4.0
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setDisabledTheme(SubstanceTheme disabledTheme) {
 		this.disabledTheme = disabledTheme;
@@ -549,6 +629,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param activeTheme
 	 *            The theme for the active visual state.
 	 * @since version 4.0
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setActiveTheme(SubstanceTheme activeTheme) {
 		this.activeTheme = activeTheme;
@@ -560,9 +642,16 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param activeTitlePaneTheme
 	 *            The theme for the title panes of active windows.
 	 * @since version 4.0
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setActiveTitlePaneTheme(SubstanceTheme activeTitlePaneTheme) {
 		this.activeTitlePaneTheme = activeTitlePaneTheme;
+
+		this.decorationThemeMap.put(DecorationAreaType.PRIMARY_TITLE_PANE,
+				activeTitlePaneTheme);
+		this.decorationThemeMap.put(DecorationAreaType.SECONDARY_TITLE_PANE,
+				activeTitlePaneTheme);
 	}
 
 	/**
@@ -571,6 +660,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param watermarkTheme
 	 *            The theme for the watermark.
 	 * @since version 4.0
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setWatermarkTheme(SubstanceTheme watermarkTheme) {
 		this.watermarkTheme = watermarkTheme;
@@ -603,6 +694,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param selectedTabFadeEnd
 	 *            The end of fade effect on the selected tabs in
 	 *            {@link JTabbedPane}s. Should be in 0.0-1.0 range.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setSelectedTabFadeEnd(double selectedTabFadeEnd) {
 		this.selectedTabFadeEnd = selectedTabFadeEnd;
@@ -615,6 +708,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param selectedTabFadeStart
 	 *            The start of fade effect on the selected tabs in
 	 *            {@link JTabbedPane}s. Should be in 0.0-1.0 range.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setSelectedTabFadeStart(double selectedTabFadeStart) {
 		this.selectedTabFadeStart = selectedTabFadeStart;
@@ -626,6 +721,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param useSameBackgroundColor
 	 *            if <code>true</code>, the light background color and the
 	 *            background color are the same.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void setUseSameBackgroundColor(boolean useSameBackgroundColor) {
 		this.useSameBackgroundColor = useSameBackgroundColor;
@@ -643,6 +740,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 *            highlight theme as well (for the same component state).
 	 * @param states
 	 *            Component states.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void registerComponentStateTheme(SubstanceTheme stateTheme,
 			float alpha, boolean useForHighlights, ComponentState... states) {
@@ -669,6 +768,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 *            highlight theme as well (for the same component state).
 	 * @param states
 	 *            Component states.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void registerComponentStateTheme(SubstanceTheme stateTheme,
 			boolean useForHighlights, ComponentState... states) {
@@ -686,6 +787,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param states
 	 *            Component states. If <code>null</code>, the specified theme
 	 *            will be applied for all states left unspecified.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void registerComponentHighlightStateTheme(
 			SubstanceTheme stateHighlightTheme, ComponentState... states) {
@@ -718,6 +821,8 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * @param states
 	 *            Component states. If <code>null</code>, the specified theme
 	 *            will be applied for all states left unspecified.
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
 	 */
 	public void registerComponentHighlightStateTheme(
 			SubstanceTheme stateHighlightTheme, float alpha,
@@ -741,15 +846,17 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 		}
 	}
 
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see org.jvnet.substance.theme.SubstanceTheme#getTheme(java.awt.Component,
-	 *      org.jvnet.substance.utils.ComponentState)
-	 */
 	@Override
 	public SubstanceTheme getTheme(Component comp, ComponentState componentState) {
-		if (SubstanceHighlightManager.getInstance().toUseHighlightTheme(comp)) {
+		return this.getTheme(comp, componentState, false);
+	}
+
+	@Override
+	public SubstanceTheme getTheme(Component comp,
+			ComponentState componentState, boolean toIgnoreHighlights) {
+		if (!toIgnoreHighlights
+				&& SubstanceHighlightManager.getInstance().toUseHighlightTheme(
+						comp)) {
 			SubstanceTheme registered = this.stateHighlightThemeMap
 					.get(componentState);
 			if (registered != null)
@@ -759,7 +866,7 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 			if (registered != null)
 				return registered;
 		}
-		return super.getTheme(comp, componentState);
+		return super.getTheme(comp, componentState, toIgnoreHighlights);
 	}
 
 	/*
@@ -813,21 +920,133 @@ public class SubstanceComplexTheme extends SubstanceTheme {
 	 * 
 	 * @param dst
 	 *            Theme to copy to.
+	 * @param shiftTransform
+	 *            Transformation to apply to the copied settings.
 	 */
-	protected void copyTo(SubstanceComplexTheme dst) {
+	protected void copyTo(SubstanceComplexTheme dst,
+			ThemeTransform shiftTransform) {
 		dst.setNonActivePainter(this.nonActivePainter);
 		dst.setSelectedTabFadeStart(this.selectedTabFadeStart);
 		dst.setSelectedTabFadeEnd(this.selectedTabFadeEnd);
 		dst.setUseSameBackgroundColor(this.useSameBackgroundColor);
 		for (Map.Entry<ComponentState, SubstanceTheme> entry : this.stateThemeMap
 				.entrySet()) {
-			dst.registerComponentStateTheme(entry.getValue(), false, entry
-					.getKey());
+			float alpha = this.stateThemeAlphaMap.get(entry.getKey());
+			if (shiftTransform == null)
+				dst.registerComponentStateTheme(entry.getValue(), alpha, false,
+						entry.getKey());
+			else
+				dst.registerComponentStateTheme(shiftTransform.transform(entry
+						.getValue()), alpha, false, entry.getKey());
 		}
 		for (Map.Entry<ComponentState, SubstanceTheme> entry : this.stateHighlightThemeMap
 				.entrySet()) {
-			dst.registerComponentHighlightStateTheme(entry.getValue(), entry
-					.getKey());
+			ComponentState state = entry.getKey();
+			if (this.stateHighlightThemeAlphaMap.containsKey(state)) {
+				float alpha = this.stateHighlightThemeAlphaMap.get(state);
+				if (shiftTransform == null)
+					dst.registerComponentHighlightStateTheme(entry.getValue(),
+							alpha, entry.getKey());
+				else
+					dst
+							.registerComponentHighlightStateTheme(
+									shiftTransform.transform(entry.getValue()),
+									alpha, entry.getKey());
+			} else {
+				if (shiftTransform == null)
+					dst.registerComponentHighlightStateTheme(entry.getValue(),
+							entry.getKey());
+				else
+					dst.registerComponentHighlightStateTheme(shiftTransform
+							.transform(entry.getValue()), entry.getKey());
+			}
 		}
 	}
+
+	/**
+	 * Registers the specified theme to be used on controls in decoration areas.
+	 * 
+	 * @param decorationTheme
+	 *            The theme to use on controls in decoration areas.
+	 * @param toUseOnActiveControls
+	 *            If <code>false</code>, the parameter theme will only be
+	 *            used on controls in default state. In this case, controls in
+	 *            disabled or active states will use the global theme.
+	 * @param areaTypes
+	 *            Enumerates the area types that are affected by the parameter
+	 *            theme.
+	 * @since version 4.3
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
+	 */
+	public void registerDecorationAreaTheme(SubstanceTheme decorationTheme,
+			boolean toUseOnActiveControls, DecorationAreaType... areaTypes) {
+		if (decorationTheme == null)
+			return;
+
+		SubstanceTheme theme = toUseOnActiveControls ? decorationTheme
+				: SubstanceThemeUtilities.getConstantTheme(decorationTheme);
+		for (DecorationAreaType areaType : areaTypes) {
+			this.decoratedAreaSet.add(areaType);
+			this.decorationThemeMap.put(areaType, theme);
+			if (toUseOnActiveControls) {
+				this.toUseDecorationOnActiveControls.add(areaType);
+			}
+		}
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see org.jvnet.substance.theme.SubstanceTheme#getDecorationTheme(org.jvnet.substance.painter.decoration.DecorationAreaType)
+	 */
+	@Override
+	public SubstanceTheme getDecorationTheme(DecorationAreaType decorationType) {
+		SubstanceTheme result = this.decorationThemeMap.get(decorationType);
+		if (result != null)
+			return result;
+		return this;
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see org.jvnet.substance.theme.SubstanceTheme#toUseDecorationPainter(org.jvnet.substance.painter.decoration.DecorationAreaType)
+	 */
+	@Override
+	public boolean toUseDecorationPainter(DecorationAreaType decorationType) {
+		return this.decoratedAreaSet.contains(decorationType);
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see org.jvnet.substance.theme.SubstanceTheme#toUseDecorationThemeOnActiveControls(org.jvnet.substance.painter.decoration.DecorationAreaType)
+	 */
+	@Override
+	public boolean toUseDecorationThemeOnActiveControls(
+			DecorationAreaType decorationType) {
+		return this.toUseDecorationOnActiveControls.contains(decorationType);
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see org.jvnet.substance.theme.SubstanceTheme#isPaintingToolbarDropShadows()
+	 */
+	@Override
+	public boolean isPaintingToolbarDropShadows() {
+		return this.isPaintingToolbarDropShadows;
+	}
+
+	/**
+	 * Marks the toolbars to have drop shadows on the top few pixels.
+	 * 
+	 * @since version 4.3
+	 * @deprecated This functionality will be moved to the base skin class in
+	 *             version 5.0.
+	 */
+	public void setPaintingToolbarDropShadows() {
+		this.isPaintingToolbarDropShadows = true;
+	}
 }
