@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -34,10 +34,10 @@ import java.awt.event.KeyEvent;
 
 import javax.swing.*;
 import javax.swing.border.EmptyBorder;
-import javax.swing.border.LineBorder;
 import javax.swing.plaf.*;
 import javax.swing.plaf.basic.BasicBorders;
 import javax.swing.plaf.basic.BasicBorders.MarginBorder;
+import javax.swing.table.JTableHeader;
 import javax.swing.text.DefaultEditorKit;
 
 import org.jvnet.lafwidget.animation.FadeKind;
@@ -45,12 +45,10 @@ import org.jvnet.substance.*;
 import org.jvnet.substance.color.*;
 import org.jvnet.substance.painter.StandardGradientPainter;
 import org.jvnet.substance.painter.SubstanceGradientPainter;
+import org.jvnet.substance.painter.decoration.DecorationAreaType;
 import org.jvnet.substance.scroll.SubstanceScrollPaneBorder;
-import org.jvnet.substance.title.SubstanceHeaderPainter;
-import org.jvnet.substance.title.SubstanceTitlePainter;
 import org.jvnet.substance.utils.*;
-import org.jvnet.substance.utils.icon.CheckBoxMenuItemIcon;
-import org.jvnet.substance.utils.icon.RadioButtonMenuItemIcon;
+import org.jvnet.substance.utils.icon.*;
 
 /**
  * Theme for <code>Substance</code> look and feel. This class is part of
@@ -258,6 +256,8 @@ public class SubstanceTheme implements SubstanceTrait {
 	 */
 	private SubstanceTheme pressedTheme;
 
+	private SubstanceTheme disabledSelectedTheme;
+
 	// /**
 	// * Contains classes of all controls that should be painted as active.
 	// */
@@ -323,82 +323,7 @@ public class SubstanceTheme implements SubstanceTrait {
 
 		Object menuArrowIcon = new UIDefaults.LazyValue() {
 			public Object createValue(UIDefaults table) {
-				// fix for defect 125 - support of RTL menus
-				return new Icon() {
-					Icon ltrIcon = SubstanceImageCreator.getArrowIcon(
-							SubstanceSizeUtils
-									.getArrowIconWidth(SubstanceSizeUtils
-											.getControlFontSize()) - 2,
-							SubstanceSizeUtils
-									.getArrowIconHeight(SubstanceSizeUtils
-											.getControlFontSize()) - 1,
-							SubstanceSizeUtils
-									.getArrowStrokeWidth(SubstanceSizeUtils
-											.getControlFontSize()),
-							SwingConstants.EAST, SubstanceCoreUtilities
-									.getActiveTheme(null, true));
-
-					Icon rtlIcon = SubstanceImageCreator.getArrowIcon(
-							SubstanceSizeUtils
-									.getArrowIconWidth(SubstanceSizeUtils
-											.getControlFontSize()) - 2,
-							SubstanceSizeUtils
-									.getArrowIconHeight(SubstanceSizeUtils
-											.getControlFontSize()) - 1,
-							SubstanceSizeUtils
-									.getArrowStrokeWidth(SubstanceSizeUtils
-											.getControlFontSize()),
-							SwingConstants.WEST, SubstanceCoreUtilities
-									.getActiveTheme(null, true));
-
-					Icon ltrIconDisabled = SubstanceImageCreator.getArrowIcon(
-							SubstanceSizeUtils
-									.getArrowIconWidth(SubstanceSizeUtils
-											.getControlFontSize()) - 2,
-							SubstanceSizeUtils
-									.getArrowIconHeight(SubstanceSizeUtils
-											.getControlFontSize()) - 1,
-							SubstanceSizeUtils
-									.getArrowStrokeWidth(SubstanceSizeUtils
-											.getControlFontSize()),
-							SwingConstants.EAST, SubstanceCoreUtilities
-									.getDisabledTheme(null, true));
-
-					Icon rtlIconDisabled = SubstanceImageCreator.getArrowIcon(
-							SubstanceSizeUtils
-									.getArrowIconWidth(SubstanceSizeUtils
-											.getControlFontSize()) - 2,
-							SubstanceSizeUtils
-									.getArrowIconHeight(SubstanceSizeUtils
-											.getControlFontSize()) - 1,
-							SubstanceSizeUtils
-									.getArrowStrokeWidth(SubstanceSizeUtils
-											.getControlFontSize()),
-							SwingConstants.WEST, SubstanceCoreUtilities
-									.getDisabledTheme(null, true));
-
-					public int getIconHeight() {
-						return 7;
-					}
-
-					public int getIconWidth() {
-						return 5;
-					}
-
-					public void paintIcon(Component c, Graphics g, int x, int y) {
-						if (c.getComponentOrientation().isLeftToRight()) {
-							if (c.isEnabled())
-								this.ltrIcon.paintIcon(c, g, x, y);
-							else
-								this.ltrIconDisabled.paintIcon(c, g, x, y);
-						} else {
-							if (c.isEnabled())
-								this.rtlIcon.paintIcon(c, g, x, y);
-							else
-								this.rtlIconDisabled.paintIcon(c, g, x, y);
-						}
-					}
-				};
+				return new MenuArrowIcon(null);
 			}
 		};
 
@@ -411,22 +336,17 @@ public class SubstanceTheme implements SubstanceTrait {
 		Color controlText = new ColorUIResource(this.colorScheme
 				.getLightColor());
 		final Color foregroundColor = this.getForegroundColor();
-		// final Color menuForegroundColor = new ColorUIResource(
-		// SubstanceColorUtilities.getMenuForegroundColor(this));
+
 		Color backgroundColor = new ColorUIResource(this.getBackgroundColor());
-		// Color menuBackgroundColor = new
-		// ColorUIResource(SubstanceColorUtilities
-		// .getBackgroundColor(this.getActiveTitlePaneTheme()));
 		Color backgroundLightColor = new ColorUIResource(this
 				.getLightBackgroundColor());
 
 		Color disabledForegroundColor = new ColorUIResource(this
-				.getDisabledForegroundColor());
-		// Color menuDisabledForegroundColor = new ColorUIResource(
-		// SubstanceColorUtilities.getDisabledForegroundColor(this
-		// .getActiveTitlePaneTheme()));
-		Color disabledBackgroundColor = new ColorUIResource(this
-				.getDisabledBackgroundColor());
+				.getDisabledTheme().getForegroundColor());
+		//
+		// new ColorUIResource(this.getDisabledForegroundColor());
+		// Color disabledBackgroundColor = new ColorUIResource(this
+		// .getDisabledBackgroundColor());
 
 		Color lineColor = new ColorUIResource(this.getLineColor());
 
@@ -442,27 +362,22 @@ public class SubstanceTheme implements SubstanceTrait {
 		Color selectionForegroundColor = new ColorUIResource(this
 				.getSelectionForegroundColor());
 
-		// Color selectionCellBackgroundColor = new ColorUIResource(
-		// SubstanceColorUtilities.getSelectionBackgroundColor(this.getHighlightBackgroundTheme()));
-		//
 		Color selectionCellForegroundColor = new ColorUIResource(this
 				.getSelectionCellForegroundColor());
 
 		Object regularMarginBorder = new BorderUIResource.CompoundBorderUIResource(
 				new SubstanceBorder(), new BasicBorders.MarginBorder());
 
-		Object regularBorder = new SubstanceBorder();
-
 		Object textBorder = new BorderUIResource.CompoundBorderUIResource(
 				new SubstanceBorder(SubstanceSizeUtils
 						.getTextBorderInsets(SubstanceSizeUtils
 								.getControlFontSize())),
 				new BasicBorders.MarginBorder());
 
-		// Object comboBorder = new BorderUIResource.CompoundBorderUIResource(
-		// new SubstanceBorder(new Insets(1, 2, 1, 2)),
-		// new BasicBorders.MarginBorder());
-		//
+		Object tooltipBorder = new SubstanceBorder(
+				SubstanceSizeUtils.getToolTipBorderInsets(SubstanceSizeUtils
+						.getControlFontSize()));
+
 		Object comboBorder = new SubstanceBorder(SubstanceSizeUtils
 				.getComboBorderInsets(SubstanceSizeUtils.getControlFontSize()));
 
@@ -479,13 +394,12 @@ public class SubstanceTheme implements SubstanceTrait {
 		// special handling of mixed dark themes
 		final SubstanceTheme topDecorationTheme = this
 				.getActiveTitlePaneTheme();
-		// (this.getKind() == ThemeKind.DARK_MIXED) ? ((SubstanceMixDarkBiTheme)
-		// this)
-		// .getOriginalDarkTheme() : this;
 
-		SubstanceTitlePainter titlePainter = SubstanceLookAndFeel
-				.getCurrentTitlePainter();
-		boolean shouldUseHeaderPainter = titlePainter instanceof SubstanceHeaderPainter;
+		int menuItemMargin = SubstanceSizeUtils
+				.getMenuItemMargin(SubstanceSizeUtils
+						.getComponentFontSize(null));
+		Insets menuItemInsets = new InsetsUIResource(menuItemMargin,
+				menuItemMargin, menuItemMargin, menuItemMargin);
 
 		Object fieldInputMap = new UIDefaults.LazyInputMap(new Object[] {
 				"ctrl C", DefaultEditorKit.copyAction,
@@ -793,7 +707,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				new InsetsUIResource(0, 0, 0, 0),
 
 				"CheckBox.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JCheckBox.class, this),
 
 				"CheckBox.border",
 				new BorderUIResource.CompoundBorderUIResource(
@@ -813,7 +728,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"CheckBoxMenuItem.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JCheckBoxMenuItem.class, this),
 
 				"CheckBoxMenuItem.borderPainted",
 				Boolean.FALSE,
@@ -836,11 +752,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				"CheckBoxMenuItem.foreground",
 				foregroundColor,
 
+				"CheckBoxMenuItem.margin",
+				menuItemInsets,
+
 				"CheckBoxMenuItem.selectionForeground",
 				selectionCellForegroundColor,
 
 				"ColorChooser.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JColorChooser.class, this),
 
 				"ColorChooser.foreground",
 				foregroundColor,
@@ -878,7 +798,8 @@ public class SubstanceTheme implements SubstanceTrait {
 						"KP_UP", "selectPrevious" }),
 
 				"ComboBox.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JComboBox.class, this),
 
 				"ComboBox.border",
 				comboBorder,
@@ -911,10 +832,12 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"Dialog.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JDialog.class,
+						this),
 
 				"EditorPane.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JEditorPane.class, this),
 
 				"EditorPane.border",
 				textBorder,
@@ -926,10 +849,12 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"EditorPane.disabledBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JEditorPane.class, this.getDisabledTheme()),
 
 				"EditorPane.inactiveBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JEditorPane.class, this.getDisabledTheme()),
 
 				"EditorPane.inactiveForeground",
 				disabledForegroundColor,
@@ -1054,7 +979,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				},
 
 				"FormattedTextField.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JFormattedTextField.class, this),
 
 				"FormattedTextField.border",
 				textBorder,
@@ -1063,13 +989,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"FormattedTextField.disabledBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JFormattedTextField.class, this.getDisabledTheme()),
 
 				"FormattedTextField.foreground",
 				foregroundColor,
 
 				"FormattedTextField.inactiveBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JFormattedTextField.class, this.getDisabledTheme()),
 
 				"FormattedTextField.inactiveForeground",
 				disabledForegroundColor,
@@ -1138,7 +1066,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				},
 
 				"Label.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JLabel.class,
+						this),
 
 				"Label.foreground",
 				foregroundColor,
@@ -1150,13 +1079,18 @@ public class SubstanceTheme implements SubstanceTrait {
 				disabledForegroundColor,
 
 				"List.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JList.class,
+						this),
 
 				"List.cellRenderer",
 				listCellRendererActiveValue,
 
 				"List.focusCellHighlightBorder",
-				regularBorder,
+				new SubstanceBorder(new Insets(1, 1, 1, 1)),
+
+				"List.focusSelectedCellHighlightBorder",
+				new BorderUIResource.EmptyBorderUIResource(1, 1, 1, 1),
+				// new SubstanceBorder(new Insets(1, 1, 1, 1)),
 
 				"List.foreground",
 				foregroundColor,
@@ -1171,7 +1105,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				menuArrowIcon,
 
 				"Menu.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JMenu.class,
+						this),
 
 				"Menu.borderPainted",
 				Boolean.FALSE,
@@ -1185,18 +1120,29 @@ public class SubstanceTheme implements SubstanceTrait {
 				"Menu.foreground",
 				foregroundColor,
 
+				"Menu.margin",
+				menuItemInsets,
+
 				"Menu.selectionForeground",
 				selectionCellForegroundColor,
 
 				"MenuBar.background",
-				shouldUseHeaderPainter ? new ColorUIResource(this
-						.getActiveTitlePaneTheme().getColorScheme()
-						.getMidColor()) : backgroundColor,
+				this.toUseDecorationPainter(DecorationAreaType.HEADER) ? new ColorUIResource(
+						this.getDecorationTheme(DecorationAreaType.HEADER)
+								.getColorScheme().getMidColor())
+						: SubstanceCoreUtilities.getDefaultBackgroundColor(
+								JMenuBar.class, this),
+				// shouldUseHeaderPainter ? new ColorUIResource(this
+				// .getActiveTitlePaneTheme().getColorScheme()
+				// .getMidColor()) : backgroundColor,
 
 				"MenuBar.foreground",
-				shouldUseHeaderPainter ? new ColorUIResource(this
-						.getActiveTitlePaneTheme().getForegroundColor())
-						: foregroundColor,
+				new ColorUIResource(this.getDecorationTheme(
+						DecorationAreaType.HEADER).getColorScheme()
+						.getForegroundColor()),
+				// shouldUseHeaderPainter ? new ColorUIResource(this
+				// .getActiveTitlePaneTheme().getForegroundColor())
+				// : foregroundColor,
 
 				"MenuBar.border",
 				null,
@@ -1208,7 +1154,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"MenuItem.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JMenuItem.class, this),
 
 				"MenuItem.borderPainted",
 				Boolean.FALSE,
@@ -1222,11 +1169,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				"MenuItem.foreground",
 				foregroundColor,
 
+				"MenuItem.margin",
+				menuItemInsets,
+
 				"MenuItem.selectionForeground",
 				selectionCellForegroundColor,
 
 				"OptionPane.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JOptionPane.class, this),
 
 				"OptionPane.errorIcon",
 				new UIDefaults.LazyValue() {
@@ -1267,13 +1218,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				},
 
 				"Panel.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JPanel.class,
+						this),
 
 				"Panel.foreground",
 				foregroundColor,
 
 				"PasswordField.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JPasswordField.class, this),
 
 				"PasswordField.border",
 				textBorder,
@@ -1282,13 +1235,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"PasswordField.disabledBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JPasswordField.class, this.getDisabledTheme()),
 
 				"PasswordField.foreground",
 				foregroundColor,
 
 				"PasswordField.inactiveBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JPasswordField.class, this.getDisabledTheme()),
 
 				"PasswordField.inactiveForeground",
 				disabledForegroundColor,
@@ -1300,7 +1255,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				selectionForegroundColor,
 
 				"PopupMenu.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JPopupMenu.class, this),
 
 				"PopupMenu.border",
 				regularMarginBorder,
@@ -1329,7 +1285,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"RadioButton.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JRadioButton.class, this),
 
 				"RadioButton.border",
 				new BorderUIResource.CompoundBorderUIResource(
@@ -1351,7 +1308,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"RadioButtonMenuItem.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JRadioButtonMenuItem.class, this),
 
 				"RadioButtonMenuItem.borderPainted",
 				Boolean.FALSE,
@@ -1367,17 +1325,22 @@ public class SubstanceTheme implements SubstanceTrait {
 				"RadioButtonMenuItem.foreground",
 				foregroundColor,
 
+				"RadioButtonMenuItem.margin",
+				menuItemInsets,
+
 				"RadioButtonMenuItem.selectionForeground",
 				selectionCellForegroundColor,
 
 				"RootPane.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JRootPane.class, this),
 
 				"RootPane.border",
 				new SubstancePaneBorder(),
 
 				"ScrollBar.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JScrollBar.class, this),
 
 				"ScrollBar.width",
 				new Integer(SubstanceSizeUtils
@@ -1392,7 +1355,8 @@ public class SubstanceTheme implements SubstanceTrait {
 								.getControlFontSize()) - 2),
 
 				"ScrollPane.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JScrollPane.class, this),
 
 				"ScrollPane.foreground",
 				foregroundColor,
@@ -1410,7 +1374,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				lineColor,
 
 				"Slider.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JSlider.class,
+						this),
 
 				"Slider.darkShadow",
 				lineColor,
@@ -1450,14 +1415,20 @@ public class SubstanceTheme implements SubstanceTrait {
 				// },
 
 				"Spinner.arrowButtonInsets",
-				null,
+				SubstanceSizeUtils
+						.getSpinnerArrowButtonInsets(SubstanceSizeUtils
+								.getControlFontSize()),
 
 				"Spinner.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JSpinner.class, this),
 
 				"Spinner.border",
 				spinnerBorder,
 
+				"Spinner.disableOnBoundaryValues",
+				Boolean.TRUE,
+
 				"Spinner.foreground",
 				foregroundColor,
 
@@ -1468,7 +1439,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				// SubstanceSizeUtils.getSpinnerArrowButtonInsets(),
 
 				"SplitPane.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JSplitPane.class, this),
 
 				"SplitPane.foreground",
 				foregroundColor,
@@ -1499,7 +1471,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				backgroundColor,
 
 				"TabbedPane.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTabbedPane.class, this),
 
 				"TabbedPane.borderHightlightColor",
 				new ColorUIResource(activeScheme.getMidColor()),
@@ -1552,10 +1525,11 @@ public class SubstanceTheme implements SubstanceTrait {
 				new Integer(0),
 
 				"Table.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JTable.class,
+						this),
 
 				"Table.focusCellBackground",
-				selectionBackgroundColor,
+				backgroundColor,
 
 				"Table.focusCellForeground",
 				foregroundColor,
@@ -1575,7 +1549,7 @@ public class SubstanceTheme implements SubstanceTrait {
 				lineColorDefault,
 
 				"Table.scrollPaneBorder",
-				regularMarginBorder,
+				new SubstanceScrollPaneBorder(),
 
 				"Table.selectionBackground",
 				selectionBackgroundColor,
@@ -1590,10 +1564,12 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"TableHeader.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTableHeader.class, this),
 
 				"TextArea.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextArea.class, this),
 
 				"TextArea.border",
 				textBorder,
@@ -1602,13 +1578,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"TextArea.disabledBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextArea.class, this.getDisabledTheme()),
 
 				"TextArea.foreground",
 				foregroundColor,
 
 				"TextArea.inactiveBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextArea.class, this.getDisabledTheme()),
 
 				"TextArea.inactiveForeground",
 				disabledForegroundColor,
@@ -1620,7 +1598,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				selectionForegroundColor,
 
 				"TextField.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextField.class, this),
 
 				"TextField.border",
 				textBorder,
@@ -1629,13 +1608,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"TextField.disabledBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextField.class, this.getDisabledTheme()),
 
 				"TextField.foreground",
 				foregroundColor,
 
 				"TextField.inactiveBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextField.class, this.getDisabledTheme()),
 
 				"TextField.inactiveForeground",
 				disabledForegroundColor,
@@ -1647,13 +1628,15 @@ public class SubstanceTheme implements SubstanceTrait {
 				selectionForegroundColor,
 
 				"TextPane.background",
-				backgroundLightColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextPane.class, this),
 
 				"TextPane.border",
 				textBorder,
 
 				"TextPane.disabledBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextPane.class, this.getDisabledTheme()),
 
 				"TextPane.foreground",
 				foregroundColor,
@@ -1662,7 +1645,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"TextPane.inactiveBackground",
-				disabledBackgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextPane.class, this.getDisabledTheme()),
 
 				"TextPane.inactiveForeground",
 				disabledForegroundColor,
@@ -1689,7 +1673,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				new InsetsUIResource(0, 0, 0, 0),
 
 				"ToolBar.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JToolBar.class, this),
 
 				"ToolBar.border",
 				new BorderUIResource(new SubstanceToolBarBorder()),
@@ -1701,7 +1686,8 @@ public class SubstanceTheme implements SubstanceTrait {
 				foregroundColor,
 
 				"ToolBarSeparator.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JToolBar.class, this),
 
 				"ToolBarSeparator.foreground",
 				lineBwColor,
@@ -1710,114 +1696,53 @@ public class SubstanceTheme implements SubstanceTrait {
 				null,
 
 				"ToolTip.border",
-				new UIDefaults.LazyValue() {
-					public Object createValue(UIDefaults table) {
-						return new LineBorder(foregroundColor);
-					}
-				},
+				tooltipBorder,
 
 				"ToolTip.borderInactive",
-				regularMarginBorder,
+				tooltipBorder,
 
 				"ToolTip.background",
-				backgroundColor,
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JToolTip.class, this),
 
 				"ToolTip.backgroundInactive",
-				new ColorUIResource(getDefaultColorScheme(this.kind)
-						.getExtraLightColor()),
+				SubstanceCoreUtilities.getDefaultBackgroundColor(
+						JTextPane.class, this.getDisabledTheme()),
 
-				"ToolTip.foreground", foregroundColor,
+				"ToolTip.foreground",
+				foregroundColor,
 
-				"ToolTip.foregroundInactive", disabledForegroundColor,
+				"ToolTip.foregroundInactive",
+				disabledForegroundColor,
 
-				"Tree.closedIcon", emptyIcon,
-				// new UIDefaults.LazyValue() {
-				// public Object createValue(UIDefaults table) {
-				// return new IconUIResource(
-				// new ImageIcon(
-				// SubstanceImageCreator
-				// .getThemeImage(
-				// SubstanceCoreUtilities
-				// .getIcon("resource/folder_go.png"),
-				// SubstanceLookAndFeel
-				// .getTheme(),
-				// true)));
-				// }
-				// },
+				"Tree.closedIcon",
+				emptyIcon,
 
-				"Tree.collapsedIcon", new UIDefaults.LazyValue() {
+				"Tree.collapsedIcon",
+				new UIDefaults.LazyValue() {
 					public Object createValue(UIDefaults table) {
-						return new IconUIResource(
-								new ImageIcon(
-										SubstanceImageCreator
-												.getTreeIcon(
-														SubstanceSizeUtils
-																.getControlFontSize(),
-														null,
-														SubstanceTheme.this
-																.getDefaultTheme()
-																.getColorScheme(),
-														SubstanceCoreUtilities
-																.isThemeDark(SubstanceTheme.this),
-														true)));
+						return new IconUIResource(SubstanceIconFactory
+								.getTreeIcon(null, true));
 					}
 				},
 
-				"Tree.expandedIcon", new UIDefaults.LazyValue() {
+				"Tree.expandedIcon",
+				new UIDefaults.LazyValue() {
 					public Object createValue(UIDefaults table) {
-						return new IconUIResource(
-								new ImageIcon(
-										SubstanceImageCreator
-												.getTreeIcon(
-														SubstanceSizeUtils
-																.getControlFontSize(),
-														null,
-														SubstanceTheme.this
-																.getDefaultTheme()
-																.getColorScheme(),
-														SubstanceCoreUtilities
-																.isThemeDark(SubstanceTheme.this),
-														false)));
+						return new IconUIResource(SubstanceIconFactory
+								.getTreeIcon(null, false));
 					}
 				},
 
-				"Tree.leafIcon", emptyIcon,
-				// new UIDefaults.LazyValue() {
-				// public Object createValue(UIDefaults table) {
-				// return new IconUIResource(
-				// new ImageIcon(
-				// SubstanceImageCreator
-				// .getThemeImage(
-				// SubstanceCoreUtilities
-				// .getIcon("resource/page.png"),
-				// SubstanceLookAndFeel
-				// .getTheme(),
-				// true)));
-				// }
-				// },
+				"Tree.leafIcon",
+				emptyIcon,
 
-				"Tree.openIcon", emptyIcon,
-				// new UIDefaults.LazyValue() {
-				// public Object createValue(UIDefaults table) {
-				// return new IconUIResource(
-				// new ImageIcon(
-				// SubstanceImageCreator
-				// .getThemeImage(
-				// SubstanceCoreUtilities
-				// .getIcon("resource/folder.png"),
-				// SubstanceLookAndFeel
-				// .getTheme(),
-				// true)));
-				// // new ImageIcon(
-				// // SubstanceImageCreator
-				// // .getTreeNodeIcon(
-				// // SubstanceCoreUtilities
-				// // .isThemeDark(SubstanceTheme.this),
-				// // SubstanceConstants.TreeIcon.OPENED)));
-				// }
-				// },
+				"Tree.openIcon",
+				emptyIcon,
 
-				"Tree.background", backgroundColor,
+				"Tree.background",
+				SubstanceCoreUtilities.getDefaultBackgroundColor(JTree.class,
+						this),
 
 				"Tree.selectionBackground", backgroundLightColor,
 
@@ -1917,7 +1842,11 @@ public class SubstanceTheme implements SubstanceTrait {
 	 * @return The disabled background color for this theme.
 	 * 
 	 * @since version 4.0
+	 * @deprecated Will be removed in version 5.0. Use
+	 *             {@link #getDisabledTheme()} and
+	 *             {@link SubstanceTheme#getBackgroundColor()}.
 	 */
+	@Deprecated
 	public Color getDisabledBackgroundColor() {
 		return SubstanceColorUtilities.getInterpolatedColor(this
 				.getBackgroundColor(), this.getLightBackgroundColor(), 0.5);
@@ -1929,7 +1858,11 @@ public class SubstanceTheme implements SubstanceTrait {
 	 * @return The disabled foreground color for this theme.
 	 * 
 	 * @since version 4.0
+	 * @deprecated Will be removed in version 5.0. Use
+	 *             {@link #getDisabledTheme()} and
+	 *             {@link SubstanceTheme#getForegroundColor()}.
 	 */
+	@Deprecated
 	public Color getDisabledForegroundColor() {
 		return SubstanceColorUtilities.getDisabledForegroundColor(this);
 	}
@@ -2097,8 +2030,6 @@ public class SubstanceTheme implements SubstanceTrait {
 	/**
 	 * Returns the theme that corresponds to the specified component state.
 	 * 
-	 * @param comp
-	 *            Component.
 	 * @param componentState
 	 *            Component state.
 	 * @return The theme that corresponds to the specified component state.
@@ -2106,6 +2037,11 @@ public class SubstanceTheme implements SubstanceTrait {
 	 * @since version 4.0
 	 */
 	public SubstanceTheme getTheme(Component comp, ComponentState componentState) {
+		return this.getTheme(comp, componentState, false);
+	}
+
+	public SubstanceTheme getTheme(Component comp,
+			ComponentState componentState, boolean toIgnoreHighlights) {
 		if (componentState.isKindActive(FadeKind.PRESS)) {
 			if (this.pressedTheme == null) {
 				this.pressedTheme = this.getActiveTheme().shade(0.2).saturate(
@@ -2113,6 +2049,14 @@ public class SubstanceTheme implements SubstanceTrait {
 			}
 			return this.pressedTheme;
 		}
+		if (componentState == ComponentState.DISABLED_SELECTED) {
+			if (this.disabledSelectedTheme == null) {
+				this.disabledSelectedTheme = new SubstanceBlendBiTheme(this
+						.getActiveTheme(), this.getDisabledTheme(), 0.25);
+			}
+			return this.disabledSelectedTheme;
+		}
+
 		switch (componentState.getColorSchemeKind()) {
 		case CURRENT:
 			return this.getActiveTheme();
@@ -2459,6 +2403,30 @@ public class SubstanceTheme implements SubstanceTrait {
 		}
 	}
 
+	/**
+	 * Creates a shift version of <code>this</code> theme. This method is part
+	 * of officially supported API.
+	 * 
+	 * @param backgroundShiftColor
+	 *            Shift color for background colors. Should have full opacity.
+	 * @param backgroundShiftFactor
+	 *            Value in 0.0...1.0 range. Larger values shift more towards the
+	 *            specified color.
+	 * @param foregroundShiftColor
+	 *            Shift color for foreground colors. Should have full opacity.
+	 * @param foregroundShiftFactor
+	 *            Value in 0.0...1.0 range. Larger values shift more towards the
+	 *            specified color.
+	 * @return Shift version of <code>this</code> theme.
+	 */
+	public SubstanceTheme shift(Color backgroundShiftColor,
+			double backgroundShiftFactor, Color foregroundShiftColor,
+			double foregroundShiftFactor) {
+		return new SubstanceShiftTheme(this, backgroundShiftColor,
+				backgroundShiftFactor, foregroundShiftColor,
+				foregroundShiftFactor);
+	}
+
 	/**
 	 * Creates a tinted (shifted towards white) version of <code>this</code>
 	 * theme. This method is part of officially supported API.
@@ -2734,4 +2702,27 @@ public class SubstanceTheme implements SubstanceTrait {
 	public double getSelectedTabFadeEnd() {
 		return 0.3;
 	}
+
+	public SubstanceTheme getDecorationTheme(DecorationAreaType decorationType) {
+		if (decorationType == DecorationAreaType.PRIMARY_TITLE_PANE
+				|| decorationType == DecorationAreaType.SECONDARY_TITLE_PANE)
+			return this;
+		return this.getDefaultTheme();
+	}
+
+	public boolean toUseDecorationPainter(DecorationAreaType decorationType) {
+		if (decorationType == DecorationAreaType.PRIMARY_TITLE_PANE
+				|| decorationType == DecorationAreaType.SECONDARY_TITLE_PANE)
+			return true;
+		return false;
+	}
+
+	public boolean toUseDecorationThemeOnActiveControls(
+			DecorationAreaType decorationType) {
+		return false;
+	}
+
+	public boolean isPaintingToolbarDropShadows() {
+		return false;
+	}
 }
