@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -43,7 +43,6 @@ import org.jvnet.substance.SubstanceLookAndFeel;
 import org.jvnet.substance.border.SubstanceBorderPainter;
 import org.jvnet.substance.button.*;
 import org.jvnet.substance.color.ColorScheme;
-import org.jvnet.substance.combo.SubstanceComboBoxButton;
 import org.jvnet.substance.painter.*;
 import org.jvnet.substance.theme.SubstanceTheme;
 import org.jvnet.substance.utils.ComponentState.ColorSchemeKind;
@@ -155,8 +154,7 @@ public class ButtonBackgroundDelegate {
 		}
 
 		// compute color scheme
-		SubstanceTheme theme = SubstanceCoreUtilities.getTheme(button, state,
-				true, true);
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(button, state);
 		ColorScheme colorScheme = theme.getColorScheme();
 		ColorScheme colorScheme2 = colorScheme;
 		ColorScheme borderScheme = theme.getBorderTheme().getColorScheme();
@@ -208,8 +206,8 @@ public class ButtonBackgroundDelegate {
 			FadeState fadeState = SubstanceFadeUtilities.getFadeState(button,
 					FadeKind.ROLLOVER, FadeKind.SELECTION, FadeKind.PRESS);
 			if (fadeState != null) {
-				SubstanceTheme prevTheme = SubstanceCoreUtilities.getTheme(
-						button, prevState, true, true);
+				SubstanceTheme prevTheme = SubstanceThemeUtilities.getTheme(
+						button, prevState);
 				colorScheme = theme.getColorScheme();
 				colorScheme2 = prevTheme.getColorScheme();
 				borderScheme = theme.getBorderTheme().getColorScheme();
@@ -221,6 +219,13 @@ public class ButtonBackgroundDelegate {
 			}
 		}
 
+		// // special handling of disabled unselected buttons
+		// if (state == ComponentState.DISABLED_SELECTED) {
+		// colorScheme2 = SubstanceThemeUtilities.getTheme(button,
+		// ComponentState.SELECTED).getColorScheme();
+		// cyclePos = 3.0f;
+		// }
+		//
 		// compute a straight side
 		Set<SubstanceConstants.Side> straightSides = SubstanceCoreUtilities
 				.getSides(button, SubstanceLookAndFeel.BUTTON_SIDE_PROPERTY);
@@ -254,7 +259,8 @@ public class ButtonBackgroundDelegate {
 				+ ":" + borderPainter.getDisplayName() + ":" + straightKey
 				+ ":" + openKey + ":" + button.getClass().getName() + ":"
 				+ isRoundButton + ":" + radius + ":" + isContentAreaFilled
-				+ ":" + isBorderPainted + ":" + button.getFont().getSize();
+				+ ":" + isBorderPainted + ":"
+				+ SubstanceSizeUtils.getComponentFontSize(button);
 
 		if (regularBackgrounds.get(key) == null) {
 			BufferedImage newBackground;
@@ -516,17 +522,6 @@ public class ButtonBackgroundDelegate {
 			int width = button.getWidth();
 			int height = button.getHeight();
 			int y = 0;
-			if (button instanceof SubstanceComboBoxButton) {
-				Component parent = button.getParent();
-				if (parent instanceof JComboBox) {
-					JComboBox jcb = (JComboBox) parent;
-					if (jcb.getBorder() != null) {
-						y++;
-						height -= 2;
-					}
-				}
-			}
-
 			if (SubstanceCoreUtilities.isScrollButton(button)) {
 				Sideable sideable = (Sideable) button;
 				PairwiseButtonBackgroundDelegate.updateBackground(g, button,
@@ -544,7 +539,7 @@ public class ButtonBackgroundDelegate {
 			SubstanceButtonShaper shaper = SubstanceCoreUtilities
 					.getButtonShaper(button);
 			SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-					.getBorderPainter(button, painter);
+					.getBorderPainter(button);
 
 			// if (MemoryAnalyzer.isRunning())
 			// MemoryAnalyzer.enqueueUsage("For '" + button.getText()
@@ -615,13 +610,10 @@ public class ButtonBackgroundDelegate {
 							tempGr.drawImage(bgImage, 0, 0, null);
 					} else {
 						if (!state.isKindActive(FadeKind.ENABLE)) {
-							tempGr
-									.setComposite(AlphaComposite.getInstance(
-											AlphaComposite.SRC_OVER,
-											SubstanceCoreUtilities.getTheme(
-													button, true)
-													.getThemeAlpha(button,
-															state)));
+							tempGr.setComposite(AlphaComposite.getInstance(
+									AlphaComposite.SRC_OVER,
+									SubstanceThemeUtilities.getTheme(button)
+											.getThemeAlpha(button, state)));
 						}
 						tempGr.drawImage(bgImage, 0, 0, null);
 					}
