@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -141,8 +141,7 @@ public class PairwiseButtonBackgroundDelegate {
 		ControlBackgroundComposite controlComposite = SubstanceCoreUtilities
 				.getControlBackgroundComposite(button);
 
-		SubstanceTheme theme = SubstanceCoreUtilities.getTheme(button, state,
-				true, false);
+		SubstanceTheme theme = SubstanceThemeUtilities.getTheme(button, state);
 		ColorScheme colorScheme = theme.getColorScheme();
 		ColorScheme colorScheme2 = colorScheme;
 		ColorScheme borderScheme = theme.getBorderTheme().getColorScheme();
@@ -159,12 +158,14 @@ public class PairwiseButtonBackgroundDelegate {
 					.getBackgroundComposite(button, button.getParent(), -1,
 							true);
 
-			colorScheme = SubstanceCoreUtilities.getScheme(button, prevState);
-			colorScheme2 = SubstanceCoreUtilities.getScheme(button, state);
-			borderScheme = SubstanceCoreUtilities.getTheme(button, state, true,
-					false).getBorderTheme().getColorScheme();
-			borderScheme2 = SubstanceCoreUtilities.getTheme(button, prevState,
-					true, false).getBorderTheme().getColorScheme();
+			colorScheme = SubstanceThemeUtilities.getTheme(button, prevState)
+					.getColorScheme();
+			colorScheme2 = SubstanceThemeUtilities.getTheme(button, state)
+					.getColorScheme();
+			borderScheme = SubstanceThemeUtilities.getTheme(button, state)
+					.getBorderTheme().getColorScheme();
+			borderScheme2 = SubstanceThemeUtilities.getTheme(button, prevState)
+					.getBorderTheme().getColorScheme();
 			cyclePos = fadeState.getFadePosition();
 			if (!fadeState.isFadingIn())
 				cyclePos = 10 - cyclePos;
@@ -237,6 +238,9 @@ public class PairwiseButtonBackgroundDelegate {
 				tempGr.dispose();
 			}
 		} else {
+			resultGr.setComposite(AlphaComposite.getInstance(
+					AlphaComposite.SRC_OVER, SubstanceThemeUtilities.getTheme(
+							button).getThemeAlpha(button, state)));
 			resultGr.drawImage(resultNonFlat, 0, 0, null);
 		}
 		return result;
@@ -291,6 +295,8 @@ public class PairwiseButtonBackgroundDelegate {
 		for (Side oSide : openSides) {
 			openKey += oSide.name() + "-";
 		}
+		boolean noBorder = SubstanceCoreUtilities.isSpinnerButton(button)
+				&& !button.getParent().isEnabled();
 		String sideKey = (side == null) ? "null" : side.toString();
 		String key = width + ":" + height + ":" + sideKey + ":" + cyclePos
 				+ ":" + openKey + ":"
@@ -298,7 +304,8 @@ public class PairwiseButtonBackgroundDelegate {
 				+ SubstanceCoreUtilities.getSchemeId(colorScheme2) + ":"
 				+ SubstanceCoreUtilities.getSchemeId(borderScheme) + ":"
 				+ SubstanceCoreUtilities.getSchemeId(borderScheme2) + ":"
-				+ button.getClass().getName() + ":" + painter.getDisplayName();
+				+ button.getClass().getName() + ":" + painter.getDisplayName()
+				+ ":" + noBorder;
 		// System.out.println("\tKey " + key);
 		if (!pairwiseBackgrounds.containsKey(key)) {
 			// System.out.println("\tNot found");
@@ -318,7 +325,7 @@ public class PairwiseButtonBackgroundDelegate {
 			// SubstanceGradientPainter painter = new StandardGradientPainter();
 
 			SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-					.getBorderPainter(button, painter);
+					.getBorderPainter(button);
 
 			int borderDelta = (int) Math.floor(SubstanceSizeUtils
 					.getBorderStrokeWidth(SubstanceSizeUtils
@@ -337,11 +344,13 @@ public class PairwiseButtonBackgroundDelegate {
 							+ deltaRight, contour, false, colorScheme,
 							colorScheme2, cyclePos, true,
 							colorScheme != colorScheme2);
-					borderPainter.paintBorder(newBackground.getGraphics(),
-							button, height + deltaTop + deltaBottom, width
-									+ deltaLeft + deltaRight, contour, null,
-							borderScheme, borderScheme2, cyclePos,
-							borderScheme != borderScheme2);
+					if (!noBorder) {
+						borderPainter.paintBorder(newBackground.getGraphics(),
+								button, height + deltaTop + deltaBottom, width
+										+ deltaLeft + deltaRight, contour,
+								null, borderScheme, borderScheme2, cyclePos,
+								borderScheme != borderScheme2);
+					}
 					newBackground = SubstanceImageCreator.getRotated(
 							newBackground, 3);
 					break;
@@ -357,11 +366,13 @@ public class PairwiseButtonBackgroundDelegate {
 							+ deltaBottom, contour, false, colorScheme,
 							colorScheme2, cyclePos, true,
 							colorScheme != colorScheme2);
-					borderPainter.paintBorder(newBackground.getGraphics(),
-							button, width + deltaLeft + deltaRight, height
-									+ deltaTop + deltaBottom, contour, null,
-							borderScheme, borderScheme2, cyclePos,
-							borderScheme != borderScheme2);
+					if (!noBorder) {
+						borderPainter.paintBorder(newBackground.getGraphics(),
+								button, width + deltaLeft + deltaRight, height
+										+ deltaTop + deltaBottom, contour,
+								null, borderScheme, borderScheme2, cyclePos,
+								borderScheme != borderScheme2);
+					}
 					break;
 				}
 			} else {
@@ -373,10 +384,13 @@ public class PairwiseButtonBackgroundDelegate {
 						+ deltaRight, height + deltaTop + deltaBottom, contour,
 						false, colorScheme, colorScheme2, cyclePos, true,
 						colorScheme != colorScheme2);
-				borderPainter.paintBorder(newBackground.getGraphics(), button,
-						width + deltaLeft + deltaRight, height + deltaTop
-								+ deltaBottom, contour, null, borderScheme,
-						borderScheme2, cyclePos, borderScheme != borderScheme2);
+				if (!noBorder) {
+					borderPainter.paintBorder(newBackground.getGraphics(),
+							button, width + deltaLeft + deltaRight, height
+									+ deltaTop + deltaBottom, contour, null,
+							borderScheme, borderScheme2, cyclePos,
+							borderScheme != borderScheme2);
+				}
 			}
 
 			BufferedImage finalBackground = SubstanceCoreUtilities
@@ -423,63 +437,17 @@ public class PairwiseButtonBackgroundDelegate {
 	public static void updateBackground(Graphics g, AbstractButton button,
 			SubstanceConstants.Side side) {
 		Graphics2D graphics = (Graphics2D) g.create();
+
 		SubstanceGradientPainter painter = SubstanceCoreUtilities
 				.getGradientPainter(button);
 		int width = button.getWidth();
 		int height = button.getHeight();
-		// BufferedImage result = SubstanceCoreUtilities.getBlankImage(width,
-		// height);
-		// Graphics2D resultGr = (Graphics2D) result.createGraphics();
 
-		try {
-			int y = 0;
+		painter = new SimplisticSoftBorderReverseGradientPainter();
+		BufferedImage bgImage = getPairwiseBackground(button, painter, width,
+				height, side, false);
 
-			// ControlBackgroundComposite composite = SubstanceCoreUtilities
-			// .getControlBackgroundComposite(button);
-			// Container parent = button.getParent();
-			// // boolean hasOverlay = false;
-			// if (parent instanceof JScrollBar) {
-			// Container gp = parent.getParent();
-			// if (gp instanceof JScrollPane) {
-			// JScrollPane jsp = (JScrollPane) gp;
-			// if (SubstanceCoreUtilities.hasOverlayProperty(jsp)) {
-			// // hasOverlay = true;
-			// JViewport viewport = jsp.getViewport();
-			// int dx = -viewport.getX() + viewport.getViewRect().x
-			// + button.getX() + parent.getX();
-			// //
-			// // button.getLocationOnScreen().x
-			// // + viewport.getViewRect().x
-			// // - viewport.getLocationOnScreen().x;
-			// int dy = button.getY() + viewport.getViewRect().y
-			// + parent.getY() - viewport.getY();
-			// //
-			// // button.getLocationOnScreen().y
-			// // + viewport.getViewRect().y
-			// // - viewport.getLocationOnScreen().y;
-			// resultGr.translate(-dx, -dy);
-			// JComponent view = (JComponent) viewport.getView();
-			// boolean wasDb = view.isDoubleBuffered();
-			// view.setDoubleBuffered(false);
-			// view.paint(resultGr);
-			// view.setDoubleBuffered(wasDb);
-			// resultGr.translate(dx, dy);
-			// }
-			// }
-			// }
-
-			// ComponentState state = ComponentState.getState(button.getModel(),
-			// button);
-			painter = new SimplisticSoftBorderReverseGradientPainter();
-			// ComponentState.ColorSchemeKind kind = state.getColorSchemeKind();
-			BufferedImage bgImage = getPairwiseBackground(button, // kind,
-					painter, width, height, side, false);
-			// resultGr.drawImage(bgImage, 0, 0, null);
-			graphics.drawImage(bgImage, 0, y, null);
-			return;
-		} finally {
-			// resultGr.dispose();
-			graphics.dispose();
-		}
+		graphics.drawImage(bgImage, 0, 0, null);
+		graphics.dispose();
 	}
 }
