@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -67,36 +67,6 @@ public class SubstanceColorUtilities {
 				.getUltraDarkColor(), scheme2.getUltraDarkColor(), cycleCoef);
 	}
 
-	/**
-	 * Returns the color of the top portion of border.
-	 * 
-	 * @param scheme
-	 *            The color scheme.
-	 * @param isDark
-	 *            Indicates whether the matching theme is dark.
-	 * @return The color of the top portion of border.
-	 */
-	public static Color getTopBorderColor(ColorScheme scheme, boolean isDark) {
-		return isDark ? SubstanceColorUtilities.getInterpolatedColor(scheme
-				.getExtraLightColor(), scheme.getForegroundColor(), 0.5)
-				: scheme.getDarkColor();
-	}
-
-	/**
-	 * Returns the color of the bottom portion of border.
-	 * 
-	 * @param scheme
-	 *            The color scheme.
-	 * @param isDark
-	 *            Indicates whether the matching theme is dark.
-	 * @return The color of the bottom portion of border.
-	 */
-	public static Color getBottomBorderColor(ColorScheme scheme, boolean isDark) {
-		return isDark ? SubstanceColorUtilities.getInterpolatedColor(scheme
-				.getExtraLightColor(), scheme.getForegroundColor(), 0.8)
-				: scheme.getMidColor();
-	}
-
 	/**
 	 * Returns the color of the middle portion of border in control backgrounds.
 	 * 
@@ -462,7 +432,7 @@ public class SubstanceColorUtilities {
 		switch (theme.getKind()) {
 		case DARK:
 			return theme.getHighlightTheme(null, ComponentState.ACTIVE)
-					.getColorScheme().getUltraLightColor();// .brighter()
+					.getColorScheme().getUltraLightColor().brighter();
 			// .brighter();
 		default:
 			return theme.getHighlightTheme(null, ComponentState.ACTIVE)
@@ -706,6 +676,34 @@ public class SubstanceColorUtilities {
 		return new Color(Color.HSBtoRGB(hue, hsbvals[1], hsbvals[2]));
 	}
 
+	/**
+	 * Derives a color based on the original color and a brightness source. The
+	 * resulting color has the same hue and saturation as the original color,
+	 * but its brightness is shifted towards the brightness of the brightness
+	 * source. Thus, a light red color shifted towards dark green will become
+	 * dark red.
+	 * 
+	 * @param original
+	 *            Original color.
+	 * @param brightnessSource
+	 *            Brightness source.
+	 * @return Derived color that has the same hue and saturation as the
+	 *         original color, but its brightness is shifted towards the
+	 *         brightness of the brightness source.
+	 */
+	public static Color deriveByBrightness(Color original,
+			Color brightnessSource) {
+		float[] hsbvalsOrig = new float[3];
+		Color.RGBtoHSB(original.getRed(), original.getGreen(), original
+				.getBlue(), hsbvalsOrig);
+		float[] hsbvalsBrightnessSrc = new float[3];
+		Color.RGBtoHSB(brightnessSource.getRed(), brightnessSource.getGreen(),
+				brightnessSource.getBlue(), hsbvalsBrightnessSrc);
+		return new Color(Color.HSBtoRGB(hsbvalsOrig[0], hsbvalsOrig[1],
+				(hsbvalsBrightnessSrc[2] + hsbvalsOrig[2]) / 2.0f));
+
+	}
+
 	/**
 	 * Returns the foreground color of the specified theme.
 	 * 
@@ -885,16 +883,16 @@ public class SubstanceColorUtilities {
 	 * @return The color of the focus ring for the specified component.
 	 */
 	public static Color getFocusColor(Component comp) {
-		SubstanceTheme activeTheme = SubstanceCoreUtilities.getActiveTheme(
-				comp, true);
+		SubstanceTheme activeTheme = SubstanceThemeUtilities.getTheme(comp,
+				ComponentState.ACTIVE);
 
 		if (comp instanceof AbstractButton) {
 			AbstractButton ab = (AbstractButton) comp;
 			ButtonModel model = ab.getModel();
 
 			ComponentState currState = ComponentState.getState(model, ab);
-			SubstanceTheme currTheme = SubstanceCoreUtilities.getTheme(comp,
-					currState, true, true);
+			SubstanceTheme currTheme = SubstanceThemeUtilities.getTheme(comp,
+					currState);
 			Color currColor = SubstanceCoreUtilities.isThemeDark(currTheme) ? currTheme
 					.getColorScheme().getUltraDarkColor()
 					: currTheme.getColorScheme().getDarkColor();
@@ -906,8 +904,8 @@ public class SubstanceColorUtilities {
 				ComponentState prevState = SubstanceCoreUtilities
 						.getPrevComponentState(ab);
 
-				SubstanceTheme prevTheme = SubstanceCoreUtilities.getTheme(
-						comp, prevState, true, true);
+				SubstanceTheme prevTheme = SubstanceThemeUtilities.getTheme(
+						comp, prevState);
 
 				Color prevColor = SubstanceCoreUtilities.isThemeDark(prevTheme) ? prevTheme
 						.getColorScheme().getUltraDarkColor()
@@ -923,10 +921,9 @@ public class SubstanceColorUtilities {
 			}
 		}
 
-		Color color = SubstanceCoreUtilities.isThemeDark(SubstanceCoreUtilities
-				.getActiveTheme(comp, true)) ? activeTheme.getColorScheme()
-				.getUltraDarkColor() : activeTheme.getColorScheme()
-				.getDarkColor();
+		Color color = SubstanceCoreUtilities.isThemeDark(activeTheme) ? activeTheme
+				.getColorScheme().getUltraDarkColor()
+				: activeTheme.getColorScheme().getDarkColor();
 		return color;
 	}
 
