@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -37,8 +37,11 @@ import javax.swing.*;
 import org.jvnet.lafwidget.animation.FadeKind;
 import org.jvnet.lafwidget.animation.FadeTracker;
 import org.jvnet.substance.SubstanceLookAndFeel;
+import org.jvnet.substance.border.InnerDelegateBorderPainter;
 import org.jvnet.substance.button.BaseButtonShaper;
 import org.jvnet.substance.button.SubstanceButtonShaper;
+import org.jvnet.substance.color.*;
+import org.jvnet.substance.theme.SubstanceTheme;
 
 /**
  * <b>Substance</b> constants.
@@ -75,31 +78,6 @@ public class SubstanceConstants {
 		BOTTOM;
 	}
 
-	/**
-	 * Enumerates various tree node states. This enum is <b>for internal use
-	 * only</b>.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	public static enum TreeIcon {
-		/**
-		 * Used for "Folder" button of file chooser.
-		 */
-		CLOSED,
-
-		// OPENED,
-
-		/**
-		 * Used for "Up Folder" button of file chooser.
-		 */
-		UP,
-
-		/**
-		 * Used for "New Folder" button of file chooser.
-		 */
-		NONE
-	}
-
 	/**
 	 * Enumerates focus indication kinds.
 	 * 
@@ -111,10 +89,17 @@ public class SubstanceConstants {
 		 * No focus indication.
 		 */
 		NONE {
+			/*
+			 * (non-Javadoc)
+			 * 
+			 * @see org.jvnet.substance.utils.SubstanceConstants.FocusKind#paintFocus(java.awt.Component,
+			 *      java.awt.Component, java.awt.Graphics2D, java.awt.Shape,
+			 *      java.awt.Rectangle, int)
+			 */
 			@Override
 			public void paintFocus(Component mainComp, Component focusedComp,
-					Graphics2D graphics, Rectangle focusRect,
-					Rectangle textRect, int extraPadding) {
+					Graphics2D graphics, Shape focusShape, Rectangle textRect,
+					int extraPadding) {
 			}
 		},
 
@@ -126,17 +111,21 @@ public class SubstanceConstants {
 			 * (non-Javadoc)
 			 * 
 			 * @see org.jvnet.substance.utils.SubstanceConstants.FocusKind#paintFocus(java.awt.Component,
-			 *      java.awt.Graphics2D, java.awt.Rectangle)
+			 *      java.awt.Component, java.awt.Graphics2D, java.awt.Shape,
+			 *      java.awt.Rectangle, int)
 			 */
 			@Override
 			public void paintFocus(Component mainComp, Component focusedComp,
-					Graphics2D graphics, Rectangle focusRect,
-					Rectangle textRect, int extraPadding) {
+					Graphics2D graphics, Shape focusShape, Rectangle textRect,
+					int extraPadding) {
 				if (textRect == null)
 					return;
+				if ((textRect.width == 0) || (textRect.height == 0))
+					return;
 
 				float dashPhase = 0.0f;
-				int fontSize = mainComp.getFont().getSize();
+				int fontSize = SubstanceSizeUtils
+						.getComponentFontSize(mainComp);
 				float dashLength = getDashLength(fontSize);
 				float dashGap = getDashGap(fontSize);
 				FadeTracker fadeTracker = FadeTracker.getInstance();
@@ -154,7 +143,9 @@ public class SubstanceConstants {
 				int delta = ((mainComp instanceof JComboBox) || (mainComp instanceof JSpinner)) ? 0
 						: 1;
 				GeneralPath contour = BaseButtonShaper.getBaseOutline(
-						textRect.width + 2 * delta, textRect.height, 2, null);
+						textRect.width + 2 * delta, textRect.height,
+						SubstanceSizeUtils
+								.getClassicButtonCornerRadius(fontSize), null);
 
 				graphics.translate(textRect.x - delta, textRect.y);
 				graphics.draw(contour);
@@ -175,22 +166,30 @@ public class SubstanceConstants {
 		 * Focus indication around the whole component.
 		 */
 		ALL {
+			/*
+			 * (non-Javadoc)
+			 * 
+			 * @see org.jvnet.substance.utils.SubstanceConstants.FocusKind#paintFocus(java.awt.Component,
+			 *      java.awt.Component, java.awt.Graphics2D, java.awt.Shape,
+			 *      java.awt.Rectangle, int)
+			 */
 			@Override
 			public void paintFocus(Component mainComp, Component focusedComp,
-					Graphics2D graphics, Rectangle focusRect,
-					Rectangle textRect, int extraPadding) {
-				if ((focusRect == null)
+					Graphics2D graphics, Shape focusShape, Rectangle textRect,
+					int extraPadding) {
+				if ((focusShape == null)
 						&& ((mainComp instanceof AbstractButton)
 								&& !(mainComp instanceof JCheckBox) && !(mainComp instanceof JRadioButton))) {
 					SubstanceButtonShaper shaper = SubstanceCoreUtilities
-							.getButtonShaper((AbstractButton) mainComp);
+							.getButtonShaper(mainComp);
 					if (shaper == null)
 						return;
 
 					// Shape currClip = graphics.getClip();
 					// if (!shaper.isProportionate()) {
 					float dashPhase = 0.0f;
-					int fontSize = mainComp.getFont().getSize();
+					int fontSize = SubstanceSizeUtils
+							.getComponentFontSize(mainComp);
 					float dashLength = getDashLength(fontSize);
 					float dashGap = getDashGap(fontSize);
 					FadeTracker fadeTracker = FadeTracker.getInstance();
@@ -212,13 +211,19 @@ public class SubstanceConstants {
 				} else {
 					// graphics.translate(textRect.x - 1, textRect.y - 1);
 					graphics.translate(1, 1);
-					Shape contour = (focusRect != null) ? focusRect
-							: BaseButtonShaper.getBaseOutline(mainComp
-									.getWidth() - 2, mainComp.getHeight() - 2,
-									3, null);
+					Shape contour = (focusShape != null) ? focusShape
+							: BaseButtonShaper
+									.getBaseOutline(
+											mainComp.getWidth() - 2,
+											mainComp.getHeight() - 2,
+											SubstanceSizeUtils
+													.getClassicButtonCornerRadius(SubstanceSizeUtils
+															.getComponentFontSize(mainComp)),
+											null);
 
 					float dashPhase = 0.0f;
-					int fontSize = mainComp.getFont().getSize();
+					int fontSize = SubstanceSizeUtils
+							.getComponentFontSize(mainComp);
 					float dashLength = getDashLength(fontSize);
 					float dashGap = getDashGap(fontSize);
 					FadeTracker fadeTracker = FadeTracker.getInstance();
@@ -252,22 +257,30 @@ public class SubstanceConstants {
 		 * the component.
 		 */
 		ALL_INNER {
+			/*
+			 * (non-Javadoc)
+			 * 
+			 * @see org.jvnet.substance.utils.SubstanceConstants.FocusKind#paintFocus(java.awt.Component,
+			 *      java.awt.Component, java.awt.Graphics2D, java.awt.Shape,
+			 *      java.awt.Rectangle, int)
+			 */
 			@Override
 			public void paintFocus(Component mainComp, Component focusedComp,
-					Graphics2D graphics, Rectangle focusRect,
-					Rectangle textRect, int extraPadding) {
+					Graphics2D graphics, Shape focusShape, Rectangle textRect,
+					int extraPadding) {
 
-				if ((focusRect == null)
+				if ((focusShape == null)
 						&& ((mainComp instanceof AbstractButton)
 								&& !(mainComp instanceof JCheckBox) && !(mainComp instanceof JRadioButton))) {
 					SubstanceButtonShaper shaper = SubstanceCoreUtilities
-							.getButtonShaper((AbstractButton) mainComp);
+							.getButtonShaper(mainComp);
 					if (shaper == null)
 						return;
 
 					if (shaper.isProportionate()) {
 						float dashPhase = 0.0f;
-						int fontSize = mainComp.getFont().getSize();
+						int fontSize = SubstanceSizeUtils
+								.getComponentFontSize(mainComp);
 						float dashLength = getDashLength(fontSize);
 						float dashGap = getDashGap(fontSize);
 						FadeTracker fadeTracker = FadeTracker.getInstance();
@@ -278,33 +291,33 @@ public class SubstanceConstants {
 											focusedComp,
 											FadeKind.FOCUS_LOOP_ANIMATION) / 10.0f);
 						}
-						graphics.setStroke(new BasicStroke(SubstanceSizeUtils
-								.getFocusStrokeWidth(fontSize),
+						float focusStrokeWidth = SubstanceSizeUtils
+								.getFocusStrokeWidth(fontSize);
+						graphics.setStroke(new BasicStroke(focusStrokeWidth,
 								BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND,
 								0.0f, new float[] { dashLength, dashGap },
 								dashPhase));
-						Insets insets = new Insets(extraPadding, extraPadding,
-								extraPadding, extraPadding);
+						int insetsPix = extraPadding;
+						Insets insets = new Insets(insetsPix, insetsPix,
+								insetsPix, insetsPix);
 
 						GeneralPath contour = shaper.getButtonOutline(
 								(AbstractButton) mainComp, insets);
 						graphics.draw(contour);
 					}
 				} else {
-					// graphics.translate(textRect.x - 1, textRect.y - 1);
-					// int delta = ((mainComp instanceof JComboBox)
-					// || (mainComp instanceof JSpinner) || (mainComp instanceof
-					// JTextComponent)) ? 2
-					// : 1;
-					graphics.translate(extraPadding, extraPadding);
-					Shape contour = (focusRect != null) ? focusRect
+					graphics.translate(extraPadding / 2, extraPadding / 2);
+					int fontSize = SubstanceSizeUtils
+							.getComponentFontSize(mainComp);
+					Shape contour = (focusShape != null) ? focusShape
 							: BaseButtonShaper.getBaseOutline(mainComp
 									.getWidth()
-									- 2 * extraPadding, mainComp.getHeight()
-									- 2 * extraPadding, 2, null);
+									- extraPadding, mainComp.getHeight()
+									- extraPadding, SubstanceSizeUtils
+									.getClassicButtonCornerRadius(fontSize),
+									null);
 
 					float dashPhase = 0.0f;
-					int fontSize = mainComp.getFont().getSize();
 					float dashLength = getDashLength(fontSize);
 					float dashGap = getDashGap(fontSize);
 					FadeTracker fadeTracker = FadeTracker.getInstance();
@@ -338,54 +351,22 @@ public class SubstanceConstants {
 		 * the component.
 		 */
 		ALL_STRONG_INNER {
+			/*
+			 * (non-Javadoc)
+			 * 
+			 * @see org.jvnet.substance.utils.SubstanceConstants.FocusKind#paintFocus(java.awt.Component,
+			 *      java.awt.Component, java.awt.Graphics2D, java.awt.Shape,
+			 *      java.awt.Rectangle, int)
+			 */
 			@Override
 			public void paintFocus(Component mainComp, Component focusedComp,
-					Graphics2D graphics, Rectangle focusRect,
-					Rectangle textRect, int extraPadding) {
-				//
-				// if ((mainComp instanceof AbstractButton)
-				// && !(mainComp instanceof JCheckBox)
-				// && !(mainComp instanceof JRadioButton)) {
-				// SubstanceButtonShaper shaper = SubstanceCoreUtilities
-				// .getButtonShaper((AbstractButton) mainComp);
-				// if (shaper == null)
-				// return;
-				//
-				// if (shaper.isProportionate()) {
-				// Insets insets = new Insets(extraPadding, extraPadding,
-				// extraPadding, extraPadding);
-				// graphics.setStroke(new BasicStroke(SubstanceSizeUtils
-				// .getFocusStrokeWidth() / 2.5f,
-				// BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
-				// GeneralPath contour = shaper.getButtonOutline(
-				// (AbstractButton) mainComp, insets);
-				// graphics.draw(contour);
-				// insets = new Insets(1 + extraPadding, 1 + extraPadding,
-				// 1 + extraPadding, 1 + extraPadding);
-				// graphics.setStroke(new BasicStroke(SubstanceSizeUtils
-				// .getFocusStrokeWidth() / 1.6f,
-				// BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
-				// contour = shaper.getButtonOutline(
-				// (AbstractButton) mainComp, insets);
-				// graphics.draw(contour);
-				// }
-				// } else {
-				// graphics.translate(extraPadding, extraPadding);
-				// Shape contour = BaseButtonShaper.getBaseOutline(mainComp
-				// .getWidth()
-				// - 2 * extraPadding, mainComp.getHeight() - 2
-				// * extraPadding, 2, null);
-				// graphics.setStroke(new BasicStroke(SubstanceSizeUtils
-				// .getFocusStrokeWidth() / 1.3f,
-				// BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
-				// graphics.draw(contour);
-				// }
-
-				if ((focusRect == null)
+					Graphics2D graphics, Shape focusShape, Rectangle textRect,
+					int extraPadding) {
+				if ((focusShape == null)
 						&& ((mainComp instanceof AbstractButton)
 								&& !(mainComp instanceof JCheckBox) && !(mainComp instanceof JRadioButton))) {
 					SubstanceButtonShaper shaper = SubstanceCoreUtilities
-							.getButtonShaper((AbstractButton) mainComp);
+							.getButtonShaper(mainComp);
 					if (shaper == null)
 						return;
 
@@ -398,17 +379,16 @@ public class SubstanceConstants {
 						graphics.draw(contour);
 					}
 				} else {
-					// graphics.translate(textRect.x - 1, textRect.y - 1);
-					// int delta = ((mainComp instanceof JComboBox)
-					// || (mainComp instanceof JSpinner) || (mainComp instanceof
-					// JTextComponent)) ? 2
-					// : 1;
-					graphics.translate(extraPadding, extraPadding);
-					Shape contour = (focusRect != null) ? focusRect
-							: BaseButtonShaper.getBaseOutline(mainComp
-									.getWidth()
-									- 2 * extraPadding, mainComp.getHeight()
-									- 2 * extraPadding, 2, null);
+					graphics.translate(extraPadding / 2, extraPadding / 2);
+					Shape contour = (focusShape != null) ? focusShape
+							: BaseButtonShaper
+									.getBaseOutline(
+											mainComp.getWidth() - extraPadding,
+											mainComp.getHeight() - extraPadding,
+											SubstanceSizeUtils
+													.getClassicButtonCornerRadius(SubstanceSizeUtils
+															.getComponentFontSize(mainComp)),
+											null);
 
 					graphics.draw(contour);
 				}
@@ -419,15 +399,23 @@ public class SubstanceConstants {
 		 * Focus indication under the component text.
 		 */
 		UNDERLINE {
+			/*
+			 * (non-Javadoc)
+			 * 
+			 * @see org.jvnet.substance.utils.SubstanceConstants.FocusKind#paintFocus(java.awt.Component,
+			 *      java.awt.Component, java.awt.Graphics2D, java.awt.Shape,
+			 *      java.awt.Rectangle, int)
+			 */
 			@Override
 			public void paintFocus(Component mainComp, Component focusedComp,
-					Graphics2D graphics, Rectangle focusRect,
-					Rectangle textRect, int extraPadding) {
+					Graphics2D graphics, Shape focusShape, Rectangle textRect,
+					int extraPadding) {
 				if (textRect == null)
 					return;
 
 				float dashPhase = 0.0f;
-				int fontSize = mainComp.getFont().getSize();
+				int fontSize = SubstanceSizeUtils
+						.getComponentFontSize(mainComp);
 				float dashLength = getDashLength(fontSize);
 				float dashGap = getDashGap(fontSize);
 				FadeTracker fadeTracker = FadeTracker.getInstance();
@@ -463,10 +451,17 @@ public class SubstanceConstants {
 		 * Strong focus indication under the component text.
 		 */
 		STRONG_UNDERLINE {
+			/*
+			 * (non-Javadoc)
+			 * 
+			 * @see org.jvnet.substance.utils.SubstanceConstants.FocusKind#paintFocus(java.awt.Component,
+			 *      java.awt.Component, java.awt.Graphics2D, java.awt.Shape,
+			 *      java.awt.Rectangle, int)
+			 */
 			@Override
 			public void paintFocus(Component mainComp, Component focusedComp,
-					Graphics2D graphics, Rectangle focusRect,
-					Rectangle textRect, int extraPadding) {
+					Graphics2D graphics, Shape focusShape, Rectangle textRect,
+					int extraPadding) {
 				if (textRect == null)
 					return;
 
@@ -488,9 +483,9 @@ public class SubstanceConstants {
 		 *            editor.
 		 * @param graphics
 		 *            Graphics context.
-		 * @param focusRect
-		 *            Focus rectangle. May be <code>null</code> - in this
-		 *            case, the bounds of <code>mainComp</code> will be used.
+		 * @param focusShape
+		 *            Focus shape. May be <code>null</code> - in this case,
+		 *            the bounds of <code>mainComp</code> will be used.
 		 * @param textRect
 		 *            Text rectangle (if relevant).
 		 * @param extraPadding
@@ -498,8 +493,8 @@ public class SubstanceConstants {
 		 *            ring painting.
 		 */
 		public abstract void paintFocus(Component mainComp,
-				Component focusedComp, Graphics2D graphics,
-				Rectangle focusRect, Rectangle textRect, int extraPadding);
+				Component focusedComp, Graphics2D graphics, Shape focusShape,
+				Rectangle textRect, int extraPadding);
 
 		/**
 		 * Paints the focus ring on the specified component.
@@ -519,6 +514,8 @@ public class SubstanceConstants {
 		/**
 		 * Returns DPI-aware dash length for dash-based focus painting.
 		 * 
+		 * @param fontSize
+		 *            The font size of the component for focus painting.
 		 * @return DPI-aware dash length for dash-based focus painting.
 		 */
 		protected static float getDashLength(int fontSize) {
@@ -528,6 +525,8 @@ public class SubstanceConstants {
 		/**
 		 * Returns DPI-aware dash gap for dash-based focus painting.
 		 * 
+		 * @param fontSize
+		 *            The font size of the component for focus painting.
 		 * @return DPI-aware dash gap for dash-based focus painting.
 		 */
 		protected static float getDashGap(int fontSize) {
@@ -547,15 +546,6 @@ public class SubstanceConstants {
 		}
 	}
 
-	// /**
-	// * Button kind in title pane. This enum is <b>for internal use only</b>.
-	// *
-	// * @author Kirill Grouchnikov
-	// */
-	// public enum ButtonTitleKind {
-	// NONE, CLOSE, CLOSE_DI
-	// }
-
 	/**
 	 * Enumerates of image-based watermarks kinds.
 	 * 
@@ -738,8 +728,45 @@ public class SubstanceConstants {
 		HARD
 	}
 
+	/**
+	 * Color shift kind for shifting themes in inner border painters. See
+	 * {@link InnerDelegateBorderPainter} for usage details.
+	 * 
+	 * @author Kirill Grouchnikov
+	 */
 	public enum ColorShiftKind {
-		SHADE, TINT, TONE, THEME_LIGHT
+		/**
+		 * Instructs the inner border painter to shade the border theme for the
+		 * inner part of the border.
+		 * 
+		 * @see ShadeColorScheme
+		 * @see SubstanceTheme#shade(double)
+		 */
+		SHADE,
+
+		/**
+		 * Instructs the inner border painter to tint the border theme for the
+		 * inner part of the border.
+		 * 
+		 * @see TintColorScheme
+		 * @see SubstanceTheme#tint(double)
+		 */
+		TINT,
+
+		/**
+		 * Instructs the inner border painter to tone the border theme for the
+		 * inner part of the border.
+		 * 
+		 * @see ToneColorScheme
+		 * @see SubstanceTheme#tone(double)
+		 */
+		TONE,
+
+		/**
+		 * Instructs the inner border painter to shift the border theme for the
+		 * inner part of the border towards its light colors.
+		 */
+		THEME_LIGHT
 	}
 
 	/**
