@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -29,16 +29,14 @@
  */
 package org.jvnet.substance.utils;
 
-import java.awt.Component;
-import java.awt.Insets;
+import java.awt.*;
 
 import javax.swing.border.Border;
 import javax.swing.plaf.BorderUIResource;
 
 import org.jvnet.substance.SubstanceLookAndFeel;
 import org.jvnet.substance.button.ClassicButtonShaper;
-import org.jvnet.substance.fonts.FontPolicy;
-import org.jvnet.substance.fonts.FontSet;
+import org.jvnet.substance.fonts.*;
 
 /**
  * This class is responsible for computing DPI-aware insets, stroke widths,
@@ -52,6 +50,8 @@ public class SubstanceSizeUtils {
 	 */
 	private static int controlFontSize = -1;
 
+	private static double pointsToPixelsRatio = 1.0;
+
 	/**
 	 * Gets the current control font size.
 	 * 
@@ -184,7 +184,9 @@ public class SubstanceSizeUtils {
 	 * @return Height of arrow icons under the specified font size.
 	 */
 	public static int getArrowIconHeight(int fontSize) {
-		return 6 + (fontSize - 11) / 3;
+		if (fontSize < 12)
+			return 3 + fontSize / 2;
+		return 3 + fontSize * 3 / 5;
 	}
 
 	/**
@@ -195,10 +197,10 @@ public class SubstanceSizeUtils {
 	 * @return Width of arrow icons under the specified font size.
 	 */
 	public static int getArrowIconWidth(int fontSize) {
-		int result = fontSize - (int) (2 * getBorderStrokeWidth(fontSize));
+		int result = fontSize - (int) (getBorderStrokeWidth(fontSize));
 		if (result % 2 == 0)
 			result--;
-		return result;
+		return 2 + result;
 	}
 
 	/**
@@ -336,6 +338,21 @@ public class SubstanceSizeUtils {
 		return getAdjustedSize(fontSize, 2, 4, 1, false);
 	}
 
+	/**
+	 * Returns the diameter of a big drag bump dot under the specified font
+	 * size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Diameter of a big drag bump dot under the specified font size.
+	 */
+	public static int getBigDragBumpDiameter(int fontSize) {
+		int result = getAdjustedSize(fontSize, 3, 3, 1, false);
+		if (result % 2 == 1)
+			result++;
+		return result;
+	}
+
 	/**
 	 * Returns the extra padding amount under the specified font size.
 	 * 
@@ -346,7 +363,14 @@ public class SubstanceSizeUtils {
 	public static int getExtraPadding(int fontSize) {
 		if (fontSize < 14)
 			return 0;
-		return (fontSize - 11) / 3;
+		return (int) SubstanceSizeUtils.getAdjustedSize(fontSize, 0, 3, 1.2f);
+	}
+
+	public static int getFocusRingPadding(int fontSize) {
+		if (fontSize < 14)
+			return 2;
+		return 3 + (int) SubstanceSizeUtils.getAdjustedSize(fontSize, 0, 3,
+				0.8f);
 	}
 
 	/**
@@ -399,6 +423,30 @@ public class SubstanceSizeUtils {
 		return result;
 	}
 
+	/**
+	 * Returns the margin for menu items under the specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Margin for menu items under the specified font size.
+	 */
+	public static int getMenuItemMargin(int fontSize) {
+		return getAdjustedSize(fontSize, 2, 4, 1, false);
+	}
+
+	/**
+	 * Returns the gap between text and icon in buttons and menu items under the
+	 * specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Gap between text and icon in button menu items under the
+	 *         specified font size.
+	 */
+	public static int getTextIconGap(int fontSize) {
+		return getAdjustedSize(fontSize, 4, 3, 1, false);
+	}
+
 	/**
 	 * Returns the minimum button height under the specified font size.
 	 * 
@@ -525,6 +573,36 @@ public class SubstanceSizeUtils {
 		return Math.max(4, 5 + (fontSize - 11) / 2);
 	}
 
+	/**
+	 * Returns the height of spinner arrow icons under the specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Height of spinner arrow icons under the specified font size.
+	 */
+	public static int getSpinnerArrowIconHeight(int fontSize) {
+		int result = SubstanceSizeUtils.getArrowIconHeight(fontSize)
+				+ (int) SubstanceSizeUtils.getAdjustedSize(fontSize, -1, 1,
+						-0.25f);
+		return result;
+	}
+
+	/**
+	 * Returns the width of spinner arrow icons under the specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Width of spinner arrow icons under the specified font size.
+	 */
+	public static int getSpinnerArrowIconWidth(int fontSize) {
+		int result = SubstanceSizeUtils.getArrowIconWidth(fontSize)
+				+ (int) SubstanceSizeUtils.getAdjustedSize(fontSize, -2, 1,
+						-0.15f);
+		if (result % 2 == 0)
+			result--;
+		return result;
+	}
+
 	/**
 	 * Returns the spinner border insets under the specified font size.
 	 * 
@@ -540,6 +618,20 @@ public class SubstanceSizeUtils {
 				comboInsets.bottom + 1, comboInsets.right);
 	}
 
+	/**
+	 * Returns the spinner arrow button insets under the specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Spinner arrow button insets under the specified font size.
+	 */
+	public static Insets getSpinnerArrowButtonInsets(int fontSize) {
+		int borderStrokeWidth = (int) Math
+				.floor(getBorderStrokeWidth(fontSize));
+		return new Insets(borderStrokeWidth, borderStrokeWidth,
+				borderStrokeWidth, borderStrokeWidth);
+	}
+
 	/**
 	 * Returns the spinner button width under the specified font size.
 	 * 
@@ -564,6 +656,53 @@ public class SubstanceSizeUtils {
 				textInsets.bottom - 1, textInsets.right);
 	}
 
+	/**
+	 * Returns the height of split pane divider arrow icons under the specified
+	 * font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Height of split pane divider arrow icons under the specified font
+	 *         size.
+	 */
+	public static int getSplitPaneArrowIconHeight(int fontSize) {
+		int result = SubstanceSizeUtils.getArrowIconHeight(fontSize)
+				+ (int) SubstanceSizeUtils.getAdjustedSize(fontSize, -1, 1,
+						-0.3f);
+		return result;
+	}
+
+	/**
+	 * Returns the width of split pane divider arrow icons under the specified
+	 * font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Width of split pane divider arrow icons under the specified font
+	 *         size.
+	 */
+	public static int getSplitPaneArrowIconWidth(int fontSize) {
+		int result = SubstanceSizeUtils.getArrowIconWidth(fontSize)
+				+ (int) SubstanceSizeUtils.getAdjustedSize(fontSize, -2, 1,
+						-0.25f);
+		if (result % 2 == 0)
+			result--;
+		return result;
+	}
+
+	/**
+	 * Returns the offset of the first split pane divider button under the
+	 * specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Offset of the first split pane divider button under the specified
+	 *         font size.
+	 */
+	public static int getSplitPaneButtonOffset(int fontSize) {
+		return getAdjustedSize(fontSize, 2, 3, 1, false);
+	}
+
 	/**
 	 * Returns the tabbed pane content insets under the specified size. The
 	 * {@link SubstanceConstants.TabContentPaneBorderKind#SINGLE_FULL} is
@@ -665,6 +804,20 @@ public class SubstanceSizeUtils {
 		return new Insets(tInset, lbrInset, lbrInset, lbrInset);
 	}
 
+	/**
+	 * Returns the tooltip border insets under the specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Tooltip border insets under the specified font size.
+	 */
+	public static Insets getToolTipBorderInsets(int fontSize) {
+		// The base insets are 1,1,1,1. We add one pixel for
+		// each 3 extra points in base control size.
+		int inset = getAdjustedSize(fontSize, 1, 3, 1, false);
+		return new Insets(inset, inset, inset, inset);
+	}
+
 	/**
 	 * Returns the icon size of tree expand / collapse icons under the specified
 	 * size.
@@ -679,4 +832,19 @@ public class SubstanceSizeUtils {
 		int extraPadding2 = 2 * extraPadding;
 		return 10 + extraPadding2;
 	}
+
+	public static double getPointsToPixelsRatio() {
+		return pointsToPixelsRatio;
+	}
+
+	public static void resetPointsToPixelsRatio(FontPolicy fontPolicy) {
+		if (fontPolicy instanceof DefaultGnomeFontPolicy) {
+			pointsToPixelsRatio = DefaultGnomeFontPolicy
+					.getPointsToPixelsRatio();
+		} else {
+			pointsToPixelsRatio = Toolkit.getDefaultToolkit()
+					.getScreenResolution() / 72.0;
+		}
+	}
+
 }
