@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -45,7 +45,7 @@ import javax.swing.plaf.UIResource;
 import org.jvnet.lafwidget.LafWidget;
 import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.animation.effects.GhostPaintingUtils;
-import org.jvnet.lafwidget.layout.TransitionLayout;
+import org.jvnet.lafwidget.utils.RenderingUtils;
 import org.jvnet.lafwidget.utils.TrackableThread;
 import org.jvnet.lafwidget.utils.LafConstants.AnimationKind;
 import org.jvnet.substance.*;
@@ -56,10 +56,12 @@ import org.jvnet.substance.button.ButtonShaperInfo;
 import org.jvnet.substance.color.ColorScheme;
 import org.jvnet.substance.painter.GradientPainterChangeListener;
 import org.jvnet.substance.painter.GradientPainterInfo;
+import org.jvnet.substance.painter.decoration.DecorationAreaType;
+import org.jvnet.substance.painter.decoration.SubstanceDecorationUtilities;
+import org.jvnet.substance.painter.text.SubstanceTextPainter;
 import org.jvnet.substance.skin.SkinInfo;
 import org.jvnet.substance.theme.*;
 import org.jvnet.substance.theme.SubstanceTheme.ThemeKind;
-import org.jvnet.substance.title.*;
 import org.jvnet.substance.utils.SubstanceConstants.FocusKind;
 import org.jvnet.substance.utils.icon.SubstanceIconFactory;
 import org.jvnet.substance.utils.icon.TransitionAwareIcon;
@@ -206,8 +208,6 @@ public class SubstanceTitlePane extends JComponent {
 
 	protected TraitMenuHandler titlePainterMenuHandler;
 
-	protected TitlePainterChangeListener titlePainterChangeListener;
-
 	protected TraitMenuHandler borderPainterMenuHandler;
 
 	protected BorderPainterChangeListener borderPainterChangeListener;
@@ -238,11 +238,14 @@ public class SubstanceTitlePane extends JComponent {
 		 */
 		private LinkedList<Double> graphValues;
 
+		private SubstanceTitlePane titlePane;
+
 		/**
 		 * Creates new heap status panel.
 		 */
-		public HeapStatusPanel() {
+		public HeapStatusPanel(SubstanceTitlePane titlePane) {
 			this.graphValues = new LinkedList<Double>();
+			this.titlePane = titlePane;
 			HeapStatusThread.getInstance();
 		}
 
@@ -273,12 +276,13 @@ public class SubstanceTitlePane extends JComponent {
 		public synchronized void paint(Graphics g) {
 			Graphics2D graphics = (Graphics2D) g.create();
 
-			ColorScheme scheme = SubstanceCoreUtilities.getActiveScheme(this
-					.getRootPane());
-
-			// special case for mixed dark themes
-			// if (scheme instanceof MixDarkBiColorScheme)
-			// scheme = ((MixDarkBiColorScheme) scheme).getOrigDarkScheme();
+			Window window = this.titlePane.getWindow();
+			boolean isSelected = (window == null) ? true : window.isActive();
+			final SubstanceTheme theme = isSelected ? SubstanceLookAndFeel
+					.getTheme().getActiveTitlePaneTheme()
+					: SubstanceLookAndFeel.getTheme()
+							.getDefaultTitlePaneTheme();
+			ColorScheme scheme = theme.getColorScheme();
 
 			graphics.setColor(scheme.getDarkColor());
 			int w = this.getWidth();
@@ -314,6 +318,7 @@ public class SubstanceTitlePane extends JComponent {
 			int strH = fm.getAscent() + fm.getDescent();
 
 			graphics.setColor(scheme.getForegroundColor());
+			RenderingUtils.installDesktopHints(graphics);
 			if (strW < (w - 5)) {
 				graphics.drawString(longFormat.toString(), (w - strW) / 2,
 						(h + strH) / 2 - 2);
@@ -327,6 +332,18 @@ public class SubstanceTitlePane extends JComponent {
 
 			graphics.dispose();
 		}
+
+		public int getPreferredWidth() {
+			BufferedImage dummy = new BufferedImage(1, 1,
+					BufferedImage.TYPE_INT_ARGB);
+			Graphics2D g2d = dummy.createGraphics();
+			RenderingUtils.installDesktopHints(g2d);
+			g2d.setFont(UIManager.getFont("Panel.font"));
+			FontMetrics fm = g2d.getFontMetrics();
+			int result = fm.stringWidth("100.9MB / 200.9MB");
+			g2d.dispose();
+			return result;
+		}
 	}
 
 	/**
@@ -500,6 +517,9 @@ public class SubstanceTitlePane extends JComponent {
 		this.setLayout(this.createLayout());
 
 		this.setToolTipText(this.getTitle());
+
+		SubstanceDecorationUtilities.setDecorationType(this,
+				DecorationAreaType.PRIMARY_TITLE_PANE);
 	}
 
 	/**
@@ -543,9 +563,6 @@ public class SubstanceTitlePane extends JComponent {
 		SubstanceLookAndFeel
 				.unregisterGradientPainterChangeListener(this.gradientPainterChangeListener);
 		this.gradientPainterChangeListener = null;
-		SubstanceLookAndFeel
-				.unregisterTitlePainterChangeListener(this.titlePainterChangeListener);
-		this.titlePainterChangeListener = null;
 		SubstanceLookAndFeel
 				.unregisterBorderPainterChangeListener(this.borderPainterChangeListener);
 		this.borderPainterChangeListener = null;
@@ -886,7 +903,7 @@ public class SubstanceTitlePane extends JComponent {
 			this.add(this.closeButton);
 
 			if (SubstanceTitlePane.canHaveHeapStatusPanel) {
-				this.heapStatusPanel = new HeapStatusPanel();
+				this.heapStatusPanel = new HeapStatusPanel(this);
 				this.add(this.heapStatusPanel);
 				boolean isHeapStatusPanelShowing = Boolean.TRUE
 						.equals(this.rootPane
@@ -1362,54 +1379,6 @@ public class SubstanceTitlePane extends JComponent {
 			SubstanceLookAndFeel
 					.registerGradientPainterChangeListener(this.gradientPainterChangeListener);
 
-			JMenu titlePainterMenu = new JMenu(SubstanceCoreUtilities
-					.getResourceBundle(rootPane).getString(
-							"SystemMenu.titlePainters"));
-			ButtonGroup bgTitlePainter = new ButtonGroup();
-			this.titlePainterMenuHandler = new TraitMenuHandler();
-			Map<String, TitlePainterInfo> allTitlePainters = SubstanceLookAndFeel
-					.getAllTitlePainters();
-			for (Map.Entry<String, TitlePainterInfo> titlePainterEntry : allTitlePainters
-					.entrySet()) {
-				final String titlePainterClassName = titlePainterEntry
-						.getValue().getClassName();
-				JRadioButtonMenuItem jmiTitlePainter = new JRadioButtonMenuItem(
-						titlePainterEntry.getKey());
-				this.titlePainterMenuHandler.addTraitButton(titlePainterEntry
-						.getValue().getDisplayName(), jmiTitlePainter);
-				jmiTitlePainter.addActionListener(new ActionListener() {
-					public void actionPerformed(ActionEvent e) {
-						SwingUtilities.invokeLater(new Runnable() {
-							public void run() {
-
-								SubstanceLookAndFeel
-										.setCurrentTitlePainter(titlePainterClassName);
-								// update all existing root panes
-								for (Frame frame : Frame.getFrames()) {
-									SwingUtilities.updateComponentTreeUI(frame);
-								}
-							}
-						});
-					}
-				});
-				if (titlePainterEntry.getKey().equals(
-						SubstanceLookAndFeel.getCurrentTitlePainterName())) {
-					jmiTitlePainter.setSelected(true);
-				}
-				bgTitlePainter.add(jmiTitlePainter);
-				titlePainterMenu.add(jmiTitlePainter);
-			}
-			menu.add(titlePainterMenu);
-			this.titlePainterChangeListener = new TitlePainterChangeListener() {
-				public void titlePainterChanged() {
-					titlePainterMenuHandler
-							.selectTraitButton(SubstanceLookAndFeel
-									.getCurrentTitlePainter());
-				}
-			};
-			SubstanceLookAndFeel
-					.registerTitlePainterChangeListener(this.titlePainterChangeListener);
-
 			if (SubstanceTitlePane.canHaveHeapStatusPanel
 					&& (!(this.rootPane
 							.getClientProperty(SubstanceTitlePane.HEAP_STATUS_PANEL_PERMANENT) instanceof Boolean))) {
@@ -1768,29 +1737,18 @@ public class SubstanceTitlePane extends JComponent {
 		if (this.getFrame() != null) {
 			this.setState(this.getFrame().getExtendedState());
 		}
-		JRootPane rootPane = this.getRootPane();
+		final JRootPane rootPane = this.getRootPane();
 		Window window = this.getWindow();
 		boolean leftToRight = (window == null) ? rootPane
 				.getComponentOrientation().isLeftToRight() : window
 				.getComponentOrientation().isLeftToRight();
 		boolean isSelected = (window == null) ? true : window.isActive();
-		int width = this.getWidth();
-		int height = this.getHeight();
+		final int width = this.getWidth();
+		final int height = this.getHeight();
 
-		// ColorScheme colorScheme = isSelected ? SubstanceCoreUtilities
-		// .getActiveScheme(rootPane) : SubstanceCoreUtilities
-		// .getDefaultScheme(rootPane);
-		//
-		// // special case for complex themes
-		// if (isSelected
-		// && (SubstanceLookAndFeel.getTheme() instanceof
-		// SubstanceComplexTheme)) {
-		// colorScheme = ((SubstanceComplexTheme) SubstanceLookAndFeel
-		// .getTheme()).getActiveTitlePaneTheme().getColorScheme();
-		// }
-		SubstanceTheme theme = isSelected ? SubstanceLookAndFeel.getTheme()
-				.getActiveTitlePaneTheme() : SubstanceLookAndFeel.getTheme()
-				.getDefaultTitlePaneTheme();
+		final SubstanceTheme theme = isSelected ? SubstanceLookAndFeel
+				.getTheme().getActiveTitlePaneTheme() : SubstanceLookAndFeel
+				.getTheme().getDefaultTitlePaneTheme();
 
 		int xOffset = 0;
 		String theTitle = this.getTitle();
@@ -1900,35 +1858,27 @@ public class SubstanceTitlePane extends JComponent {
 			}
 		}
 
-		SubstanceTitlePainter titlePainter = SubstanceCoreUtilities
-				.getTitlePainter(rootPane);
-
 		Graphics2D graphics = (Graphics2D) g.create();
 		Font font = SubstanceLookAndFeel.getFontPolicy().getFontSet(
 				"Substance", null).getWindowTitleFont();
 		graphics.setFont(font);
-		titlePainter.paintTitleBackground(graphics, rootPane, width + 1,
-				height, leftEnd, rightEnd, theme, 0.0f);
-		//
-		// graphics, 0, 0, width,
-		// height + 1, colorScheme, false, false);
 
-		if (SubstanceCoreUtilities.toDrawWatermark(this)) {
-			// paint the watermark over the title pane
-			SubstanceLookAndFeel.getCurrentWatermark().drawWatermarkImage(
-					graphics, this, 0, 0, width, height);
+		SubstanceTextPainter textPainter = SubstanceLookAndFeel
+				.getCurrentTextPainter();
+		textPainter.init(this, null, true);
 
-			// paint the background second time with 50% translucency, making
-			// the watermark 'bleed' through.
-			Composite oldComp = graphics.getComposite();
-			graphics.setComposite(TransitionLayout.getAlphaComposite(
-					this.rootPane, 0.5f));
-			titlePainter.paintTitleBackground(graphics, rootPane, width + 1,
-					height, leftEnd, rightEnd, theme, 0.0f);
-			// SubstanceImageCreator.paintRectangularBackground(graphics, 0, 0,
-			// width,
-			// height + 1, colorScheme, false, false);
-			graphics.setComposite(oldComp);
+		if (textPainter.needsBackgroundImage()) {
+			textPainter
+					.attachCallback(new SubstanceTextPainter.BackgroundPaintingCallback() {
+						public void paintBackground(Graphics g) {
+							SubstanceDecorationUtilities
+									.paintDecorationBackground(g,
+											SubstanceTitlePane.this, false);
+						}
+					});
+		} else {
+			SubstanceDecorationUtilities.paintDecorationBackground(graphics,
+					SubstanceTitlePane.this, false);
 		}
 
 		// draw the title (if needed)
@@ -1939,40 +1889,8 @@ public class SubstanceTitlePane extends JComponent {
 			SubstanceCoreUtilities.paintTextWithDropShadow(this, graphics,
 					theme.getForegroundColor(), theTitle, width, height,
 					xOffset, yOffset);
-
-			// Object oldAAValue = graphics
-			// .getRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING);
-			// graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
-			// RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
-			//
-			// Color foregroundColor =
-			// theme.getForegroundColor();//getTitlePaneForegroundColor(theme);
-			//
-			// // blur the text shadow
-			// BufferedImage blurred =
-			// SubstanceCoreUtilities.getBlankImage(width,
-			// height);
-			// Graphics2D gBlurred = (Graphics2D) blurred.getGraphics();
-			// gBlurred.setFont(graphics.getFont());
-			// gBlurred.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
-			// RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
-			// gBlurred.setColor(SubstanceColorUtilities
-			// .getNegativeColor(foregroundColor));
-			// ConvolveOp convolve = new ConvolveOp(new Kernel(3, 3, new float[]
-			// {
-			// .0f, .02f, .05f, .02f, .0f, .05f, .05f, .05f, .05f }),
-			// ConvolveOp.EDGE_NO_OP, null);
-			// gBlurred.drawString(theTitle, xOffset + 1, yOffset + 1);
-			// blurred = convolve.filter(blurred, null);
-			//
-			// graphics.drawImage(blurred, 0, 0, null);
-			//
-			// graphics.setColor(foregroundColor);
-			// RenderingUtils.installDesktopHints(graphics);
-			// graphics.drawString(theTitle, xOffset, yOffset);
-			// graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
-			// oldAAValue);
 		}
+		textPainter.renderSurface(graphics);
 
 		GhostPaintingUtils.paintGhostImages(this, graphics);
 
@@ -2083,6 +2001,11 @@ public class SubstanceTitlePane extends JComponent {
 		public void actionPerformed(ActionEvent e) {
 			Frame frame = SubstanceTitlePane.this.getFrame();
 			if (frame != null) {
+				if (frame instanceof JFrame) {
+					SubstanceRootPaneUI rpUI = (SubstanceRootPaneUI) ((JFrame) frame)
+							.getRootPane().getUI();
+					rpUI.setMaximized();
+				}
 				frame.setExtendedState(SubstanceTitlePane.this.state
 						| Frame.MAXIMIZED_BOTH);
 			}
@@ -2287,11 +2210,11 @@ public class SubstanceTitlePane extends JComponent {
 						&& SubstanceTitlePane.this.heapStatusPanel.isVisible()) {
 					spacing = 5;
 					x += leftToRight ? (-spacing - SubstanceTitlePane.this.heapStatusPanel
-							.getPreferredSize().width)
+							.getPreferredWidth())
 							: spacing;
 					SubstanceTitlePane.this.heapStatusPanel.setBounds(x, 1,
 							SubstanceTitlePane.this.heapStatusPanel
-									.getPreferredSize().width,
+									.getPreferredWidth(),
 							SubstanceTitlePane.this.getHeight() - 3);
 				}
 			}
@@ -2408,6 +2331,8 @@ public class SubstanceTitlePane extends JComponent {
 				Boolean.valueOf(isVisible));
 		if (!isVisible) {
 			HeapStatusThread.unregisterPanel(this.heapStatusPanel);
+		} else {
+			HeapStatusThread.registerPanel(this.heapStatusPanel);
 		}
 		this.repaint();
 	}
