@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -84,6 +84,11 @@ public class SubstanceIconFactory {
 	 */
 	private static Map<String, Icon> sliderVerticalIcons = new HashMap<String, Icon>();
 
+	/**
+	 * Icons for tree collapse / expand in {@link SubstanceTreeUI}.
+	 */
+	private static Map<String, Icon> treeIcons = new HashMap<String, Icon>();
+
 	/**
 	 * Resets image maps (used when setting new theme).
 	 * 
@@ -96,6 +101,7 @@ public class SubstanceIconFactory {
 		SubstanceIconFactory.sliderHorizontalIcons.clear();
 		SubstanceIconFactory.sliderRoundIcons.clear();
 		SubstanceIconFactory.sliderVerticalIcons.clear();
+		SubstanceIconFactory.treeIcons.clear();
 
 		SubstanceIconFactory.titlePaneIcons.get(IconKind.CLOSE).clear();
 		SubstanceIconFactory.titlePaneIcons.get(IconKind.MINIMIZE).clear();
@@ -155,6 +161,18 @@ public class SubstanceIconFactory {
 		return SubstanceIconFactory.sliderVerticalIcons.get(key);
 	}
 
+	public static Icon getTreeIcon(JTree tree, boolean isCollapsed) {
+		int fontSize = SubstanceSizeUtils.getComponentFontSize(tree);
+		int size = SubstanceSizeUtils.getTreeIconSize(fontSize);
+
+		String key = size + ":" + isCollapsed;
+		if (SubstanceIconFactory.treeIcons.get(key) == null) {
+			Icon icon = new TreeIcon(size, isCollapsed);
+			SubstanceIconFactory.treeIcons.put(key, icon);
+		}
+		return SubstanceIconFactory.treeIcons.get(key);
+	}
+
 	/**
 	 * Trackable slider (for sliders that do not use {@link SubstanceSliderUI}
 	 * as their UI (such as
@@ -280,10 +298,10 @@ public class SubstanceIconFactory {
 
 			float cyclePos = state.getCycleCount();
 
-			SubstanceTheme theme = SubstanceCoreUtilities.getTheme(slider,
-					state, true, false);
-			SubstanceTheme theme2 = SubstanceCoreUtilities.getTheme(slider,
-					prevState, true, false);
+			SubstanceTheme theme = SubstanceThemeUtilities.getTheme(slider,
+					state);
+			SubstanceTheme theme2 = SubstanceThemeUtilities.getTheme(slider,
+					prevState);
 			ColorScheme colorScheme = theme.getColorScheme();
 			ColorScheme colorScheme2 = theme2.getColorScheme();
 			ColorScheme borderScheme = theme.getBorderTheme().getColorScheme();
@@ -334,7 +352,7 @@ public class SubstanceIconFactory {
 			SubstanceGradientPainter painter = SubstanceLookAndFeel
 					.getCurrentGradientPainter();
 			SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-					.getBorderPainter(slider, painter);
+					.getBorderPainter(slider);
 
 			int borderDelta = (int) Math.floor(SubstanceSizeUtils
 					.getBorderStrokeWidth(SubstanceSizeUtils
@@ -492,10 +510,10 @@ public class SubstanceIconFactory {
 
 			float cyclePos = state.getCycleCount();
 
-			SubstanceTheme theme = SubstanceCoreUtilities.getTheme(slider,
-					state, true, false);
-			SubstanceTheme theme2 = SubstanceCoreUtilities.getTheme(slider,
-					prevState, true, false);
+			SubstanceTheme theme = SubstanceThemeUtilities.getTheme(slider,
+					state);
+			SubstanceTheme theme2 = SubstanceThemeUtilities.getTheme(slider,
+					prevState);
 			ColorScheme colorScheme = theme.getColorScheme();
 			ColorScheme colorScheme2 = theme2.getColorScheme();
 			ColorScheme borderScheme = theme.getBorderTheme().getColorScheme();
@@ -548,7 +566,7 @@ public class SubstanceIconFactory {
 			SubstanceGradientPainter painter = SubstanceLookAndFeel
 					.getCurrentGradientPainter();
 			SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-					.getBorderPainter(slider, painter);
+					.getBorderPainter(slider);
 
 			int borderDelta = (int) Math.floor(SubstanceSizeUtils
 					.getBorderStrokeWidth(SubstanceSizeUtils
@@ -704,10 +722,10 @@ public class SubstanceIconFactory {
 
 			float cyclePos = state.getCycleCount();
 
-			SubstanceTheme theme = SubstanceCoreUtilities.getTheme(slider,
-					state, true, false);
-			SubstanceTheme theme2 = SubstanceCoreUtilities.getTheme(slider,
-					prevState, true, false);
+			SubstanceTheme theme = SubstanceThemeUtilities.getTheme(slider,
+					state);
+			SubstanceTheme theme2 = SubstanceThemeUtilities.getTheme(slider,
+					prevState);
 			ColorScheme colorScheme = theme.getColorScheme();
 			ColorScheme colorScheme2 = theme2.getColorScheme();
 			ColorScheme borderScheme = theme.getBorderTheme().getColorScheme();
@@ -760,7 +778,7 @@ public class SubstanceIconFactory {
 			SubstanceGradientPainter painter = SubstanceLookAndFeel
 					.getCurrentGradientPainter();
 			SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-					.getBorderPainter(slider, painter);
+					.getBorderPainter(slider);
 
 			int borderDelta = (int) Math.floor(SubstanceSizeUtils
 					.getBorderStrokeWidth(SubstanceSizeUtils
@@ -865,6 +883,121 @@ public class SubstanceIconFactory {
 		}
 	}
 
+	/**
+	 * Collapse / expand icons for JTrees in {@link SubstanceTreeUI}.
+	 * 
+	 * @author Kirill Grouchnikov
+	 */
+	private static class TreeIcon implements Icon, UIResource {
+		/**
+		 * Icon hash.
+		 */
+		private static Map<String, Icon> icons = new HashMap<String, Icon>();
+
+		/**
+		 * The collapsed indication of this icon
+		 */
+		private boolean isCollapsed;
+
+		private int size;
+
+		/**
+		 * Simple constructor.
+		 * 
+		 * @param isCollapsed
+		 *            The collapsed indication of <code>this</code> icon.
+		 */
+		public TreeIcon(int size, boolean isCollapsed) {
+			this.isCollapsed = isCollapsed;
+			this.size = size;
+		}
+
+		/**
+		 * Retrieves icon that matches the specified state of the slider thumb.
+		 * 
+		 * @param state
+		 *            Slider state.
+		 * @param prevState
+		 *            The previous slider state.
+		 * @param slider
+		 *            The slider itself.
+		 * @param sliderIcon
+		 *            The slider icon.
+		 * @return Icon that matches the specified state of the slider thumb.
+		 */
+		private static synchronized Icon getIcon(JTree tree, boolean isCollapsed) {
+			ComponentState state = ((tree == null) || tree.isEnabled()) ? ComponentState.DEFAULT
+					: ComponentState.DISABLED_UNSELECTED;
+			SubstanceTheme theme = SubstanceThemeUtilities.getTheme(tree, state);
+
+			// if ((tree != null) && SubstanceCoreUtilities.hasColorization(tree)) {
+			// Color treeBackgr = tree.getBackground();
+			// if (!(treeBackgr instanceof UIResource)) {
+			// double colorization = SubstanceCoreUtilities
+			// .getColorizationFactor(tree);
+			// if (!tree.isEnabled())
+			// colorization /= 2.0;
+			// theme = SubstanceShiftTheme.getShiftedTheme(theme,
+			// treeBackgr, colorization, null, 0.0);
+			// }
+			// }
+			//
+			int fontSize = SubstanceSizeUtils.getComponentFontSize(tree);
+
+			String key = fontSize + ":" + theme.getDisplayName() + ":"
+					+ isCollapsed;
+
+			Icon result = TreeIcon.icons.get(key);
+			if (result != null)
+				return result;
+
+			result = new ImageIcon(SubstanceImageCreator.getTreeIcon(tree,
+					theme, isCollapsed));
+			TreeIcon.icons.put(key, result);
+
+			return result;
+		}
+
+		/*
+		 * (non-Javadoc)
+		 * 
+		 * @see javax.swing.Icon#paintIcon(java.awt.Component,
+		 *      java.awt.Graphics, int, int)
+		 */
+		public void paintIcon(Component c, Graphics g, int x, int y) {
+			if (!(g instanceof Graphics2D)) {
+				return;
+			}
+
+			// The following check is here because some applications
+			// (like JIDE's ExpandablePanel) may decide to use the
+			// "Tree.collapsedIcon" and "Tree.expandedIcon" UIManager
+			// entries to paint on non-JTree components. Sigh.
+			JTree tree = (c instanceof JTree) ? (JTree) c : null;
+			Icon iconToDraw = TreeIcon.getIcon(tree, this.isCollapsed);
+
+			iconToDraw.paintIcon(c, g, x, y);
+		}
+
+		/*
+		 * (non-Javadoc)
+		 * 
+		 * @see javax.swing.Icon#getIconWidth()
+		 */
+		public int getIconWidth() {
+			return this.size;
+		}
+
+		/*
+		 * (non-Javadoc)
+		 * 
+		 * @see javax.swing.Icon#getIconHeight()
+		 */
+		public int getIconHeight() {
+			return this.size;
+		}
+	}
+
 	/**
 	 * Icon kind of a title pane button.
 	 * 
