@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -48,9 +48,9 @@ import org.jvnet.substance.*;
 import org.jvnet.substance.button.*;
 import org.jvnet.substance.fonts.FontSet;
 import org.jvnet.substance.painter.*;
+import org.jvnet.substance.painter.decoration.DecorationAreaType;
 import org.jvnet.substance.tabbed.*;
 import org.jvnet.substance.theme.*;
-import org.jvnet.substance.title.TitlePainterChangeListener;
 import org.jvnet.substance.utils.SubstanceConstants;
 import org.jvnet.substance.utils.SubstanceConstants.Side;
 import org.jvnet.substance.utils.SubstanceConstants.TabCloseKind;
@@ -159,17 +159,6 @@ public class Check extends JFrame {
 		}
 	}
 
-	private class MyTitlePainterChangeListener implements
-			TitlePainterChangeListener {
-		public void titlePainterChanged() {
-			if (UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) {
-				out("Substance title painter is '"
-						+ SubstanceLookAndFeel.getCurrentTitlePainterName()
-						+ "'");
-			}
-		}
-	}
-
 	private boolean toUseThemeObjs;
 
 	private boolean toUseWatermarksObjs;
@@ -186,7 +175,7 @@ public class Check extends JFrame {
 
 	private JToolBar toolbar;
 
-	public Check() {
+	public Check(boolean toRegisterListeners) {
 		super(
 				"Substance test with very very very very very very very very very very very very very very long title");
 
@@ -223,7 +212,7 @@ public class Check extends JFrame {
 		// });
 
 		final JXPanel jxPanel = new JXPanel(new BorderLayout());
-		toolbar = getToolbar("", 22);
+		toolbar = getToolbar("", 22, true);
 		jxPanel.add(toolbar, BorderLayout.NORTH);
 
 		JXStatusBar statusBar = getStatusBar(jxPanel, jtp);
@@ -231,7 +220,14 @@ public class Check extends JFrame {
 
 		// JPanel buttons = new JPanel(new FlowLayout());
 
-		taskPaneContainer = new JXTaskPaneContainer();
+		taskPaneContainer = new JXTaskPaneContainer() {
+			@Override
+			public boolean getScrollableTracksViewportWidth() {
+				return false;
+			}
+		};
+		taskPaneContainer.setScrollableTracksViewportHeight(false);
+		taskPaneContainer.setScrollableTracksViewportWidth(false);
 		taskPaneContainer.putClientProperty(
 				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
 				new ClassicButtonShaper());
@@ -263,7 +259,9 @@ public class Check extends JFrame {
 
 		final JScrollPane scrollPane = new JScrollPane(taskPaneContainer,
 				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
-				JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
+				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
+		SubstanceLookAndFeel.setDecorationType(scrollPane,
+				DecorationAreaType.GENERAL);
 		scrollPane.setOpaque(false);
 		scrollPane.getViewport().setOpaque(false);
 
@@ -448,19 +446,21 @@ public class Check extends JFrame {
 		smallVerticalButtonPanel.setLayout(new GridLayout(4, 4));
 		for (int row = 0; row < 4; row++) {
 			for (int col = 0; col < 4; col++) {
-				smallVerticalButtonPanel.add(new JButton("vert"));
+				JButton vertButton = new JButton("vert");
+				vertButton.setToolTipText("Vertical button " + row + ":" + col);
+				smallVerticalButtonPanel.add(vertButton);
 			}
 		}
 		verticalButtonPanel.add(smallVerticalButtonPanel);
 		jtp.addTab("Vertical buttons", verticalButtonPanel);
 
-		jtp.addTab("Colored components", new ColoredControlsPanel());
+		jtp.addTab("Colored", new ColoredControlsPanel());
+
+		jtp.addTab("Colorized", new ColorizedControlsPanel());
 
 		JPanel buttonStripPanel = new JPanel();
 		buttonStripPanel.setLayout(new FlowLayout());
 
-		jtp.addTab("Theme combo", new ThemesPanel());
-
 		jtp.addTab("Sizes", new SizesPanel());
 
 		jtp.addTab("Alignment", AlignmentPanel.getAlignmentPanel());
@@ -826,23 +826,17 @@ public class Check extends JFrame {
 		// toolBar3.add(translucencySlider);
 		// this.add(toolBar3, AKDockLayout.NORTH);
 
-		// register sample listeners
-		SubstanceLookAndFeel
-				.registerThemeChangeListener(new MyThemeChangeListener());
-		SubstanceLookAndFeel
-				.registerWatermarkChangeListener(new MyWatermarkChangeListener());
-		SubstanceLookAndFeel
-				.registerButtonShaperChangeListener(new MyButtonShaperChangeListener());
-		SubstanceLookAndFeel
-				.registerGradientPainterChangeListener(new MyGradientPainterChangeListener());
-		SubstanceLookAndFeel
-				.registerTitlePainterChangeListener(new MyTitlePainterChangeListener());
-		// SubstanceLookAndFeel
-		// .registerSkinChangeListener(new SkinChangeListener() {
-		// public void skinChanged() {
-		// TransitionLayoutManager.getInstance().track(jtp, true);
-		// }
-		// });
+		if (toRegisterListeners) {
+			// register sample listeners
+			SubstanceLookAndFeel
+					.registerThemeChangeListener(new MyThemeChangeListener());
+			SubstanceLookAndFeel
+					.registerWatermarkChangeListener(new MyWatermarkChangeListener());
+			SubstanceLookAndFeel
+					.registerButtonShaperChangeListener(new MyButtonShaperChangeListener());
+			SubstanceLookAndFeel
+					.registerGradientPainterChangeListener(new MyGradientPainterChangeListener());
+		}
 
 		TabCloseCallback closeCallbackMain = new TabCloseCallback() {
 			public TabCloseKind onAreaClick(JTabbedPane tabbedPane,
@@ -1045,12 +1039,12 @@ public class Check extends JFrame {
 		statusBar.add(fontSizePanel, fontSizePanelConstraints);
 
 		JPanel alphaPanel = new JPanel(new FlowLayout(FlowLayout.LEADING, 0, 0));
-		alphaPanel.setOpaque(false);
+		// alphaPanel.setOpaque(false);
 
 		final JLabel alphaLabel = new JLabel("100%");
 		final JSlider alphaSlider = new JSlider(0, 100, 100);
 		alphaSlider.setFocusable(false);
-		alphaSlider.setOpaque(false);
+		// alphaSlider.setOpaque(false);
 		alphaSlider.addChangeListener(new ChangeListener() {
 			public void stateChanged(ChangeEvent e) {
 				int currValue = alphaSlider.getValue();
@@ -1143,50 +1137,10 @@ public class Check extends JFrame {
 		// }
 		// System.setProperty("sun.awt.noerasebackground", "false");
 
-		if (UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) {
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceMixTheme(
-					new SubstanceAquaTheme(), new SubstanceBottleGreenTheme()));
-			SubstanceLookAndFeel.addMixedThemeBy(new SubstanceBarbyPinkTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceAquaTheme(),
-					new SubstanceLightAquaTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceAquaTheme(),
-					new SubstanceBottleGreenTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceBarbyPinkTheme(),
-					new SubstanceRaspberryTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceBottleGreenTheme(),
-					new SubstanceLimeGreenTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceBrownTheme(),
-					new SubstanceSunGlareTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceSunsetTheme(),
-					new SubstanceOrangeTheme());
-
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceBottleGreenTheme(),
-					new SubstanceLimeGreenTheme(),
-					new SubstanceBottleGreenTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceAquaTheme(),
-					new SubstanceLimeGreenTheme(),
-					new SubstanceBottleGreenTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceBarbyPinkTheme(),
-					new SubstancePurpleTheme(), new SubstanceRaspberryTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceRaspberryTheme(),
-					new SubstanceSunsetTheme(), new SubstanceOrangeTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceBrownTheme(),
-					new SubstanceSunGlareTheme(), new SubstanceOrangeTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceAquaTheme(),
-					new SubstanceLightAquaTheme(), new SubstancePurpleTheme(),
-					new SubstanceBarbyPinkTheme());
-			SubstanceLookAndFeel.addMixedTheme(new SubstanceBrownTheme(),
-					new SubstanceTerracottaTheme(),
-					new SubstanceSunGlareTheme(), new SubstanceOrangeTheme(),
-					new SubstanceSunsetTheme());
-		}
-		// SubstanceLookAndFeel.addMixedDarkThemeBy(new
-		// SubstanceCharcoalTheme());
-
 		SwingUtilities.invokeLater(new Runnable() {
 			public void run() {
 				long time2 = System.currentTimeMillis();
-				Check c = new Check();
+				Check c = new Check(true);
 				c.addComponentListener(new ComponentAdapter() {
 					@Override
 					public void componentResized(ComponentEvent e) {
@@ -1203,7 +1157,10 @@ public class Check extends JFrame {
 						.getHeight()) / 2);
 
 				c.setVisible(true);
-				c.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
+				c
+						.setDefaultCloseOperation(System
+								.getProperty("javawebstart.version") != null ? JFrame.EXIT_ON_CLOSE
+								: JFrame.DISPOSE_ON_CLOSE);
 				long time3 = System.currentTimeMillis();
 				out("App " + (time3 - time2));
 			}
@@ -1245,14 +1202,21 @@ public class Check extends JFrame {
 		return null;
 	}
 
-	public static JToolBar getToolbar(String label, int size) {
+	public static JToolBar getToolbar(String label, int size, boolean hasStrings) {
 		JToolBar toolBar = new JToolBar();
 		// toolBar.setLayout(new BoxLayout(toolBar,BoxLayout.LINE_AXIS));
 		// toolBar.setFloatable(false);
 
-		JButton buttonCut = new JButton(getIcon(size + "/edit-cut"));
+		JButton buttonCut = new JButton(hasStrings ? "cut" : null, getIcon(size
+				+ "/edit-cut"));
+		buttonCut.putClientProperty(
+				SubstanceLookAndFeel.BUTTON_NO_MIN_SIZE_PROPERTY, Boolean.TRUE);
 		toolBar.add(buttonCut);
-		JButton buttonCopy = new JButton(getIcon(size + "/edit-copy"));
+		JButton buttonCopy = new JButton(hasStrings ? "copy" : null,
+				getIcon(size + "/edit-copy"));
+		buttonCopy.putClientProperty(
+				SubstanceLookAndFeel.BUTTON_NO_MIN_SIZE_PROPERTY, Boolean.TRUE);
+		buttonCopy.setEnabled(false);
 		toolBar.add(buttonCopy);
 		JButton buttonPaste = new JButton(getIcon(size + "/edit-paste"));
 		toolBar.add(buttonPaste);
@@ -1343,6 +1307,7 @@ public class Check extends JFrame {
 
 		toolBar.add(Box.createGlue());
 		JButton buttonExit = new JButton(getIcon(size + "/process-stop"));
+		buttonExit.setToolTipText("Closes the test application");
 		buttonExit.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				System.exit(0);
@@ -1452,4 +1417,8 @@ public class Check extends JFrame {
 			}
 		}
 	}
+
+	public JTabbedPane getMainTabbedPane() {
+		return this.jtp;
+	}
 }
