@@ -30,11 +30,11 @@ public class Checkmark extends JFrame {
 				JCheckBox jcb = new JCheckBox();
 				jcb.setSelected(true);
 				BufferedImage cbi = SubstanceImageCreator.getCheckBox(jcb, i,
-						ComponentState.SELECTED, st, st, 0, 1.0f);
+						ComponentState.SELECTED, st, st, 0, 1.0f, false);
 				g.setColor(new Color(255, 196, 196));
 				g.fillRect(x, 100, cbi.getWidth(), cbi.getHeight());
 				g.drawImage(SubstanceImageCreator.getCheckBox(jcb, i,
-						ComponentState.SELECTED, st, st, 0, 1.0f),
+						ComponentState.SELECTED, st, st, 0, 1.0f, false),
 						x, 100, null);
 				x += (i+2);
 			}
