@@ -1,17 +1,12 @@
 package test;
 
-import java.awt.*;
-import java.awt.event.ActionEvent;
-import java.awt.event.ActionListener;
+import java.awt.BorderLayout;
+import java.awt.FlowLayout;
 
 import javax.swing.*;
-import javax.swing.plaf.metal.MetalLookAndFeel;
 
-import org.jvnet.lafwidget.LafWidget;
-import org.jvnet.lafwidget.utils.LafConstants.AnimationKind;
 import org.jvnet.substance.SubstanceLookAndFeel;
 import org.jvnet.substance.skin.SubstanceBusinessBlackSteelLookAndFeel;
-import org.jvnet.substance.skin.SubstanceFieldOfWheatLookAndFeel;
 
 public class Menu extends JFrame {
 	public Menu() {
@@ -22,7 +17,8 @@ public class Menu extends JFrame {
 		JMenu menu1 = new JMenu("menu1");
 		JMenu submenu = new JMenu("Submenu");
 		submenu.add(new JMenuItem("submenuitem 111"));
-		menu1.add(submenu);
+		// menu1.add(submenu);
+		menu1.add(new JCheckBoxMenuItem("Submenu"));
 		jmb.add(menu1);
 		this.setJMenuBar(jmb);
 
@@ -35,8 +31,8 @@ public class Menu extends JFrame {
 	}
 
 	public static void main(String[] args) throws Exception {
-		UIManager.setLookAndFeel(new SubstanceFieldOfWheatLookAndFeel());
-		//UIManager.put(LafWidget.ANIMATION_KIND, AnimationKind.DEBUG);
+		UIManager.setLookAndFeel(new SubstanceBusinessBlackSteelLookAndFeel());
+		// UIManager.put(LafWidget.ANIMATION_KIND, AnimationKind.DEBUG);
 		UIManager.put(SubstanceLookAndFeel.USE_THEMED_DEFAULT_ICONS,
 				Boolean.TRUE);
 		JFrame.setDefaultLookAndFeelDecorated(true);
