@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -45,6 +45,7 @@ import org.jvnet.substance.utils.SubstanceConstants.FocusKind;
 import org.jvnet.substance.utils.SubstanceConstants.Side;
 
 import test.Check;
+import test.check.command.*;
 
 import com.jgoodies.forms.builder.DefaultFormBuilder;
 import com.jgoodies.forms.layout.FormLayout;
@@ -61,40 +62,41 @@ public class ButtonsPanel extends JPanel {
 	 */
 	public JButton defaultButton;
 
-	/**
-	 * A configure command that disables the specified button.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	private class DisableCommand implements
-			ConfigurationCommand<AbstractButton> {
-		/*
-		 * (non-Javadoc)
-		 * 
-		 * @see test.check.ConfigurationCommand#invoke(java.lang.Object)
-		 */
-		public void configure(AbstractButton ab) {
-			ab.setEnabled(false);
-		}
-	}
+	// /**
+	// * A configure command that disables the specified button.
+	// *
+	// * @author Kirill Grouchnikov
+	// */
+	// private class DisableCommand implements
+	// ConfigurationCommand<AbstractButton> {
+	// /*
+	// * (non-Javadoc)
+	// *
+	// * @see test.check.ConfigurationCommand#invoke(java.lang.Object)
+	// */
+	// public void configure(AbstractButton ab) {
+	// ab.setEnabled(false);
+	// }
+	// }
 
-	/**
-	 * A configure command that selects the specified button.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	private class SelectCommand implements ConfigurationCommand<AbstractButton> {
-		/*
-		 * (non-Javadoc)
-		 * 
-		 * @see test.check.ConfigurationCommand#invoke(java.lang.Object)
-		 */
-		public void configure(AbstractButton ab) {
-			if (ab instanceof JButton)
-				return;
-			ab.setSelected(true);
-		}
-	}
+	// /**
+	// * A configure command that selects the specified button.
+	// *
+	// * @author Kirill Grouchnikov
+	// */
+	// private class SelectCommand implements
+	// ConfigurationCommand<AbstractButton> {
+	// /*
+	// * (non-Javadoc)
+	// *
+	// * @see test.check.ConfigurationCommand#invoke(java.lang.Object)
+	// */
+	// public void configure(AbstractButton ab) {
+	// if (ab instanceof JButton)
+	// return;
+	// ab.setSelected(true);
+	// }
+	// }
 
 	/**
 	 * A configure command that sets a specified theme on the specified button.
@@ -580,47 +582,6 @@ public class ButtonsPanel extends JPanel {
 		}
 	}
 
-	/**
-	 * A configure command that sets the specified client property to the
-	 * specified value.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	private class ClientPropertyCommand implements
-			ConfigurationCommand<AbstractButton> {
-		/**
-		 * Client property name.
-		 */
-		private String propName;
-
-		/**
-		 * Client property value.
-		 */
-		private Object propValue;
-
-		/**
-		 * Creates an client property configuration command.
-		 * 
-		 * @param propName
-		 *            Client property name.
-		 * @param propValue
-		 *            Client property value.
-		 */
-		public ClientPropertyCommand(String propName, Object propValue) {
-			this.propName = propName;
-			this.propValue = propValue;
-		}
-
-		/*
-		 * (non-Javadoc)
-		 * 
-		 * @see test.check.ConfigurationCommand#invoke(java.lang.Object)
-		 */
-		public void configure(AbstractButton ab) {
-			ab.putClientProperty(this.propName, this.propValue);
-		}
-	}
-
 	/**
 	 * A configure command that marks the button to not have content area
 	 * filled.
@@ -668,12 +629,19 @@ public class ButtonsPanel extends JPanel {
 	private AbstractButton[] getRow() {
 		AbstractButton[] result = new AbstractButton[4];
 		result[0] = new JButton("sample");
+		result[0].setName("Button " + rowCount);
 		result[1] = new JToggleButton("sample");
+		result[1].setName("Toggle " + rowCount);
 		result[2] = new JCheckBox("sample");
+		result[2].setName("Check " + rowCount);
 		result[3] = new JRadioButton("sample");
+		result[3].setName("Radio " + rowCount);
+		rowCount++;
 		return result;
 	}
 
+	private int rowCount = 0;
+
 	/**
 	 * Adds a row of buttons configured with the specified text, icon and
 	 * configuration command.
@@ -688,7 +656,7 @@ public class ButtonsPanel extends JPanel {
 	 *            Configuration command to apply.
 	 */
 	private void addRow(DefaultFormBuilder builder, String label, Icon icon,
-			ConfigurationCommand<AbstractButton> cmd) {
+			ConfigurationCommand<? super AbstractButton> cmd) {
 		AbstractButton[] row = this.getRow();
 		if (cmd != null) {
 			for (AbstractButton ab : row) {
@@ -716,7 +684,8 @@ public class ButtonsPanel extends JPanel {
 						+ "left:pref:grow(1), 4dlu, left:pref:grow(1), "
 						+ "4dlu, left:pref:grow(1)", "");
 		lm.setColumnGroups(new int[][] { { 3, 5, 7, 9 } });
-		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
+		DefaultFormBuilder builder = new DefaultFormBuilder(lm,
+				new ScrollablePanel());
 		builder.setDefaultDialogBorder();
 
 		builder.append("");
@@ -743,12 +712,16 @@ public class ButtonsPanel extends JPanel {
 		this.addRow(builder, "Disabled", null, new DisableCommand());
 		this.addRow(builder, "Selected", null, new SelectCommand());
 		this.addRow(builder, "Disabled selected", null,
-				new ChainCommand<AbstractButton>(new DisableCommand(),
+				new ChainCommand<Component>(new DisableCommand(),
 						new SelectCommand()));
 		this.addRow(builder, "HTML text", null, new TextCommand(
 				"<html>text <b>text</b> <font color='red'>text</font>"));
 		this.addRow(builder, "With tooltip", null, new TooltipTextCommand(
 				"Sample tooltip"));
+		this
+				.addRow(builder, "Disabled with tooltip", null,
+						new ChainCommand(new TooltipTextCommand(
+								"Sample tooltip"), new DisableCommand()));
 		this.addRow(builder, "Popup menu", null, new PopupMenuCommand());
 		this.addRow(builder, "With icon", Check.getIcon("flag_sweden"),
 				new IconCommand(Check.getIcon("flag_sweden")));
@@ -914,10 +887,13 @@ public class ButtonsPanel extends JPanel {
 
 		this.addRow(builder, "Fixed font", null, new FontCommand(new Font(
 				"Arial", Font.PLAIN, 12)));
+		this.addRow(builder, "Null text", null, new TextCommand(null));
+		this.addRow(builder, "Empty text", null, new TextCommand(""));
 
 		JPanel panel = builder.getPanel();
 		JScrollPane jsp = new JScrollPane(panel);
 		panel.setOpaque(false);
+		jsp.putClientProperty(LafWidget.AUTO_SCROLL, Boolean.TRUE);
 		jsp.setOpaque(false);
 		jsp.getViewport().setOpaque(false);
 		this.add(jsp, BorderLayout.CENTER);
