@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -31,16 +31,23 @@ package test.check;
 
 import java.awt.*;
 import java.awt.event.*;
+import java.io.IOException;
+import java.util.Map;
+import java.util.Vector;
 
+import javax.imageio.ImageIO;
 import javax.swing.*;
 
 import org.jvnet.lafwidget.LafWidget;
 import org.jvnet.lafwidget.combo.AutoCompletionMatcher;
-import org.jvnet.substance.SubstanceLookAndFeel;
+import org.jvnet.substance.*;
 import org.jvnet.substance.combo.ComboPopupPrototypeCallback;
 import org.jvnet.substance.combo.WidestComboPopupPrototype;
+import org.jvnet.substance.theme.SubstanceTheme;
+import org.jvnet.substance.theme.ThemeInfo;
 
 import test.Check;
+import test.check.command.*;
 
 import com.jgoodies.forms.builder.DefaultFormBuilder;
 import com.jgoodies.forms.layout.FormLayout;
@@ -52,36 +59,34 @@ import com.jgoodies.forms.layout.FormLayout;
  */
 public class CombosPanel extends ControllablePanel {
 	/**
-	 * A configure command that enables the specified combobox.
+	 * Renderer for the combobox containing the theme list.
 	 * 
 	 * @author Kirill Grouchnikov
 	 */
-	public static class EnableCommand implements
-			ConfigurationCommand<JComboBox> {
-		/*
-		 * (non-Javadoc)
-		 * 
-		 * @see test.check.ConfigurationCommand#invoke(java.lang.Object)
+	protected static class ThemeComboRenderer extends JLabel implements
+			ListCellRenderer {
+		/**
+		 * The current theme.
 		 */
-		public void configure(JComboBox combo) {
-			combo.setEnabled(true);
-		}
-	}
+		protected SubstanceTheme currTheme;
 
-	/**
-	 * A configure command that disables the specified combobox.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	public static class DisableCommand implements
-			ConfigurationCommand<JComboBox> {
-		/*
-		 * (non-Javadoc)
-		 * 
-		 * @see test.check.ConfigurationCommand#invoke(java.lang.Object)
-		 */
-		public void configure(JComboBox combo) {
-			combo.setEnabled(false);
+		public Component getListCellRendererComponent(JList list, Object value,
+				int index, boolean isSelected, boolean cellHasFocus) {
+			SubstanceTheme theme = (SubstanceTheme) value;
+			this.setBackground(theme.getColorScheme().getMidColor());
+			this.setForeground(theme.getColorScheme().getForegroundColor());
+			this.setText(theme.getDisplayName());
+			this.currTheme = theme;
+			return this;
+		}
+
+		@Override
+		public void paint(Graphics g) {
+			int width = this.getWidth();
+			int height = this.getHeight();
+			SubstanceImageCreator.paintRectangularBackground(g, 0, 0, width,
+					height, this.currTheme.getColorScheme(), 0.0f, false);
+			super.paint(g);
 		}
 	}
 
@@ -129,7 +134,7 @@ public class CombosPanel extends ControllablePanel {
 	 *            Configuration command to apply.
 	 */
 	public static void run(Component comp,
-			ConfigurationCommand<JComboBox> command) {
+			ConfigurationCommand<? super JComboBox> command) {
 		if (comp instanceof JComboBox) {
 			command.configure((JComboBox) comp);
 			return;
@@ -259,7 +264,8 @@ public class CombosPanel extends ControllablePanel {
 	 */
 	private JPanel getLeftComboPanel() {
 		FormLayout lm = new FormLayout("right:pref, 4dlu, fill:pref:grow", "");
-		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
+		DefaultFormBuilder builder = new DefaultFormBuilder(lm,
+				new ScrollablePanel());
 		builder.setDefaultDialogBorder();
 
 		builder.appendSeparator("Simple combos");
@@ -286,9 +292,41 @@ public class CombosPanel extends ControllablePanel {
 		JComboBox comboColored = new JComboBox(new Object[] { "entry31",
 				"entry32", "entry33", "entry34", "entry35", "entry36", "aaa",
 				"abb", "abc" });
+		comboColored.setName("Colored combo");
 		comboColored.setBackground(new Color(255, 128, 128));
 		comboColored.setForeground(new Color(0, 0, 128));
-		builder.append("Colored", comboColored);
+		builder.append("Pink background", comboColored);
+
+		JComboBox comboColors = new JComboBox(new Object[] {
+				new Color(255, 128, 128), new Color(128, 255, 128),
+				new Color(128, 128, 255), new Color(255, 255, 128),
+				new Color(255, 128, 255), new Color(128, 255, 255), });
+		comboColors.setName("Colors combo");
+		comboColors.setRenderer(new SubstanceDefaultListCellRenderer() {
+			public Component getListCellRendererComponent(JList list,
+					Object value, int index, boolean isSelected,
+					boolean cellHasFocus) {
+				JLabel comp = (JLabel) super.getListCellRendererComponent(list,
+						value, index, isSelected, cellHasFocus);
+				Color color = (Color) value;
+				comp.setBackground(color);
+				comp.setText(color.getRed() + ":" + color.getGreen() + ":"
+						+ color.getBlue());
+				return comp;
+			}
+		});
+		builder.append("Colors", comboColors);
+
+		JComboBox comboFlat = new JComboBox(new Object[] { "entry1", "entry2",
+				"entry3", "entry4", "entry5", "entry6" });
+		comboFlat.putClientProperty(SubstanceLookAndFeel.FLAT_PROPERTY,
+				Boolean.TRUE);
+		builder.append("Flat button", comboFlat);
+
+		JComboBox comboDefaultCoreRenderer = new JComboBox(new Object[] {
+				"entry1", "entry2", "entry3", "entry4", "entry5", "entry6" });
+		comboDefaultCoreRenderer.setRenderer(new DefaultListCellRenderer());
+		builder.append("Default core renderer", comboDefaultCoreRenderer);
 
 		builder.appendSeparator("Auto-completion");
 
@@ -408,6 +446,18 @@ public class CombosPanel extends ControllablePanel {
 				LafWidget.COMBO_BOX_NO_AUTOCOMPLETION, Boolean.TRUE);
 		builder.append("No completion", comboNoAutoCompletion);
 
+		final JComboBox comboCompletionColored = new JComboBox(new Object[] {
+				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
+		comboCompletionColored.setEditable(true);
+		comboCompletionColored.addActionListener(new ActionListener() {
+			public void actionPerformed(ActionEvent e) {
+				Check.out("'" + comboCompletionColored.getSelectedItem()
+						+ "' selected");
+			}
+		});
+		comboCompletionColored.setBackground(new Color(255, 128, 128));
+		builder.append("Colored", comboCompletionColored);
+
 		builder.appendSeparator("Miscellaneous");
 
 		JComboBox colorCombo = new ColorComboBox();
@@ -424,6 +474,42 @@ public class CombosPanel extends ControllablePanel {
 		comboHebrew.setMaximumRowCount(6);
 		builder.append("RTL (Hebrew)", comboHebrew);
 
+		JComboBox comboHebrewCustomRenderer = new JComboBox(new Object[] {
+				"\u05e8\u05d0\u05e9\u05d9 1", "\u05e8\u05d0\u05e9\u05d9 2",
+				"\u05e8\u05d0\u05e9\u05d9 3", "\u05e8\u05d0\u05e9\u05d9 4",
+				"\u05e8\u05d0\u05e9\u05d9 5", "\u05e8\u05d0\u05e9\u05d9 6",
+				"\u05e8\u05d0\u05e9\u05d9 7", "\u05e8\u05d0\u05e9\u05d9 8",
+				"\u05e8\u05d0\u05e9\u05d9 9" });
+		comboHebrewCustomRenderer.setToolTipText("RTL combo");
+		comboHebrewCustomRenderer
+				.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
+		comboHebrewCustomRenderer.setMaximumRowCount(6);
+		comboHebrewCustomRenderer
+				.setRenderer(new SubstanceDefaultComboBoxRenderer(
+						comboHebrewCustomRenderer) {
+					private static final long serialVersionUID = 2629717016682715818L;
+
+					@Override
+					public Component getListCellRendererComponent(JList list,
+							Object value, int index, boolean isSelected,
+							boolean hasFocus) {
+						Component c = super.getListCellRendererComponent(list,
+								value, index, isSelected, hasFocus);
+						try {
+							((JLabel) c)
+									.setIcon(new ImageIcon(
+											ImageIO
+													.read(getClass()
+															.getResource(
+																	"/test/check/icons/flag_israel.png"))));
+						} catch (IOException e) {
+							e.printStackTrace();
+						}
+						return c;
+					}
+				});
+		builder.append("RTL (Hebrew) with icon", comboHebrewCustomRenderer);
+
 		final JComboBox combo151 = new JComboBox();
 		final ComboBoxTextEditor151 editor = new ComboBoxTextEditor151();
 		combo151.setEditor(editor);
@@ -446,6 +532,44 @@ public class CombosPanel extends ControllablePanel {
 		});
 		builder.append("Defect 151", combo151);
 
+		if (UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) {
+			Vector<SubstanceTheme> themeVector = new Vector<SubstanceTheme>();
+			Map<String, ThemeInfo> allThemes = SubstanceLookAndFeel
+					.getAllThemes();
+			for (Map.Entry<String, ThemeInfo> themeEntry : allThemes.entrySet()) {
+				themeVector.add(SubstanceTheme.createInstance(themeEntry
+						.getValue()));
+			}
+			JComboBox themeCombo = new JComboBox(themeVector);
+			themeCombo.addItemListener(new ItemListener() {
+				public void itemStateChanged(ItemEvent evt) {
+					// Get the affected item
+					final Object item = evt.getItem();
+
+					if (evt.getStateChange() == ItemEvent.SELECTED) {
+						SwingUtilities.invokeLater(new Runnable() {
+							public void run() {
+								try {
+									SubstanceLookAndFeel
+											.setCurrentTheme((SubstanceTheme) item);
+									// UIManager
+									// .setLookAndFeel(new
+									// SubstanceLookAndFeel());
+									for (Frame frame : Frame.getFrames()) {
+										SwingUtilities
+												.updateComponentTreeUI(frame);
+									}
+								} catch (Exception exc) {
+								}
+							};
+						});
+					}
+				}
+			});
+			themeCombo.setRenderer(new ThemeComboRenderer());
+			builder.append("Themes", themeCombo);
+		}
+
 		return builder.getPanel();
 	}
 
@@ -456,7 +580,8 @@ public class CombosPanel extends ControllablePanel {
 	 */
 	private JPanel getRightComboPanel() {
 		FormLayout lm = new FormLayout("right:pref, 4dlu, left:pref:grow", "");
-		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
+		DefaultFormBuilder builder = new DefaultFormBuilder(lm,
+				new ScrollablePanel());
 		builder.setDefaultDialogBorder();
 
 		builder.appendSeparator("Popup flyout");
@@ -570,7 +695,8 @@ public class CombosPanel extends ControllablePanel {
 	public CombosPanel() {
 		this.setLayout(new BorderLayout());
 
-		JPanel gridPanel = new JPanel(new GridLayout(1, 2));
+		JPanel gridPanel = new ScrollablePanel();
+		gridPanel.setLayout(new GridLayout(1, 2));
 		gridPanel.add(getLeftComboPanel());
 		gridPanel.add(getRightComboPanel());
 		this.add(new JScrollPane(gridPanel,
@@ -579,7 +705,8 @@ public class CombosPanel extends ControllablePanel {
 				BorderLayout.CENTER);
 
 		FormLayout lm = new FormLayout("fill:pref:grow", "");
-		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
+		DefaultFormBuilder builder = new DefaultFormBuilder(lm,
+				new ScrollablePanel());
 
 		JButton checkEscape = new JButton("Check ESC key");
 		checkEscape.addActionListener(new ActionListener() {
