@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -48,6 +48,7 @@ import org.jvnet.substance.SubstanceDefaultComboBoxRenderer;
 import org.jvnet.substance.SubstanceLookAndFeel;
 import org.jvnet.substance.painter.AlphaControlBackgroundComposite;
 import org.jvnet.substance.utils.SubstanceCoreUtilities;
+import org.jvnet.substance.utils.SubstanceConstants.FocusKind;
 import org.jvnet.substance.utils.SubstanceConstants.MenuGutterFillKind;
 
 import test.Check;
@@ -259,6 +260,28 @@ public class ControlPanelFactory {
 		});
 		builder.append("Menu fill", menuGutterFillCombo);
 
+		final JComboBox focusKindCombo = new JComboBox(FocusKind.values());
+		focusKindCombo.setRenderer(new SubstanceDefaultComboBoxRenderer(
+				focusKindCombo) {
+			@Override
+			public Component getListCellRendererComponent(JList list,
+					Object value, int index, boolean isSelected,
+					boolean cellHasFocus) {
+				FocusKind focusKind = (FocusKind) value;
+				return super.getListCellRendererComponent(list, focusKind
+						.name().toLowerCase(), index, isSelected, cellHasFocus);
+			}
+		});
+		focusKindCombo.setSelectedItem(SubstanceCoreUtilities
+				.getFocusKind(null));
+		focusKindCombo.addActionListener(new ActionListener() {
+			public void actionPerformed(ActionEvent e) {
+				UIManager.put(SubstanceLookAndFeel.FOCUS_KIND, focusKindCombo
+						.getSelectedItem());
+			}
+		});
+		builder.append("Focus kind", focusKindCombo);
+
 		// final JCheckBox fullScreenMode = new JCheckBox("Full screen mode");
 		// fullScreenMode.setSelected(false);
 		// fullScreenMode.addActionListener(new ActionListener() {
@@ -478,8 +501,12 @@ public class ControlPanelFactory {
 				.getIcon("JFileChooserColor16"));
 		bfo.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JFileChooser jfc = new JFileChooser();
-				jfc.showOpenDialog(mainFrame);
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JFileChooser jfc = new JFileChooser();
+						jfc.showOpenDialog(mainFrame);
+					}
+				});
 			}
 		});
 		builder.append("File chooser", bfo);
@@ -488,8 +515,12 @@ public class ControlPanelFactory {
 				.getIcon("JFileChooserColor16"));
 		bfs.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JFileChooser jfc = new JFileChooser();
-				jfc.showSaveDialog(mainFrame);
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JFileChooser jfc = new JFileChooser();
+						jfc.showSaveDialog(mainFrame);
+					}
+				});
 			}
 		});
 		builder.append("", bfs);
@@ -497,11 +528,15 @@ public class ControlPanelFactory {
 		JButton bc = new JButton("Open", Check.getIcon("JColorChooserColor16"));
 		bc.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				Color color = JColorChooser.showDialog(mainFrame,
-						"Color chooser", new Color(23, 45, 200));
-				if (color != null) {
-					Check.out("Chosen " + color.toString());
-				}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						Color color = JColorChooser.showDialog(mainFrame,
+								"Color chooser", new Color(23, 45, 200));
+						if (color != null) {
+							Check.out("Chosen " + color.toString());
+						}
+					}
+				});
 			}
 		});
 		builder.append("Color chooser", bc);
@@ -536,11 +571,16 @@ public class ControlPanelFactory {
 		JButton bop = new JButton("Show");
 		bop.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JOptionPane pane = new JOptionPane("Sample option pane");
-				JDialog dialog = pane.createDialog(mainFrame, "Sample title");
-				dialog.setVisible(true);
-				dialog.dispose();
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JOptionPane pane = new JOptionPane("Sample option pane");
+						JDialog dialog = pane.createDialog(mainFrame,
+								"Sample title");
+						dialog.setVisible(true);
+						dialog.dispose();
+					}
+				});
+			};
 		});
 		builder.append("Plain", bop);
 
@@ -548,49 +588,70 @@ public class ControlPanelFactory {
 				.getIcon("22/dialog-information"));
 		bopi.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JOptionPane.showMessageDialog(mainFrame, "Sample info message",
-						"Sample title", JOptionPane.INFORMATION_MESSAGE);
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JOptionPane.showMessageDialog(mainFrame,
+								"Sample info message", "Sample title",
+								JOptionPane.INFORMATION_MESSAGE);
+					}
+				});
+			};
 		});
 		builder.append("Info", bopi);
 
 		JButton bope = new JButton("Show", Check.getIcon("22/dialog-error"));
 		bope.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JOptionPane.showMessageDialog(mainFrame,
-						"Sample error message", "Sample title",
-						JOptionPane.ERROR_MESSAGE);
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JOptionPane.showMessageDialog(mainFrame,
+								"Sample error message", "Sample title",
+								JOptionPane.ERROR_MESSAGE);
+					}
+				});
+			};
 		});
 		builder.append("Error", bope);
 
 		JButton bopw = new JButton("Show", Check.getIcon("22/dialog-warning"));
 		bopw.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JOptionPane.showMessageDialog(mainFrame,
-						"Sample warning message", "Sample title",
-						JOptionPane.WARNING_MESSAGE);
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JOptionPane.showMessageDialog(mainFrame,
+								"Sample warning message", "Sample title",
+								JOptionPane.WARNING_MESSAGE);
+					}
+				});
+			};
 		});
 		builder.append("Warning", bopw);
 
 		JButton bopq = new JButton("Show", Check.getIcon("22/help-browser"));
 		bopq.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JOptionPane.showMessageDialog(mainFrame,
-						"Sample question message", "Sample title",
-						JOptionPane.QUESTION_MESSAGE);
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JOptionPane.showMessageDialog(mainFrame,
+								"Sample question message", "Sample title",
+								JOptionPane.QUESTION_MESSAGE);
+					}
+				});
+			};
 		});
 		builder.append("Question", bopq);
 
 		JButton bopc = new JButton("Show");
 		bopc.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JOptionPane.showOptionDialog(mainFrame, new JPanel(), "Option",
-						JOptionPane.OK_CANCEL_OPTION,
-						JOptionPane.PLAIN_MESSAGE, null, null, null);
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JOptionPane.showOptionDialog(mainFrame, new JPanel(),
+								"Option", JOptionPane.OK_CANCEL_OPTION,
+								JOptionPane.PLAIN_MESSAGE, null, null, null);
+					}
+				});
+			};
 		});
 		builder.append("Custom", bopc);
 
@@ -604,7 +665,7 @@ public class ControlPanelFactory {
 						dialog.setSize(400, 300);
 						dialog.setLocationRelativeTo(mainFrame);
 						dialog.setLayout(new BorderLayout());
-						JPanel panel = new JPanel();
+						JDesktopPane panel = new JDesktopPane();
 						dialog.add(panel, BorderLayout.CENTER);
 						// dialog.setVisible(true);
 						JOptionPane.showInternalInputDialog(panel,
@@ -622,74 +683,89 @@ public class ControlPanelFactory {
 		JButton openDisposable = new JButton("Open");
 		openDisposable.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				if (disposableDialog != null) {
-					disposableDialog.setVisible(true);
-					return;
-				}
-				disposableDialog = new JDialog();
-				disposableDialog.setTitle("Disposable");
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						if (disposableDialog != null) {
+							disposableDialog.setVisible(true);
+							return;
+						}
+						disposableDialog = new JDialog();
+						disposableDialog.setTitle("Disposable");
 
-				JTree tree = new JTree();
-				JScrollPane jsp = new JScrollPane(tree,
-						JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
-						JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
-				jsp.putClientProperty(LafWidget.COMPONENT_PREVIEW_PAINTER,
-						new DefaultPreviewPainter());
+						JTree tree = new JTree();
+						JScrollPane jsp = new JScrollPane(tree,
+								JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
+								JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
+						jsp.putClientProperty(
+								LafWidget.COMPONENT_PREVIEW_PAINTER,
+								new DefaultPreviewPainter());
 
-				disposableDialog.setLayout(new BorderLayout());
-				disposableDialog.add(jsp, BorderLayout.CENTER);
+						disposableDialog.setLayout(new BorderLayout());
+						disposableDialog.add(jsp, BorderLayout.CENTER);
 
-				disposableDialog.setSize(200, 100);
-				disposableDialog.setLocationRelativeTo(null);
-				disposableDialog
-						.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
-				disposableDialog.setVisible(true);
-			}
+						disposableDialog.setSize(200, 100);
+						disposableDialog.setLocationRelativeTo(null);
+						disposableDialog
+								.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
+						disposableDialog.setVisible(true);
+					}
+				});
+			};
 		});
 		builder.append("Disposable dialog", openDisposable);
 
 		JButton bd = new JButton("Open");
 		bd.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				SimpleDialog sd = new SimpleDialog();
-				sd.setModal(false);
-				sd.pack();
-				Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
-				// center the frame in the physical screen
-				sd.setLocation((d.width - sd.getWidth()) / 2, (d.height - sd
-						.getHeight()) / 2);
-				sd.setVisible(true);
-				simpleDialog = sd;
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						SimpleDialog sd = new SimpleDialog();
+						sd.setModal(false);
+						sd.pack();
+						Dimension d = Toolkit.getDefaultToolkit()
+								.getScreenSize();
+						// center the frame in the physical screen
+						sd.setLocation((d.width - sd.getWidth()) / 2,
+								(d.height - sd.getHeight()) / 2);
+						sd.setVisible(true);
+						simpleDialog = sd;
+					}
+				});
+			};
 		});
 		builder.append("Open a dialog", bd);
 
 		JButton bcd = new JButton("Close");
 		bcd.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				if (simpleDialog != null) {
-					simpleDialog.removeAll();
-					simpleDialog.dispose();
-					ReferenceQueue<JButton> weakQueue = new ReferenceQueue<JButton>();
-					WeakReference<JButton> weakRef = new WeakReference<JButton>(
-							simpleDialog.b1, weakQueue);
-					weakRef.enqueue();
-					simpleDialog.b1 = null;
-					simpleDialog = null;
-					System.gc();
-					// Wait until the weak reference is on the queue and remove
-					// it
-					Check.out("Waiting to remove");
-					try {
-						Reference<?> ref = weakQueue.remove();
-						ref.clear();
-					} catch (InterruptedException ie) {
-						ie.printStackTrace();
-						return;
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						if (simpleDialog != null) {
+							simpleDialog.removeAll();
+							simpleDialog.dispose();
+							ReferenceQueue<JButton> weakQueue = new ReferenceQueue<JButton>();
+							WeakReference<JButton> weakRef = new WeakReference<JButton>(
+									simpleDialog.b1, weakQueue);
+							weakRef.enqueue();
+							simpleDialog.b1 = null;
+							simpleDialog = null;
+							System.gc();
+							// Wait until the weak reference is on the queue and
+							// remove
+							// it
+							Check.out("Waiting to remove");
+							try {
+								Reference<?> ref = weakQueue.remove();
+								ref.clear();
+							} catch (InterruptedException ie) {
+								ie.printStackTrace();
+								return;
+							}
+							Check.out("Removed");
+						}
 					}
-					Check.out("Removed");
-				}
-			}
+				});
+			};
 		});
 		builder.append("Close the dialog", bcd);
 
@@ -738,114 +814,175 @@ public class ControlPanelFactory {
 		});
 		builder.append("Dialog with ESC close", buttonDialogCloseOnEsc);
 
+		JButton buttonDialogUndecorated = new JButton("Show");
+		buttonDialogCloseOnEsc.addActionListener(new ActionListener() {
+			public void actionPerformed(ActionEvent e) {
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						final JDialog dialog = new JDialog((Frame) null, "");
+						dialog.setSize(200, 200);
+						dialog.setUndecorated(true);
+						dialog.setLayout(new BorderLayout());
+
+						JPanel myContentPane = new JPanel();
+						myContentPane.setLayout(new BorderLayout());
+						dialog.setContentPane(myContentPane);
+
+						dialog.add(new JLabel("Press Esc to close dialog"),
+								BorderLayout.NORTH);
+
+						// connect "Esc" key with "System.exit(0)"
+						String actionName = "VK_ESCAPE";
+						Action action = new AbstractAction(actionName) {
+							public void actionPerformed(ActionEvent e) {
+								dialog.dispose();
+							}
+						};
+						myContentPane.getActionMap().put(actionName, action);
+						myContentPane.getInputMap(
+								JComponent.WHEN_IN_FOCUSED_WINDOW).put(
+								KeyStroke.getKeyStroke(
+										java.awt.event.KeyEvent.VK_ESCAPE, 0),
+								actionName);
+
+						dialog.setLocationRelativeTo(null);
+						dialog.setVisible(true);
+					}
+				});
+			}
+		});
+		builder.append("Undecorated dialog", buttonDialogUndecorated);
+
 		builder.appendSeparator("Miscellaneous");
 
 		JButton btf = new JButton("Show");
 		btf.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JFrame testFrame = new JFrame("test1");
-				testFrame.setSize(262, 100);
-				testFrame.setLocation(400, 400);
-				testFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
-				testFrame.setVisible(true);
-			}
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JFrame testFrame = new JFrame("test1");
+						testFrame.setSize(262, 100);
+						testFrame.setLocation(400, 400);
+						testFrame
+								.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
+						testFrame.setVisible(true);
+					}
+				});
+			};
 		});
 		builder.append("Regular frame", btf);
 
 		JButton btfU = new JButton("Show");
 		btfU.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JFrame.setDefaultLookAndFeelDecorated(false);
-				JDialog.setDefaultLookAndFeelDecorated(false);
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JFrame.setDefaultLookAndFeelDecorated(false);
+						JDialog.setDefaultLookAndFeelDecorated(false);
 
-				JFrame testFrame = new JFrame("test undecorated");
-				// testFrame.setUndecorated(true);
+						JFrame testFrame = new JFrame("test undecorated");
+						// testFrame.setUndecorated(true);
 
-				testFrame.setSize(262, 100);
-				testFrame.setLocation(400, 400);
-				testFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
-				testFrame.setVisible(true);
+						testFrame.setSize(262, 100);
+						testFrame.setLocation(400, 400);
+						testFrame
+								.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
+						testFrame.setVisible(true);
 
-				JFrame.setDefaultLookAndFeelDecorated(true);
-				JDialog.setDefaultLookAndFeelDecorated(true);
-			}
+						JFrame.setDefaultLookAndFeelDecorated(true);
+						JDialog.setDefaultLookAndFeelDecorated(true);
+					}
+				});
+			};
 		});
 		builder.append("Undecorated frame", btfU);
 
 		JButton bcp = new JButton("Open");
 		bcp.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
 
-				JFrame colorFrame = new JFrame();
-				final ColorPanel cp1 = new ColorPanel("default");
-				final ColorPanel cp2 = new ColorPanel("green");
-				cp2.setBackground(Color.green);
-				final ColorPanel cp3 = new ColorPanel("red");
-				cp3.setBackground(Color.red);
-				final ColorPanel cp4 = new ColorPanel("black");
-				cp4.setBackground(Color.black);
-				// cp2.setBackground(Color.GREEN);
-				colorFrame.setLayout(new LayoutManager() {
-					public void addLayoutComponent(String name, Component comp) {
+						JFrame colorFrame = new JFrame();
+						final ColorPanel cp1 = new ColorPanel("default");
+						final ColorPanel cp2 = new ColorPanel("green");
+						cp2.setBackground(Color.green);
+						final ColorPanel cp3 = new ColorPanel("red");
+						cp3.setBackground(Color.red);
+						final ColorPanel cp4 = new ColorPanel("black");
+						cp4.setBackground(Color.black);
+						// cp2.setBackground(Color.GREEN);
+						colorFrame.setLayout(new LayoutManager() {
+							public void addLayoutComponent(String name,
+									Component comp) {
+							}
+
+							public void layoutContainer(Container parent) {
+								int h = parent.getHeight() / 2;
+								int w = parent.getWidth() / 2;
+								cp1.setBounds(0, 0, w, h);
+								cp2.setBounds(0, h, w, parent.getHeight() - h);
+								cp3.setBounds(w, 0, w, h + 1);
+								cp4.setBounds(w, h + 1, w, parent.getHeight()
+										- h);
+							}
+
+							public Dimension minimumLayoutSize(Container parent) {
+								return preferredLayoutSize(parent);
+							}
+
+							public Dimension preferredLayoutSize(
+									Container parent) {
+								return new Dimension(100, 100);
+							}
+
+							public void removeLayoutComponent(Component comp) {
+							}
+
+						});
+						colorFrame.add(cp1);
+						colorFrame.add(cp2);
+						colorFrame.add(cp3);
+						colorFrame.add(cp4);
+
+						colorFrame.setSize(400, 399);
+						colorFrame.setLocation(300, 300);
+						colorFrame
+								.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
+						colorFrame.setVisible(true);
 					}
-
-					public void layoutContainer(Container parent) {
-						int h = parent.getHeight() / 2;
-						int w = parent.getWidth() / 2;
-						cp1.setBounds(0, 0, w, h);
-						cp2.setBounds(0, h, w, parent.getHeight() - h);
-						cp3.setBounds(w, 0, w, h + 1);
-						cp4.setBounds(w, h + 1, w, parent.getHeight() - h);
-					}
-
-					public Dimension minimumLayoutSize(Container parent) {
-						return preferredLayoutSize(parent);
-					}
-
-					public Dimension preferredLayoutSize(Container parent) {
-						return new Dimension(100, 100);
-					}
-
-					public void removeLayoutComponent(Component comp) {
-					}
-
 				});
-				colorFrame.add(cp1);
-				colorFrame.add(cp2);
-				colorFrame.add(cp3);
-				colorFrame.add(cp4);
-
-				colorFrame.setSize(400, 399);
-				colorFrame.setLocation(300, 300);
-				colorFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
-				colorFrame.setVisible(true);
-			}
+			};
 		});
 		builder.append("Color panels", bcp);
 
 		JButton paneDialog = new JButton("Open");
 		paneDialog.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				JDialog dialog = new JDialog(mainFrame, true);
-				dialog.setTitle("Test text pane in scroll pane");
-				JTextPane textPane = new JTextPane();
-				String contents = "";
-				for (int i = 0; i < 100; i++)
-					contents += "This is sample line " + i
-							+ " and a lot of other irrelevant text\n";
-				textPane.replaceSelection(contents);
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						JDialog dialog = new JDialog(mainFrame, true);
+						dialog.setTitle("Test text pane in scroll pane");
+						JTextPane textPane = new JTextPane();
+						String contents = "";
+						for (int i = 0; i < 100; i++)
+							contents += "This is sample line " + i
+									+ " and a lot of other irrelevant text\n";
+						textPane.replaceSelection(contents);
 
-				JScrollPane scroll = new JScrollPane(textPane);
-				JPanel panel = new JPanel();
-				panel.setLayout(new BorderLayout());
-				panel.add(scroll, BorderLayout.CENTER);
+						JScrollPane scroll = new JScrollPane(textPane);
+						JPanel panel = new JPanel();
+						panel.setLayout(new BorderLayout());
+						panel.add(scroll, BorderLayout.CENTER);
 
-				dialog.setLayout(new BorderLayout());
-				dialog.add(panel, BorderLayout.CENTER);
-				dialog.setSize(400, 400);
-				dialog.setLocation(400, 200);
-				dialog.setVisible(true);
-			}
+						dialog.setLayout(new BorderLayout());
+						dialog.add(panel, BorderLayout.CENTER);
+						dialog.setSize(400, 400);
+						dialog.setLocation(400, 200);
+						dialog.setVisible(true);
+					}
+				});
+			};
 		});
 		builder.append("Text pane dialog", paneDialog);
 
