@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -51,7 +51,6 @@ import org.jvnet.substance.painter.GradientPainterChangeListener;
 import org.jvnet.substance.skin.SkinChangeListener;
 import org.jvnet.substance.skin.SubstanceBusinessBlackSteelLookAndFeel;
 import org.jvnet.substance.theme.*;
-import org.jvnet.substance.title.TitlePainterChangeListener;
 import org.jvnet.substance.watermark.WatermarkChangeListener;
 
 import test.Check;
@@ -101,7 +100,7 @@ public class SampleFrame extends JFrame {
 	}
 
 	protected static class TraitChangeListener implements ThemeChangeListener,
-			WatermarkChangeListener, TitlePainterChangeListener,
+			WatermarkChangeListener, 
 			GradientPainterChangeListener, ButtonShaperChangeListener,
 			SkinChangeListener {
 		protected JTabbedPane jtp;
@@ -159,10 +158,9 @@ public class SampleFrame extends JFrame {
 		SubstanceLookAndFeel.registerButtonShaperChangeListener(tcl);
 		SubstanceLookAndFeel.registerGradientPainterChangeListener(tcl);
 		SubstanceLookAndFeel.registerSkinChangeListener(tcl);
-		SubstanceLookAndFeel.registerTitlePainterChangeListener(tcl);
 		SubstanceLookAndFeel.registerWatermarkChangeListener(tcl);
 
-		this.add(Check.getToolbar("", 16), BorderLayout.NORTH);
+		this.add(Check.getToolbar("", 16, false), BorderLayout.NORTH);
 		this.add(this.tabbed, BorderLayout.CENTER);
 		this.tabbed.putClientProperty(LafWidget.TABBED_PANE_PREVIEW_PAINTER,
 				new DefaultTabPreviewPainter());
@@ -287,9 +285,9 @@ public class SampleFrame extends JFrame {
 		this.tabbed.addTab("Regular", transPanel);
 
 		JPanel samplePanel = new JPanel(new BorderLayout());
-		FormLayout lm = new FormLayout("fill:default:grow(1), 4dlu,"
+		FormLayout lm = new FormLayout("fill:default:grow(1), 2dlu,"
 				+ "fill:default:grow(1)",
-				"pref, 0px, pref, 0px, pref, 0px, pref");
+				"pref, 0dlu, pref, 0dlu, pref, 0dlu, pref");
 		PanelBuilder builder = new PanelBuilder(lm);
 		builder.setBorder(new EmptyBorder(0, 2, 0, 2));
 		CellConstraints cc = new CellConstraints();
@@ -503,7 +501,7 @@ public class SampleFrame extends JFrame {
 			public void run() {
 				JFrame.setDefaultLookAndFeelDecorated(true);
 				SampleFrame sf = new SampleFrame(true);
-				sf.setSize(300, 225);
+				sf.setSize(315, 245);
 				sf.setLocationRelativeTo(null);
 				sf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
 				sf.setVisible(true);
