@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -827,6 +827,9 @@ public class SampleMenuFactory {
 								"org.jvnet.substance.skin.SubstanceBusinessBlueSteelLookAndFeel"));
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame, "Creme",
 				"org.jvnet.substance.skin.SubstanceCremeLookAndFeel"));
+		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame,
+				"Creme Coffee",
+				"org.jvnet.substance.skin.SubstanceCremeCoffeeLookAndFeel"));
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame, "Moderate",
 				"org.jvnet.substance.skin.SubstanceModerateLookAndFeel"));
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame, "Nebula",
@@ -845,6 +848,9 @@ public class SampleMenuFactory {
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame,
 				"Field of Wheat",
 				"org.jvnet.substance.skin.SubstanceFieldOfWheatLookAndFeel"));
+		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame,
+				"Finding Nemo",
+				"org.jvnet.substance.skin.SubstanceFindingNemoLookAndFeel"));
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame,
 				"Green Magic",
 				"org.jvnet.substance.skin.SubstanceGreenMagicLookAndFeel"));
@@ -887,7 +893,7 @@ public class SampleMenuFactory {
 		coreLafMenus.add(SubstanceLafChanger.getMenuItem(frame, "Motif",
 				"com.sun.java.swing.plaf.motif.MotifLookAndFeel"));
 		coreLafMenus.add(SubstanceLafChanger.getMenuItem(frame, "Nimbus",
-				"sun.swing.plaf.nimbus.NimbusLookAndFeel"));
+				"com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel"));
 
 		JMenu customLafMenus = new JMenu("Custom LAFs");
 		lafMenu.add(customLafMenus);
@@ -988,6 +994,8 @@ public class SampleMenuFactory {
 				"a03.swing.plaf.A03LookAndFeel"));
 		customLafMenus.add(SubstanceLafChanger.getMenuItem(frame, "Alloy",
 				"com.incors.plaf.alloy.AlloyLookAndFeel"));
+		customLafMenus.add(SubstanceLafChanger.getMenuItem(frame, "EaSynth",
+				"com.easynth.lookandfeel.EaSynthLookAndFeel"));
 		customLafMenus.add(SubstanceLafChanger.getMenuItem(frame, "FH",
 				"com.shfarr.ui.plaf.fh.FhLookAndFeel"));
 		customLafMenus.add(SubstanceLafChanger.getMenuItem(frame, "Hippo",
