@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -279,63 +279,21 @@ public class TablePanel extends ControllablePanel {
 		this.table.setDefaultRenderer(Color.class, new MyColorTableRenderer());
 		this.table.setDefaultRenderer(Float.class, new MyFloatTableRenderer());
 		final JScrollPane tableScrollpane = new JScrollPane(this.table);
+		tableScrollpane.setName("Main table in table panel");
 
 		JComboBox combo = new JComboBox(new Object[] { "aa", "bb", "cc" });
 		combo.setBorder(null);
 		this.table.getColumnModel().getColumn(1).setCellEditor(
 				new DefaultCellEditor(combo));
 
-		// JTable table = new JTable(
-		// new String[][]{{"cell11", "cell12", "cell13", "cell14"},
-		// {"cell21", "cell22", "cell23", "cell24"},
-		// {"cell31", "cell32", "cell33", "cell34"},
-		// {"cell41", "cell42", "cell43", "cell44"}},
-		// new String[]{"column1", "column2", "column3", "column4"});
-
 		this.table
 				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
-		// ListSelectionModel rowSM = this.table.getSelectionModel();
-		// rowSM.addListSelectionListener(new ListSelectionListener() {
-		// public void valueChanged(ListSelectionEvent e) {
-		// // Ignore extra messages.
-		// if (e.getValueIsAdjusting()) {
-		// return;
-		// }
-		//
-		// ListSelectionModel lsm = (ListSelectionModel) e.getSource();
-		// if (lsm.isSelectionEmpty()) {
-		// Check.out("No rows are selected.");
-		// } else {
-		// int selectedRow = lsm.getMinSelectionIndex();
-		// Check.out("Row " + selectedRow + " is now selected.");
-		// }
-		// }
-		// });
-		// We allow both row and column selection, which
-		// implies that we *really* want to allow individual
-		// cell selection.
+		// We allow row selection as the default
 		this.table.setCellSelectionEnabled(true);
-		this.table.setColumnSelectionAllowed(true);
-		// ListSelectionModel colSM = this.table.getColumnModel()
-		// .getSelectionModel();
-		// colSM.addListSelectionListener(new ListSelectionListener() {
-		// public void valueChanged(ListSelectionEvent e) {
-		// // Ignore extra messages.
-		// if (e.getValueIsAdjusting()) {
-		// return;
-		// }
-		//
-		// ListSelectionModel lsm = (ListSelectionModel) e.getSource();
-		// if (lsm.isSelectionEmpty()) {
-		// Check.out("No columns are selected.");
-		// } else {
-		// int selectedCol = lsm.getMinSelectionIndex();
-		// Check.out("Column " + selectedCol + " is now selected.");
-		// }
-		// }
-		// });
+		this.table.setRowSelectionAllowed(true);
+		this.table.setColumnSelectionAllowed(false);
 
-		this.table.setShowGrid(true);
+		this.table.setShowGrid(false);
 		this.table.setDragEnabled(false);
 		this.table
 				.setTableHeader(new JTableHeader(this.table.getColumnModel()));
@@ -361,9 +319,41 @@ public class TablePanel extends ControllablePanel {
 				}, -1, true);
 
 		FormLayout lm = new FormLayout("right:pref, 4dlu, fill:pref:grow", "");
-		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
+		DefaultFormBuilder builder = new DefaultFormBuilder(lm,
+				new ScrollablePanel());
 
 		builder.appendSeparator("Table settings");
+		final JCheckBox isEnabled = new JCheckBox("is enabled");
+		isEnabled.setSelected(table.isEnabled());
+		isEnabled.addActionListener(new ActionListener() {
+			public void actionPerformed(ActionEvent e) {
+				table.setEnabled(isEnabled.isSelected());
+				// the table header is not repainted on disabling / enabling :(
+				table.getTableHeader().repaint();
+			}
+		});
+		builder.append("Enabled", isEnabled);
+
+		JButton changeFirstColumn = new JButton("change 1st column");
+		changeFirstColumn.addActionListener(new ActionListener() {
+			public void actionPerformed(ActionEvent e) {
+				new Thread(new Runnable() {
+					public void run() {
+						for (int i = 0; i < table.getModel().getRowCount(); i++) {
+							table.getModel().setValueAt(
+									Thread.currentThread().getName() + " " + i,
+									i, 0);
+							try {
+								Thread.sleep(200);
+							} catch (InterruptedException e) {
+							}
+						}
+					}
+				}).start();
+			}
+		});
+		builder.append("Change values", changeFirstColumn);
+
 		final JSlider rowCountSlider = new JSlider(20, 1000, this.table
 				.getModel().getRowCount());
 		rowCountSlider.setPaintLabels(false);
@@ -376,12 +366,6 @@ public class TablePanel extends ControllablePanel {
 						.getValue()));
 			}
 		});
-		// final JCheckBox isBigTable = new JCheckBox("Is big table");
-		// isBigTable.addActionListener(new ActionListener() {
-		// public void actionPerformed(ActionEvent e) {
-		// table.setModel(new MyTableModel(isBigTable.isSelected() ? 1000: 20));
-		// }
-		// });
 		builder.append("Row count", rowCountSlider);
 
 		final JCheckBox areRowsSelectable = new JCheckBox("Rows selectable");
@@ -576,7 +560,7 @@ public class TablePanel extends ControllablePanel {
 		JButton tahoma12 = new JButton("Tahoma 12");
 		tahoma12.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				table.setFont(new Font("Arial", Font.PLAIN, 12));
+				table.setFont(new Font("Tahoma", Font.PLAIN, 12));
 			}
 		});
 		builder.append("Set font", tahoma12);
@@ -584,7 +568,7 @@ public class TablePanel extends ControllablePanel {
 		JButton tahoma13 = new JButton("Tahoma 13");
 		tahoma13.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
-				table.setFont(new Font("Arial", Font.PLAIN, 13));
+				table.setFont(new Font("Tahoma", Font.PLAIN, 13));
 			}
 		});
 		builder.append("Set font", tahoma13);
